within AICE.Blocks.Fluid.Compressible.Interfaces.Adaptors.BaseClasses;

partial model PartialAdaptorThermoFluid

  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  parameter AICE.Blocks.Types.Energy energy
  annotation(
    Dialog(tab="Advanced", group="Energy directions")
  );

  Modelica.Fluid.Interfaces.FluidPort_a port_a(
    redeclare replaceable package Medium = Medium
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

protected
  Medium.MassFlowRate m_flow_a, m_flow_b;
  Medium.AbsolutePressure p;
  Medium.SpecificEnthalpy h_a, h_b;
  Medium.MassFraction[Medium.nXi] Xi_a, Xi_b;
  Medium.Temperature T_a, T_b;
  Medium.ThermodynamicState state_a, state_b;
  Modelica.SIunits.HeatFlowRate dH;

equation

  0 = m_flow_a + m_flow_b;
  
  port_a.m_flow = m_flow_a; //m_flow_aが流入を正とした値　つまりSink用．従ってm_flow_bは流出を正とした値
  port_a.p = p;
  port_a.h_outflow = h_b;
  port_a.Xi_outflow = Xi_b;

  inStream(port_a.h_outflow) = h_a;
  inStream(port_a.Xi_outflow) = Xi_a;
  
  dH = (if m_flow_a > 0 then h_a * m_flow_b else h_b * m_flow_b);
  //_a が流入時に使う値，_bが流出時に使う値と思うが，逆にすると結果が正解になる　m_flow と合わせて持ち帰ることでdHは正に維持される
  state_a = Medium.setState_phX(port_a.p, h_a, Xi_a);
  state_b = Medium.setState_phX(port_a.p, h_b, Xi_b);
//隣接するポートからPhXをもらい、その他の状態量求める
  T_a = Medium.temperature(state_a);
  T_b = Medium.temperature(state_b);
  
end PartialAdaptorThermoFluid;