within AICE.Blocks.Fluid.Compressible.Sources.BaseClasses;

partial model FixedBoundaryParameters

  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  parameter Boolean use_p=true "select p or d"
    annotation (
      Evaluate = true,
      Dialog(group = "Boundary pressure or Boundary density")
    );

  parameter Medium.AbsolutePressure p=Medium.p_default "Boundary pressure"
    annotation (
      Dialog(group = "Boundary pressure or Boundary density", enable = use_p)
    );

  parameter Medium.Density d=(
    if use_T
    then Medium.density_pTX(
      Medium.p_default,Medium.T_default,Medium.X_default
    )
    else Medium.density_phX(
      Medium.p_default,Medium.h_default,Medium.X_default
    )
  ) "Boundary density"
    annotation (
      Dialog(group = "Boundary pressure or Boundary density", enable=not use_p)
    );

  parameter Boolean use_T=true "select T or h"
    annotation (
      Evaluate = true,
      Dialog(group = "Boundary temperature or Boundary specific enthalpy")
    );

  parameter Medium.Temperature T=Medium.T_default "Boundary temperature"
    annotation (
      Dialog(group = "Boundary temperature or Boundary specific enthalpy", enable = use_T)
    );

  parameter Medium.SpecificEnthalpy h=Medium.h_default "Boundary specific enthalpy"
    annotation (
      Dialog(group="Boundary temperature or Boundary specific enthalpy", enable = not use_T)
    );

  parameter Medium.MassFraction X[Medium.nX](quantity=Medium.substanceNames)=Medium.X_default
    "Boundary mass fractions m_i/m"
    annotation (
      Dialog(group = "Only for multi-substance flow", enable=Medium.nXi > 0)
    );

  // // temporary comment out parameter for "C"
  // parameter Medium.ExtraProperty C[Medium.nC](quantity=Medium.extraPropertiesNames)=fill(0, Medium.nC)
  //   "Boundary trace substances"
  //   annotation (
  //     Dialog(group = "Only for trace-substance flow", enable=Medium.nC > 0)
  //   );

end FixedBoundaryParameters;
