within AICE.Blocks.Fluid.Compressible.Vessels.BaseClasses;

partial model ClosedVolumeParameters_hot

  replaceable package Medium_hot = Modelica.Media.Interfaces.PartialMedium;

  outer Modelica.Fluid.System system_hot;

  parameter SI.Volume V_hot "Volume_hot";

  // Assumptions
  parameter Modelica.Fluid.Types.Dynamics energyDynamics_hot=system_hot.energyDynamics_hot
    "Formulation of energy balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions_hot", group="Dynamics"));

  parameter Modelica.Fluid.Types.Dynamics massDynamics_hot=system_hot.massDynamics_hot
    "Formulation of mass balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions_hot", group="Dynamics"));

  // Initialization
  parameter Medium_hot.AbsolutePressure p_hot_start = system_hot.p_hot_start "Start value of pressure"
  annotation(Dialog(tab = "Initialization_hot"));

  parameter Boolean use_T_hot_start = true "= true, use T_start, otherwise h_start"
  annotation(Dialog(tab = "Initialization_hot"), Evaluate=true);

  parameter Medium_hot.Temperature T_hot_start = (
    if use_T_hot_start then
      system_hot.T_start
    else
      Medium_hot.temperature_phX(p_hot_start,h_hot_start,X_hot_start)
  ) "Start value of temperature"
  annotation(Dialog(tab = "Initialization_hot", enable = use_T_start));

  parameter Medium_hot.SpecificEnthalpy h_hot_start = (
    if use_T_hot_start then
      Medium_hot.specificEnthalpy_pTX(p_hot_start, T_hot_start, X_hot_start)
    else
      Medium_hot.h_default
  ) "Start value of specific enthalpy"
  annotation(Dialog(tab = "Initialization_hot", enable = not use_T_hot_start));

  parameter Medium_hot.MassFraction X_hot_start[Medium_hot.nX] = Medium_hot.X_default
  "Start value of mass fractions m_i/m"
  annotation (
    Dialog(
      tab="Initialization_hot" // ,
      // enable=Medium_hot.nXi > 0
    )
  );

  // Port properties
  parameter Medium_hot.MassFlowRate m_hot_flow_nominal = (
    if system_hot.use_eps_Re_hot then
      system_hot.m_hot_flow_nominal
    else
      1e2*system_hot.m_hot_flow_small
  ) "Nominal value for mass flow rates in ports"
    annotation(Dialog(tab="Advanced_hot", group="Port properties"));

  parameter SI.MassFlowRate m_hot_flow_small(min=0) = (
    if system_hot.use_eps_Re_hot then
      system_hot.eps_m_hot_flow*m_hot_flow_nominal
    else
      system_hot.m_hot_flow_small
  ) "Regularization range at zero mass flow rate"
    annotation(Dialog(tab="Advanced_hot", group="Port properties"));

  parameter Boolean use_Re_hot = system_hot.use_eps_Re_hot
    "= true, if turbulent region is defined by Re, otherwise by m_flow_small"
    annotation(Dialog(tab="Advanced_hot", group="Port properties"), Evaluate=true);

end ClosedVolumeParameters_hot;
