within AICE.Blocks.Fluid.Incompressible.Interfaces.Adaptors.BaseClasses;

partial model PartialAdaptor

  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  parameter AICE.Blocks.Types.Energy energy
  annotation(
    Dialog(tab="Advanced", group="Energy directions")
  );

  Modelica.Fluid.Interfaces.FluidPort_a port_a(
    redeclare replaceable package Medium = Medium
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

protected

  Medium.MassFlowRate m_flow_a, m_flow_b;
  Medium.Density d;
  SI.VolumeFlowRate V_flow_b;
  Medium.AbsolutePressure p;
  Medium.SpecificEnthalpy h_a, h_b;
  Medium.MassFraction[Medium.nXi] Xi_a, Xi_b;

equation

  m_flow_b = d * V_flow_b;
  d = Modelica.Fluid.Utilities.regStep(
    m_flow_a,
    Medium.density_phX(p, h_a, Xi_a),
    Medium.density_phX(p, h_b, Xi_b)
  );

  0 = m_flow_a + m_flow_b;

  port_a.m_flow = m_flow_a;
  port_a.p = p;

  port_a.h_outflow = h_b;
  port_a.Xi_outflow = Xi_b;

  inStream(port_a.h_outflow) = h_a;
  inStream(port_a.Xi_outflow) = Xi_a;

end PartialAdaptor;
