within AICE.Blocks.Fluid.Incompressible.Valves.BaseClasses;

partial model ValveIncompressibleParameters

  outer Modelica.Fluid.System system;
  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  function valveCharacteristic = Modelica.Fluid.Valves.BaseClasses.ValveCharacteristics.linear;

  parameter Modelica.Fluid.Types.CvTypes CvData=Modelica.Fluid.Types.CvTypes.OpPoint
    "Selection of flow coefficient"
  annotation(Dialog(group = "Flow Coefficient"));

  parameter SI.Area Av(
    fixed=if CvData == Modelica.Fluid.Types.CvTypes.Av then true else false,
    start=m_flow_nominal/(sqrt(rho_nominal*dp_nominal))*valveCharacteristic(
        opening_nominal)
  ) "Av (metric) flow coefficient"
  annotation(
    Dialog(
      group = "Flow Coefficient"  //,
      // enable = (CvData==Modelica.Fluid.Types.CvTypes.Av)
    )
  );

  parameter Real Kv = 0 "Kv (metric) flow coefficient [m3/h]"
  annotation(
    Dialog(
      group = "Flow Coefficient"  //,
      // enable = (CvData==Modelica.Fluid.Types.CvTypes.Kv)
    )
  );

  parameter Real Cv = 0 "Cv (US) flow coefficient [USG/min]"
  annotation(
    Dialog(
      group = "Flow Coefficient"  //,
      // enable = (CvData==Modelica.Fluid.Types.CvTypes.Cv)
    )
  );

  parameter SI.Pressure dp_nominal "Nominal pressure drop"
  annotation(Dialog(group="Nominal operating point"));

  parameter Medium.MassFlowRate m_flow_nominal "Nominal mass flowrate"
  annotation(Dialog(group="Nominal operating point"));

  parameter Medium.Density rho_nominal=Medium.density_pTX(Medium.p_default, Medium.T_default, Medium.X_default)
    "Nominal inlet density"
  annotation(
    Dialog(
      group="Nominal operating point"  //,
      // enable = (CvData==Modelica.Fluid.Types.CvTypes.OpPoint)
    )
  );

  parameter Real opening_nominal(min=0,max=1)=1 "Nominal opening"
  annotation(
    Dialog(
      group="Nominal operating point"  //,
      // enable = (CvData==Modelica.Fluid.Types.CvTypes.OpPoint)
    )
  );

  parameter Boolean filteredOpening=false
    "= true, if opening is filtered with a 2nd order CriticalDamping filter"
    annotation(Dialog(group="Filtered opening"),choices(checkBox=true));

  parameter Modelica.SIunits.Time riseTime=1
    "Rise time of the filter (time to reach 99.6 % of an opening step)"
    annotation(Dialog(group="Filtered opening",enable=filteredOpening));

  parameter Real leakageOpening(min=0,max=1)=1e-3
    "The opening signal is limited by leakageOpening (to improve the numerics)"
    annotation(Dialog(group="Filtered opening",enable=filteredOpening));

  // Assumption
  parameter Boolean allowFlowReversal = system.allowFlowReversal
    "= true to allow flow reversal, false restricts to design direction (port_a -> port_b)"
    annotation(Dialog(tab="Assumptions"), Evaluate=true);

  parameter Boolean checkValve=false "Reverse flow stopped"
    annotation(Dialog(tab="Assumptions"));

  // Advanced
  parameter Boolean use_Re = system.use_eps_Re
    "= true, if turbulent region is defined by Re, otherwise by m_flow_small"
    annotation(Dialog(tab="Advanced"), Evaluate=true);

  // Note: value of dp_start shall be refined by derived model, basing on local dp_nominal
  parameter Medium.AbsolutePressure dp_start(min=-Modelica.Constants.inf) = dp_nominal
      "Guess value of dp = port_a.p - port_b.p"
    annotation(Dialog(tab = "Advanced"));

  parameter Medium.MassFlowRate m_flow_start = m_flow_nominal
      "Guess value of m_flow = port_a.m_flow"
    annotation(Dialog(tab = "Advanced"));

  // Note: value of m_flow_small shall be refined by derived model, basing on local m_flow_nominal
  parameter Medium.MassFlowRate m_flow_small = (
    if system.use_eps_Re then
      system.eps_m_flow*m_flow_nominal
    else
      system.m_flow_small
  ) "Small mass flow rate for regularization of zero flow"
    annotation(Dialog(tab = "Advanced"));

  // Diagnostics
  parameter Boolean show_T = true
      "= true, if temperatures at port_a and port_b are computed"
    annotation(Dialog(tab="Advanced",group="Diagnostics"));

  parameter Boolean show_V_flow = true
      "= true, if volume flow rate at inflowing port is computed"
    annotation(Dialog(tab="Advanced",group="Diagnostics"));

end ValveIncompressibleParameters;
