within AICE.Blocks.Fluid.Incompressible.Vessels.BaseClasses;

partial model OpenTankParameters

  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  outer Modelica.Fluid.System system;

  parameter SI.Height height "Height of tank";
  parameter SI.Area crossArea "Area of tank";

  // Assumptions
  parameter Modelica.Fluid.Types.Dynamics energyDynamics=system.energyDynamics
    "Formulation of energy balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));

  parameter Modelica.Fluid.Types.Dynamics massDynamics=system.massDynamics
    "Formulation of mass balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));

  // Initialization
  parameter Medium.AbsolutePressure p_start = system.p_ambient "Start value of pressure"
  annotation(Dialog(tab = "Initialization"));

  parameter Boolean use_T_start = true "= true, use T_start, otherwise h_start"
  annotation(Dialog(tab = "Initialization"), Evaluate=true);

  parameter Medium.Temperature T_start = (
    if use_T_start then
      system.T_start
    else
      Medium.temperature_phX(p_start,h_start,X_start)
  ) "Start value of temperature"
  annotation(Dialog(tab = "Initialization", enable = use_T_start));

  parameter Medium.SpecificEnthalpy h_start = (
    if use_T_start then
      Medium.specificEnthalpy_pTX(p_start, T_start, X_start)
    else
      Medium.h_default
  ) "Start value of specific enthalpy"
  annotation(Dialog(tab = "Initialization", enable = not use_T_start));

  parameter Medium.MassFraction X_start[Medium.nX] = Medium.X_default
  "Start value of mass fractions m_i/m"
  annotation (
    Dialog(
      tab="Initialization" // ,
      // enable=Medium.nXi > 0
    )
  );

  // Port properties
  parameter Medium.MassFlowRate m_flow_nominal = (
    if system.use_eps_Re then
      system.m_flow_nominal
    else
      1e2*system.m_flow_small
  ) "Nominal value for mass flow rates in ports"
    annotation(Dialog(tab="Advanced", group="Port properties"));

  parameter SI.MassFlowRate m_flow_small(min=0) = (
    if system.use_eps_Re then
      system.eps_m_flow*m_flow_nominal
    else
      system.m_flow_small
  ) "Regularization range at zero mass flow rate"
    annotation(Dialog(tab="Advanced", group="Port properties"));

  parameter Boolean use_Re = system.use_eps_Re
    "= true, if turbulent region is defined by Re, otherwise by m_flow_small"
    annotation(Dialog(tab="Advanced", group="Port properties"), Evaluate=true);

end OpenTankParameters;