within AICE.Blocks.Fluid.Incompressible.Vessels.BaseClasses;

block PartialLumpedVesselOneport_AOTI
  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  extends Interfaces.FluidPort_AOTI_a;
  
  parameter AICE.Blocks.Types.Energy energy_port_a = AICE.Blocks.Types.Energy.Sink
  annotation(
    Dialog(tab="Advanced", group="Energy directions")
  );

  AICE.Blocks.Fluid.Incompressible.Interfaces.Adaptors.Adaptor_AOTI adaptor_a(
    redeclare replaceable package Medium = Medium,
    energy = energy_port_a
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 0}, extent = {{10, -20}, {-10, 20}}, rotation = 0)
    )
  );

  replaceable Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel acausalModel(
  redeclare replaceable package Medium = Medium,
  nPorts = 1,
  use_HeatTransfer = false,
  use_portsData = false
) annotation(
  Placement(
    visible = true,
    transformation(origin = {0, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
  )
);


equation

  connect(acausalModel.ports[1], adaptor_a.port_a) annotation(
    Line(points = {{0, +20}, {0,0}, {-40, 0}}, color = {0, 127, 255})
  );

  connect(adaptor_a.absolutePressure_b, absolutePressure_a) annotation(
    Line(points = {{-60, +12}, {-72, +12}, {-72, +60}, {-100, +60}}, color = {0, 0, 127})
  );
  connect(adaptor_a.specificEnthalpy_mix_b, specificEnthalpy_mix_a) annotation(
    Line(points = {{-60,  +8}, {-76,  +8}, {-76, +40}, {-100, +40}}, color = {0, 0, 127})
  );
  connect(adaptor_a.massFractions_mix_b[:], massFractions_mix_a[:]) annotation(
    Line(points = {{-60,  +4}, {-80,  +4}, {-80, +20}, {-100, +20}}, color = {0, 0, 127}, thickness = 0.5)
  );
  connect(adaptor_a.massFractions_outflow_b[:], massFractions_outflow_a[:]) annotation(
    Line(points = {{-60,  -4}, {-80,  -4}, {-80, -20}, {-100, -20}}, color = {0, 0, 127}, thickness = 0.5)
  );
  connect(adaptor_a.specificEnthalpy_outflow_b, specificEnthalpy_outflow_a) annotation(
    Line(points = {{-60,  -8}, {-76,  -8}, {-76, -40}, {-100, -40}}, color = {0, 0, 127})
  );
  connect(adaptor_a.volumeFlowRate_b, volumeFlowRate_a) annotation(
    Line(points = {{-60, -12}, {-72, -12}, {-72, -60}, {-100, -60}}, color = {0, 0, 127})
  );

annotation(
    experiment(StartTime = 0, StopTime = 1, Tolerance = 1e-6, Interval = 0.002),
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl"));
end PartialLumpedVesselOneport_AOTI;