within AICE.Blocks.Mechanics.Rotational.Components.BaseClasses;
model FMEPB "Chen Frynn Friction model for MVEM"
  extends Modelica.Mechanics.Rotational.Interfaces.PartialTwoFlanges;
  extends Icons.Rotational.FMEPB;
  parameter Modelica.SIunits.Torque C0_Nm= 6.42347848;
  parameter Real C1_NmpRPM = 6.88671e-3;
  parameter Real C2_NmpRPM2 = -3.23252e-7;
  parameter Real C3_NmpPa = 1E-7;

  Modelica.SIunits.Torque tau_FRIC;
  Real omega_RPM, w;
  discrete SI.Pressure p_cyl_max_now, p_cyl_max_pre;
  discrete Real n_crank_pre, n_crank_now;
  //Real n_crank_now;

  input SI.Pressure p_cyl_fmep;
    
algorithm
  //p_cyl_fmepはMultiCylinderEngineでPcylと結合されるので、（ここでは無定義に見えるが信じて）使って良い。
  n_crank_pre := delay(n_crank_now, 1.0E-8);
  n_crank_now := floor((1+flange_a.phi) / Modelica.Constants.pi / 4);
  
  // 2周回る度にp_cyl_maxをリセット リセット前にp_cyl_max_preに値を渡し新しいサイクルのFMEP計算に使う
  if n_crank_now - n_crank_pre > 0.5 then
    p_cyl_max_pre := delay(p_cyl_max_now, 1.0E-8);
    p_cyl_max_now := 0.0;
  else
    p_cyl_max_pre := delay(p_cyl_max_pre, 1.0E-8);
    p_cyl_max_now := max(p_cyl_fmep, delay(p_cyl_max_now, 1.0E-8));
    
  end if;

equation

  w = der(flange_a.phi);
  omega_RPM = w * 30.0 / Modelica.Constants.pi ;
  tau_FRIC = C0_Nm * (if (omega_RPM < -10) then -1 elseif (omega_RPM <10) then omega_RPM/10 else 1) + ( C1_NmpRPM  + C2_NmpRPM2 * abs(omega_RPM) ) * omega_RPM +    C3_NmpPa * p_cyl_max_pre ;

  flange_a.phi = flange_b.phi;
  flange_a.tau + flange_b.tau - tau_FRIC = 0.0;

end FMEPB;