within AICE.Examples.METI_2020.Components.BaseClasses;

partial model MultiCylinderEngineParameters
  constant Integer nCylinders = 4;
  // # geometry parameters
  // ## total displacement volume
  parameter CarUnits.Volume V_displacement "Total displacement volume" annotation(
    Dialog(group = "Geometries"));
  // ## Compression ratio (CR)
  parameter CarUnits.CompressionRatio compressionRatio "Compression ratio V_BDC/V_TDC" annotation(
    Dialog(group = "Geometries"));
  // ## intake pipe (post-throttle)
  parameter CarUnits.Volume V_intakeManifold "Volume of intake manifold" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter d_valve_intake "Diameter of intake valve" annotation(
    Dialog(group = "Geometries"));
  // ## exhaust pipe
  parameter CarUnits.Diameter d_valve_exhaust "Diameter of exhaust valve" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Volume V_exhaustManifold "Volume of exhaust manifold" annotation(
    Dialog(group = "Geometries"));
  // ## crank & chamber
  parameter CarUnits.Length l_connRod "Length of connecting rod" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Position offset_crank "Crank offset from piston axis" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter bore "Diameter of cylinder" annotation(
    Dialog(group = "Geometries"));
  // # start value parameters
  // ## intake pipe (post-throttle)
  parameter SI.AbsolutePressure p_start_intakeManifold "Start value of pressure at intake manifold" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_intakeManifold "Start value of temperature at intake manifold" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## chambers
  parameter SI.AbsolutePressure p_start_chamber1 "Start value of pressure at 1st chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber1 "Start value of temperature at 1st chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber2 "Start value of pressure at 2nd chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber2 "Start value of temperature at 2nd chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber3 "Start value of pressure at 3rd chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber3 "Start value of temperature at 3rd chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber4 "Start value of pressure at 4th chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber4 "Start value of temperature at 4th chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## exhaust pipe
  parameter SI.AbsolutePressure p_start_exhaustManifold "Start value of pressure at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_exhaustManifold "Start value of temperature at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // # table data path parameters
  parameter String fileName_lift_intake "Table file name of the relationship between cam angle and lift at intake valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which intake valve lift table is present")));
  parameter String fileName_CD_intake "Table file name of the relationship between lift and CD (discharge coefficient) at intake valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which intake valve CD table is present")));
  parameter String fileName_lift_exhaust "Table file name of the relationship between cam angle and lift at exhaust valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which exhaust valve lift table is present")));
  parameter String fileName_CD_exhaust "Table file name of the relationship between lift and CD (discharge coefficient) at exhaust valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which exhaust valve CD table is present")));
  // # other parameters
  parameter CarUnits.AirByFuelRatio airByFuel "Ratio of air/fuel for injection amount calculation";
  parameter SI.Angle injectionStart "ATDC from piston compression TDC";
  parameter SI.Angle injectionPeriod "Angle between start and stop of injection";
  parameter Real m_wiebe;
  parameter SI.Angle phi_combustion_begin "BTDC from piston compression TDC";
  parameter SI.Angle phi_combustion_period "Anglar period until 1-exp(-a_wiebe) of fuel burns";
  parameter SI.Angle exhaustValveCentor "ATDC from exhaust";
  parameter SI.Angle intakeValveCentor "ATDC from exhaust";
  parameter Real c1_woschni_intakeExhaust;
  parameter Real c1_woschni_compressionPower;
  parameter Real c2_woschni;
end MultiCylinderEngineParameters;