within AICE.Examples.METI_2020.Components.BaseClasses;

partial model SampleEngineUnitParameters
  constant Integer nCylinders = 4;
  // # boundary condition parameters
  // ## intake boundary
  parameter SI.AbsolutePressure p_intake = 1.0e5 "Average of intake air pressure" annotation(
    Dialog(tab = "Boundaries", group = "Pressures"));
  parameter SI.Temperature T_intake = 298.15 "Average of intake air temperature" annotation(
    Dialog(tab = "Boundaries", group = "Temperatures"));
  // ## exhaust boundary
  parameter SI.AbsolutePressure p_exhaust = 1.0e5 "Average of exhaust gas pressure" annotation(
    Dialog(tab = "Boundaries", group = "Pressures"));
  parameter SI.Temperature T_exhaust = 298.15 "Average of exhaust gas temperature" annotation(
    Dialog(tab = "Boundaries", group = "Temperatures"));
  // # geometry parameters
  // ## total displacement volume
  parameter CarUnits.Volume V_displacement = 0.002 "Total displacement volume" annotation(
    Dialog(group = "Geometries"));
  // ## Compression ratio (CR)
  parameter CarUnits.CompressionRatio compressionRatio = 12.0 "Compression ratio V_BDC/V_TDC" annotation(
    Dialog(group = "Geometries"));
  // ## intake pipe (pre-throttle)
  parameter CarUnits.Volume V_intakePipe = 1.e-3 "Volume of intake pipe" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter d_intakePipe = 0.1 "Diameter of intake pipe" annotation(
    Dialog(group = "Geometries"));
  // ## throttle
  parameter CarUnits.Diameter d_throttle = 0.1 "Diameter of throttle" annotation(
    Dialog(group = "Geometries"));
  // ## intake pipe (post-throttle)
  parameter CarUnits.Volume V_intakeManifold = 1.e-3 "Volume of intake manifold" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter d_valve_intake = 0.030 "Diameter of intake valve" annotation(
    Dialog(group = "Geometries"));
  // ## exhaust pipe
  parameter CarUnits.Diameter d_valve_exhaust = 0.035 "Diameter of exhaust valve" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Volume V_exhaustManifold = 2.e-3 "Volume of exhaust manifold" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter d_exhaustPipe = 0.04 "Diameter of exhaust pipe" annotation(
    Dialog(group = "Geometries"));
  // ## crank & chamber
  parameter CarUnits.Length l_connRod = 0.138 "Length of connecting rod" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Position offset_crank = 0 "Crank offset from piston axis" annotation(
    Dialog(group = "Geometries"));
  parameter CarUnits.Diameter bore = 0.086 "Diameter of cylinder" annotation(
    Dialog(group = "Geometries"));
  // # start value parameters
  // ## intake pipe (pre-throttle)
  parameter SI.AbsolutePressure p_start_intakePipe = 0.99e5 "Start value of pressure at intake pipe" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_intakePipe = 298.15 "Start value of temperature at intake pipe" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## intake pipe (post-throttle)
  parameter SI.AbsolutePressure p_start_intakeManifold = 1.5e5 "Start value of pressure at intake manifold" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_intakeManifold = 398 "Start value of temperature at intake manifold" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## chambers
  parameter SI.AbsolutePressure p_start_chamber1 = 20e5 "Start value of pressure at 1st chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber1 = 1262. "Start value of temperature at 1st chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber2 = 4.0e5 "Start value of pressure at 2nd chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber2 = 957.4 "Start value of temperature at 2nd chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber3 = 1.8e5 "Start value of pressure at 3rd chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber3 = 433.0 "Start value of temperature at 3rd chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  parameter SI.AbsolutePressure p_start_chamber4 = 2.735e5 "Start value of pressure at 4th chamber" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_chamber4 = 959.8 "Start value of temperature at 4th chamber" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## exhaust manifold
  parameter SI.AbsolutePressure p_start_exhaustManifold = 2.8e5 "Start value of pressure at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_exhaustManifold = 398 "Start value of temperature at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));
  // ## exhaust pipe
  parameter SI.AbsolutePressure p_start_exhaustPipe = 1.01e5 "Start value of pressure at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Pressures"));
  parameter SI.Temperature T_start_exhaustPipe = 398 "Start value of temperature at exhaust manifold" annotation(
    Dialog(tab = "Initialization", group = "Temperatures"));

  // # table data path parameters
  parameter String fileName_lift_intake = Files.loadResource("modelica://AICE/Resources/TableData/Valve/liftFromAngle_sample.txt") "Table file name of the relationship between cam angle and lift at intake valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which intake valve lift table is present")));
  parameter String fileName_CD_intake = Files.loadResource("modelica://AICE/Resources/TableData/Valve/CDFromLift_sample.txt") "Table file name of the relationship between lift and CD (discharge coefficient) at intake valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which intake valve CD table is present")));
  parameter String fileName_lift_exhaust = Files.loadResource("modelica://AICE/Resources/TableData/Valve/liftFromAngle_sample.txt") "Table file name of the relationship between cam angle and lift at exhaust valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which exhaust valve lift table is present")));
  parameter String fileName_CD_exhaust = Files.loadResource("modelica://AICE/Resources/TableData/Valve/CDFromLift_sample.txt") "Table file name of the relationship between lift and CD (discharge coefficient) at exhaust valve" annotation(
    Dialog(group = "Table data definition", loadSelector(filter = "Text files (*.txt);;MATLAB MAT-files (*.mat)", caption = "Open file in which exhaust valve CD table is present")));
  // # other parameters
  parameter CarUnits.AirByFuelRatio airByFuel = 15.0 "Ratio of air/fuel for injection amount calculation";
  parameter SI.Angle injectionStart = -1.22173 "ATDC from piston compression TDC";
  parameter SI.Angle injectionPeriod = 0.031415926535897934 "Angle between start and stop of injection";
  parameter Real m_wiebe = 1;
  parameter SI.Angle phi_combustion_begin = 0.0 "BTDC from piston compression TDC";//0.0698132 
  parameter SI.Angle phi_combustion_period = 1.05 "Anglar period until 1-exp(-a_wiebe) of fuel burns";
  parameter SI.Angle exhaustValveCentor = -2.181 "ATDC from exhaust"; // -2.35619
  parameter SI.Angle intakeValveCentor = 2.181 "ATDC from exhaust"; //+2.35619
  parameter Real c1_woschni_intakeExhaust = 6.18;
  parameter Real c1_woschni_compressionPower = 2.28;
  parameter Real c2_woschni = 0.00324;
  
  //turbocharger
  parameter SI.Area Av_wastegate = 3e-6 "wastegate sectional area";
  parameter SI.Inertia Inertia_tc = 1e-4 "turbocharger rotor assy inertia";
  parameter SI.AngularVelocity w_tc_init = 1.5e4 "turbocharger initial speed";  
end SampleEngineUnitParameters;