within AICE.Examples.METI_2020.Components;

block Driver
  parameter Real A; // linear weight for constant speed driving, should be set appropriate value at 200km/h cruising
  parameter Real B; // power for constant speed driving, should be adjusted for 100km/h cruising
  parameter Real C; // linear weight for acceleration. should be set appropriate value at 200km/h-2G acceleration
  parameter Real D; // power for acceleration
  parameter Real E; // linear gain of brake
  
  AICE.Blocks.Interfaces.RealInput VehicleSpeed_target_kph annotation(
    Placement(visible = true, transformation(origin = {-110, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-110, 90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Interfaces.RealInput VehicleSpeed_actual_kph annotation(
    Placement(visible = true, transformation(origin = {-110, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-110, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Interfaces.RealOutput AccelPedal_pct annotation(
    Placement(visible = true, transformation(origin = {110, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {110, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Interfaces.RealOutput BrakePedal_pct annotation(
    Placement(visible = true, transformation(origin = {110, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {110, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Add G_long_target(k1 = 1 / 3.6 * 9.81, k2 = -1 / 3.6 * 9.81)  annotation(
    Placement(visible = true, transformation(origin = {-78, 56}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Add add_steady_accel(k1 = A, k2 = C)  annotation(
    Placement(visible = true, transformation(origin = {56, 56}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Product product_v_a annotation(
    Placement(visible = true, transformation(origin = {-24, 48}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain normalizer_v_veh(k = 1 / 200)  annotation(
    Placement(visible = true, transformation(origin = {-81, 89}, extent = {{-7, -7}, {7, 7}}, rotation = 0)));
  Modelica.Blocks.Math.Gain normalizer_v_ac(k = 1 / 2)  annotation(
    Placement(visible = true, transformation(origin = {-54, 42}, extent = {{-6, -6}, {6, 6}}, rotation = 0)));
  AICE.Math.Power power_v_a(power = D)  annotation(
    Placement(visible = true, transformation(origin = {12, 48}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Math.Power power_v(power = B)  annotation(
    Placement(visible = true, transformation(origin = {8, 86}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain gain_brk(k = (-1) * E)  annotation(
    Placement(visible = true, transformation(origin = {-56, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Tables.CombiTable1Ds transitionfactor(table = [-6, 1; -5, 1; -1, 0;0, 0])  annotation(
    Placement(visible = true, transformation(origin = {-36, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Product product_brktransition annotation(
    Placement(visible = true, transformation(origin = {10, -54}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Tables.CombiTable1Ds StationalBrake(table = [0, 1; 0.1, 1; 0.5, 0; 1, 0]) annotation(
    Placement(visible = true, transformation(origin = {-56, -86}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain gain_brk_stational(k =  E) annotation(
    Placement(visible = true, transformation(origin = {-20, -86}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Add add_cruise_station_brake(k1 = 1, k2 = 1) annotation(
    Placement(visible = true, transformation(origin = {44, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Abs abs1 annotation(
    Placement(visible = true, transformation(origin = {-6, 48}, extent = {{-4, -4}, {4, 4}}, rotation = 0)));
  Modelica.Blocks.Math.Sign sign1 annotation(
    Placement(visible = true, transformation(origin = {-6, 28}, extent = {{-4, -4}, {4, 4}}, rotation = 0)));
  Modelica.Blocks.Math.Product product annotation(
    Placement(visible = true, transformation(origin = { 34, 46}, extent = {{-4, -4}, {4, 4}}, rotation = 0)));
  Modelica.Blocks.Math.Max AP_max annotation(
    Placement(visible = true, transformation(origin = {82, 92}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant const0(k = 0)  annotation(
    Placement(visible = true, transformation(origin = {48, 98}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant constant1(k = 0)  annotation(
    Placement(visible = true, transformation(origin = {-41, 79}, extent = {{-5, -5}, {5, 5}}, rotation = 0)));
  Modelica.Blocks.Math.Max max annotation(
    Placement(visible = true, transformation(origin = {-18, 86}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Min AP_min annotation(
    Placement(visible = true, transformation(origin = { 82, 128}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant const100(k = 100) annotation(
    Placement(visible = true, transformation(origin = {48, 134}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant constant2(k = 0) annotation(
    Placement(visible = true, transformation(origin = {44, -86}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Max BP_max annotation(
    Placement(visible = true, transformation(origin = {76, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant constant3(k = 100) annotation(
    Placement(visible = true, transformation(origin = {52, -26}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Min BP_min annotation(
    Placement(visible = true, transformation(origin = {80, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Add creepCorrection(k1 = 1, k2 = 1) annotation(
    Placement(visible = true, transformation(origin = {84, 56}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant constant4(k = -5) annotation(
    Placement(visible = true, transformation(origin = {56, 26}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(G_long_target.u1, VehicleSpeed_target_kph) annotation(
    Line(points = {{-90, 62}, {-104, 62}, {-104, 90}, {-110, 90}}, color = {0, 0, 127}));
  connect(G_long_target.u2, VehicleSpeed_actual_kph) annotation(
    Line(points = {{-90, 50}, {-110, 50}}, color = {0, 0, 127}));
  connect(VehicleSpeed_target_kph, normalizer_v_veh.u) annotation(
    Line(points = {{-110, 90}, {-101, 90}, {-101, 89}, {-89, 89}}, color = {0, 0, 127}));
  connect(normalizer_v_veh.y, product_v_a.u1) annotation(
    Line(points = {{-73, 89}, {-55, 89}, {-55, 54}, {-36, 54}}, color = {0, 0, 127}));
  connect(normalizer_v_ac.u, G_long_target.y) annotation(
    Line(points = {{-61, 42}, {-67, 42}, {-67, 56}}, color = {0, 0, 127}));
  connect(normalizer_v_ac.y, product_v_a.u2) annotation(
    Line(points = {{-47, 42}, {-36, 42}}, color = {0, 0, 127}));
  connect(power_v.y, add_steady_accel.u1) annotation(
    Line(points = {{18, 86}, {27, 86}, {27, 62}, {44, 62}}, color = {0, 0, 127}));
  connect(gain_brk.u, normalizer_v_ac.y) annotation(
    Line(points = {{-68, -60}, {-72, -60}, {-72, 14}, {-42, 14}, {-42, 42}, {-48, 42}, {-48, 42}}, color = {0, 0, 127}));
  connect(transitionfactor.y[1], product_brktransition.u1) annotation(
    Line(points = {{-25, -30}, {-25, -48}, {-2, -48}}, color = {0, 0, 127}));
  connect(product_brktransition.u2, gain_brk.y) annotation(
    Line(points = {{-2, -60}, {-44, -60}}, color = {0, 0, 127}));
  connect(StationalBrake.u, VehicleSpeed_target_kph) annotation(
    Line(points = {{-68, -86}, {-96, -86}, {-96, 90}, {-110, 90}}, color = {0, 0, 127}));
  connect(gain_brk_stational.u, StationalBrake.y[1]) annotation(
    Line(points = {{-32, -86}, {-45, -86}}, color = {0, 0, 127}));
  connect(product_brktransition.y, add_cruise_station_brake.u1) annotation(
    Line(points = {{22, -54}, {32, -54}, {32, -54}, {32, -54}}, color = {0, 0, 127}));
  connect(add_cruise_station_brake.u2, gain_brk_stational.y) annotation(
    Line(points = {{32, -66}, {26, -66}, {26, -86}, {-8, -86}, {-8, -86}}, color = {0, 0, 127}));
  connect(product_v_a.y, abs1.u) annotation(
    Line(points = {{-12, 48}, {-11, 48}}, color = {0, 0, 127}));
  connect(abs1.y, power_v_a.x) annotation(
    Line(points = {{-2, 48}, {2, 48}}, color = {0, 0, 127}));
  connect(sign1.u, product_v_a.y) annotation(
    Line(points = {{-11, 28}, {-14, 28}, {-14, 48}, {-12, 48}}, color = {0, 0, 127}));
  connect(product.u1, power_v_a.y) annotation(
    Line(points = {{30, 48}, {22, 48}, {22, 48}, {22, 48}}, color = {0, 0, 127}));
  connect(product.u2, sign1.y) annotation(
    Line(points = {{30, 44}, {28, 44}, {28, 28}, {-2, 28}, {-2, 28}}, color = {0, 0, 127}));
  connect(product.y, add_steady_accel.u2) annotation(
    Line(points = {{38, 46}, {44, 46}, {44, 50}}, color = {0, 0, 127}));
  connect(max.u1, normalizer_v_veh.y) annotation(
    Line(points = {{-30, 92}, {-54, 92}, {-54, 88}, {-74, 88}, {-74, 90}}, color = {0, 0, 127}));
  connect(max.u2, constant1.y) annotation(
    Line(points = {{-30, 80}, {-36, 80}, {-36, 80}, {-36, 80}}, color = {0, 0, 127}));
  connect(max.y, power_v.x) annotation(
    Line(points = {{-6, 86}, {-4, 86}, {-4, 86}, {-2, 86}}, color = {0, 0, 127}));
  connect(AP_max.y, AP_min.u2) annotation(
    Line(points = {{93, 92}, {95.5, 92}, {95.5, 110}, {70, 110}, {70, 122}}, color = {0, 0, 127}));
  connect(AP_min.y, AccelPedal_pct) annotation(
    Line(points = {{93, 128}, {104, 128}, {104, 60}, {110, 60}}, color = {0, 0, 127}));
  connect(AP_min.u1, const100.y) annotation(
    Line(points = {{70, 134}, {59, 134}}, color = {0, 0, 127}));
  connect(constant2.y, BP_max.u2) annotation(
    Line(points = {{55, -86}, {64, -86}}, color = {0, 0, 127}));
  connect(add_cruise_station_brake.y, BP_max.u1) annotation(
    Line(points = {{56, -60}, {58, -60}, {58, -74}, {64, -74}, {64, -74}}, color = {0, 0, 127}));
  connect(BP_max.y, BP_min.u2) annotation(
    Line(points = {{88, -80}, {90, -80}, {90, -64}, {68, -64}, {68, -56}, {68, -56}}, color = {0, 0, 127}));
  connect(constant3.y, BP_min.u1) annotation(
    Line(points = {{64, -26}, {68, -26}, {68, -44}, {68, -44}}, color = {0, 0, 127}));
  connect(BP_min.y, BrakePedal_pct) annotation(
    Line(points = {{92, -50}, {96, -50}, {96, -60}, {110, -60}, {110, -60}}, color = {0, 0, 127}));
  connect(creepCorrection.u2, constant4.y) annotation(
    Line(points = {{72, 50}, {72, 26}, {67, 26}}, color = {0, 0, 127}));
  connect(creepCorrection.u1, add_steady_accel.y) annotation(
    Line(points = {{72, 62}, {68.5, 62}, {68.5, 56}, {67, 56}}, color = {0, 0, 127}));
  connect(AP_max.u2, creepCorrection.y) annotation(
    Line(points = {{70, 86}, {64, 86}, {64, 74}, {95, 74}, {95, 56}}, color = {0, 0, 127}));
  connect(const0.y, AP_max.u1) annotation(
    Line(points = {{60, 98}, {70, 98}, {70, 98}, {70, 98}}, color = {0, 0, 127}));
  connect(creepCorrection.y, transitionfactor.u) annotation(
    Line(points = {{96, 56}, {98, 56}, {98, 0}, {-56, 0}, {-56, -30}, {-48, -30}, {-48, -30}}, color = {0, 0, 127}));
end Driver;