within AICE.Examples.METI_2020.Components;

model SingleCylinderEngine
  extends Icons.PowerUnits.SingleCylinder;
  extends BaseClasses.SingleCylinderEngineParameters;
  extends AICE.PowerUnits.Combustions.Interfaces.PartialFluidEngine;
  final parameter CarUnits.Length pistonStroke = chamber.pistonStroke;
  final parameter CarUnits.Radius r_crank = pistonStroke / 2 * sqrt(1 - 4 * offset_crank ^ 2 / (4 * l_connRod ^ 2 - pistonStroke ^ 2));
  AICE.PowerUnits.Combustions.Cams.CamProfileFromFile cam_intake annotation(
    Placement(visible = true, transformation(origin = {-30, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  Modelica.Fluid.Sensors.MassFlowRate intakeMassFlowRate(redeclare replaceable package Medium = Medium) annotation(
    Placement(visible = true, transformation(origin = {-76, 55}, extent = {{-5, -5}, {5, 5}}, rotation = -90)));
  Modelica.Fluid.Valves.ValveCompressible valve_intake(redeclare replaceable package Medium = Medium) annotation(
    Placement(visible = true, transformation(origin = {-30, 25}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.PowerUnits.Combustions.Cams.CamProfileFromFile cam_exhaust annotation(
    Placement(visible = true, transformation(origin = {+30, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  Modelica.Fluid.Valves.ValveCompressible valve_exhaust(redeclare replaceable package Medium = Medium) annotation(
    Placement(visible = true, transformation(origin = {+30, 25}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Fluid.Sources.AngleBasedInjectorAbyF injector(redeclare replaceable package Medium = Medium, nPorts = 1) annotation(
    Placement(visible = true, transformation(origin = {0, 40}, extent = {{-10, 10}, {10, -10}}, rotation = -90)));
  AICE.PowerUnits.Combustions.Chambers.WiebeWoschniChamber chamber(redeclare replaceable package Medium = Medium, use_HeatTransfer = true, use_portsData = false, nPorts = 3, V_displacement = V_displacement, compressionRatio = compressionRatio, bore = bore, phi_combustion_begin = phi_combustion_begin, phi_combustion_period = phi_combustion_period, m_wiebe = m_wiebe, c1_woschni_intakeExhaust = c1_woschni_intakeExhaust, c1_woschni_compressionPower = c1_woschni_compressionPower, c2_woschni = c2_woschni, use_phi_piston = true, use_intakeValveOpens = true, use_exhaustValveOpens = true) annotation(
    Placement(visible = true, transformation(origin = {0, 0}, extent = {{20, -20}, {-20, 20}}, rotation = 180)));
  AICE.Mechanics.Rotational.Components.CrankAndConnRod crankAndConnRod(r_crank = r_crank, l_connRod = l_connRod, offset_crank = offset_crank) annotation(
    Placement(visible = true, transformation(origin = {0, -50}, extent = {{10, 10}, {-10, -10}}, rotation = -90)));
  Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(
    Placement(visible = true, transformation(origin = {60, 0}, extent = {{10, -10}, {-10, 10}})));
  Modelica.Blocks.Interfaces.RealInput fuelmpr annotation(
    Placement(visible = true, transformation(origin = {-100, 40}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {-100, 40}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
  Modelica.Blocks.Math.Product product annotation(
    Placement(visible = true, transformation(origin = {-50, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(flange, cam_intake.flange) annotation(
    Line(points = {{100, 0}, {80, 0}, {80, 90}, {-30, 90}, {-30, 80}}));
  connect(cam_intake.opening, valve_intake.opening) annotation(
    Line(points = {{-30, 60}, {-30, 30}}, color = {0, 0, 127}));
  connect(cam_intake.opens, chamber.intakeValveOpens) annotation(
    Line(points = {{-24, 64}, {-24, 58}, {42, 58}, {42, -12}, {20, -12}}, color = {255, 0, 255}, pattern = LinePattern.Dash));
  connect(flange, cam_exhaust.flange) annotation(
    Line(points = {{100, 0}, {80, 0}, {80, 90}, {+30, 90}, {+30, 80}}));
  connect(cam_exhaust.opening, valve_exhaust.opening) annotation(
    Line(points = {{+30, 60}, {+30, 30}}, color = {0, 0, 127}));
  connect(cam_exhaust.opens, chamber.exhaustValveOpens) annotation(
    Line(points = {{36, 64}, {36, 58}, {42, 58}, {42, -8}, {20, -8}}, color = {255, 0, 255}, pattern = LinePattern.Dash));
  connect(flange, angleSensor.flange) annotation(
    Line(points = {{100, 0}, {80, 0}, {70, 0}}));
  connect(angleSensor.phi, injector.phi_piston) annotation(
    Line(points = {{50, 0}, {45, 0}, {45, 55}, {0, 55}, {0, 50}}, color = {0, 0, 127}, pattern = LinePattern.Dash));
// fluid connections
  connect(port_intake, intakeMassFlowRate.port_a) annotation(
    Line(points = {{-60, 100}, {-60, 80}, {-76, 80}, {-76, 60}}, color = {0, 127, 255}));
  connect(intakeMassFlowRate.port_b, valve_intake.port_a) annotation(
    Line(points = {{-76, 50}, {-76, 25}, {-40, 25}}, color = {0, 127, 255}));
  connect(valve_intake.port_b, chamber.ports[1]) annotation(
    Line(points = {{-20, 25}, {-6, 25}, {-6, 20}, {0, 20}}, color = {0, 127, 255}));
  connect(injector.ports[1], chamber.ports[2]) annotation(
    Line(points = {{0, 30}, {0, 20}}, color = {0, 127, 255}, thickness = 0.5));
  connect(valve_exhaust.port_a, chamber.ports[3]) annotation(
    Line(points = {{+20, 25}, {+6, 25}, {+6, 20}, {0, 20}}, color = {0, 127, 255}));
  connect(port_exhaust, valve_exhaust.port_b) annotation(
    Line(points = {{+60, 100}, {+60, 25}, {+40, 25}}, color = {0, 127, 255}));
// other connections
  connect(chamber.heatPort, heatPort) annotation(
    Line(points = {{-20, 0}, {-60, 0}, {-60, -100}, {-100, -100}, {-100, -100}}, color = {191, 0, 0}));
  connect(crankAndConnRod.flangeR, flange) annotation(
    Line(points = {{0, -60}, {0, -60}, {0, -80}, {80, -80}, {80, 0}, {100, 0}, {100, 0}}));
  connect(crankAndConnRod.flangeT, chamber.flange) annotation(
    Line(points = {{0, -40}, {0, -40}, {0, -20}, {0, -20}}, color = {0, 127, 0}));
  connect(intakeMassFlowRate.m_flow, product.u1) annotation(
    Line(points = {{-70, 56}, {-62, 56}}, color = {0, 0, 127}));
  connect(product.u2, fuelmpr) annotation(
    Line(points = {{-62, 44}, {-82, 44}, {-82, 40}, {-100, 40}}, color = {0, 0, 127}));
  connect(product.y, injector.intakeMassFlowRate) annotation(
    Line(points = {{-38, 50}, {-6, 50}, {-6, 50}, {-6, 50}}, color = {0, 0, 127}, pattern = LinePattern.Dash));
  connect(angleSensor.phi, chamber.phi_piston) annotation(
    Line(points = {{50, 0}, {45, 0}, {45, -16}, {20, -16}}, color = {0, 0, 127}, pattern = LinePattern.Dash));
end SingleCylinderEngine;