within AICE.Examples.METI_2020.Components;

block ValveController

  parameter SI.Temperature T_radiatorActivated;
  parameter SI.Temperature T_radiatorDeactivated;

  parameter Real radiatorOpening_max;
  parameter Real radiatorOpening_min;

  parameter SI.Time t_open = 1.0
    "Time from full close to full open";

  AICE.Blocks.Interfaces.TemperatureInput T
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.RealOutput opening_radiator(
    min = 0.0, max = 1.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.RealOutput opening_bypass(
    min = 0.0, max = 1.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {100, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {100, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  // components
  Modelica.Blocks.Logical.Hysteresis hysteresis(
    uLow = T_radiatorDeactivated,
    uHigh = T_radiatorActivated
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Real opening_discrete;

equation

  connect(T, hysteresis.u) annotation(
    Line(points = {{-100, 0}, {-60, 0}}, color = {0, 0, 127})
  );

  opening_discrete = (
    if hysteresis.y then
      radiatorOpening_max
    else 
      radiatorOpening_min
    );

  der(opening_radiator) = (
    t_open ^ (-1)
    * Modelica.Fluid.Utilities.regStep(
      (opening_discrete - opening_radiator),
      +1, -1
    )
  );

  1 = opening_radiator + opening_bypass;

  annotation(
    Icon(
      graphics = {
        Rectangle(
          fillColor = {85, 170, 0}, fillPattern = FillPattern.Solid,
          extent = {{-100, 100}, {100, -100}}
        ),
        Text(
          extent={{-150,100},{150,130}},
          lineColor={0,0,255},
          textString="%name"
        )
      }
    )
  );


end ValveController;
