within AICE.Examples.METI_2020;

model TestNAEngine_SpeedMode
  extends Modelica.Icons.Example;
  inner Modelica.Fluid.System system annotation();
  // Placement(
  //   visible = true,
  //   transformation(origin = {-70, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
  // )
  AICE.Blocks.Sources.ConstantAngle throttleOpening(k = 1.39626) annotation(
    Placement(visible = true, transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Mechanics.Rotational.Sources.FixedSpeed engineSpeed(w = 314.159) annotation(
    Placement(visible = true, transformation(origin = {50, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  AICE.Blocks.Thermal.HeatTransfer.Sources.FixedTemperature engineTemperature(T = 550) annotation(
    Placement(visible = true, transformation(origin = {0, -50}, extent = {{-10, 10}, {10, -10}}, rotation = 90)));
  AICE.Examples.METI_2020.Components.SampleEngineUnitNA engine annotation(
    Placement(visible = true, transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.CycleMean cycleMeanFuelFlowRate_kgPerSec(y(final quantity = "MassFlowRate", final unit = "kg/s")) annotation(
    Placement(visible = true, transformation(origin = {50, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain cycleMeanFuelFlowRate(k = 1000, y(final quantity = "MassFlowRate", final unit = "g/s")) annotation(
    Placement(visible = true, transformation(origin = {80, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.CycleMean cycleMeanTorque(y(final quantity = "Torque", final unit = "N.m")) annotation(
    Placement(visible = true, transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant fuelMpr(k = 1)  annotation(
    Placement(visible = true, transformation(origin = {-50, 38}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(throttleOpening.y, engine.phi_throttle) annotation(
    Line(points = {{-40, 0}, {-10, 0}}, color = {0, 0, 127}));
  connect(engineSpeed.angularVelocity, engine.angularVelocity) annotation(
    Line(points = {{40, +6}, {10, +6}}, color = {0, 0, 127}));
  connect(engineSpeed.torque, engine.torque) annotation(
    Line(points = {{40, -6}, {10, -6}}, color = {0, 0, 255}));
  connect(engineSpeed.angularVelocity, cycleMeanFuelFlowRate_kgPerSec.angularVelocity) annotation(
    Line(points = {{40, 6}, {30, 6}, {30, 80}, {40, 80}}, color = {0, 0, 127}));
  connect(engine.fuelFlowRate, cycleMeanFuelFlowRate_kgPerSec.u) annotation(
    Line(points = {{-6, 6}, {-6, 86}, {20, 86}, {40, 86}}, color = {0, 0, 127}));
  connect(cycleMeanFuelFlowRate_kgPerSec.y, cycleMeanFuelFlowRate.u) annotation(
    Line(points = {{60, 80}, {70, 80}}, color = {0, 0, 127}));
  connect(engineSpeed.angularVelocity, cycleMeanTorque.angularVelocity) annotation(
    Line(points = {{40, 6}, {30, 6}, {30, 50}, {70, 50}}, color = {0, 0, 127}));
  connect(engine.torque, cycleMeanTorque.u) annotation(
    Line(points = {{10, -6}, {20, -6}, {20, 56}, {70, 56}}, color = {0, 0, 255}));
  connect(engine.temperature, engineTemperature.temperature) annotation(
    Line(points = {{6, -10}, {6, -10}, {6, -40}, {6, -40}}, color = {0, 0, 127}));
  connect(engine.heatFlowRate, engineTemperature.heatFlowRate) annotation(
    Line(points = {{-6, -10}, {-6, -10}, {-6, -40}, {-6, -40}}, color = {0, 0, 127}));
  connect(fuelMpr.y, engine.fuelMpr) annotation(
    Line(points = {{-38, 38}, {-20, 38}, {-20, 4}, {-10, 4}, {-10, 4}}, color = {0, 0, 127}));
  annotation(
    experiment(StartTime = 0, StopTime = 0.08, Tolerance = 1e-6, Interval = 1e-5),
    __OpenModelica_simulationFlags( lv = "LOG_STATS", maxStepSize = "0.5e-6", outputFormat = "mat", s = "dassl", jacobian = "numerical", ls = "lis"));
end TestNAEngine_SpeedMode;
