within AICE.Examples.METI_2020;
//Based on METI_2019.TestEngine
//Engine Speed Parameter assigned
//This model is base of TestEngine2, specialized for torque mapping
model TestTCEngine_LoadMode
  extends Modelica.Icons.Example;
  //inner Modelica.Fluid.System system annotation();
  // Placement(
  //   visible = true,
  //   transformation(origin = {-70, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
  // )
  parameter CarUnits.AngularVelocity_rpm EngineSpeed = 2000;
  AICE.Blocks.Mechanics.Rotational.Components.FlyWheel_AOTI flyWheel(Inertia = 0.1, w_init = 100) annotation(
    Placement(visible = true, transformation(origin = {90, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  AICE.Blocks.Thermal.HeatTransfer.Sources.FixedTemperature engineTemperature(T = 550) annotation(
    Placement(visible = true, transformation(origin = {0, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  AICE.Examples.METI_2020.Components.SampleEngineUnitTCCACTWC3 engine annotation(
    Placement(visible = true, transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.CycleMean cycleMeanFuelFlowRate_kgPerSec(y(final quantity = "MassFlowRate", final unit = "kg/s")) annotation(
    Placement(visible = true, transformation(origin = {50, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain cycleMeanFuelFlowRate(k = 1000, y(final quantity = "MassFlowRate", final unit = "g/s")) annotation(
    Placement(visible = true, transformation(origin = {80, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.CycleMean cycleMeanTorque(y(final quantity = "Torque", final unit = "N.m")) annotation(
    Placement(visible = true, transformation(origin = {80, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.ECU_Simple eCU_Simple annotation(
    Placement(visible = true, transformation(origin = {-60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant accelPedalOpening(k = 0.8) annotation(
    Placement(visible = true, transformation(origin = {-90, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Electrical.Components.MotorStarter_OnewayGearSimple_AITO Starter_OnewayGear(BackVoltage_table = [-1, 0; 0, 0; 1000, 20; 1001, 20], k = 10, ratio = 10)  annotation(
    Placement(visible = true, transformation(origin = {22, -28}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Add add_flangeTorque annotation(
    Placement(visible = true, transformation(origin = {62, -12}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(flyWheel.angularVelocity, engine.angularVelocity) annotation(
    Line(points = {{80, 6}, {10, 6}}, color = {0, 0, 127}));
  connect(engineTemperature.temperature, engine.temperature) annotation(
    Line(points = {{6, -40}, {6, -10}}, color = {0, 0, 127}));
  connect(engineTemperature.heatFlowRate, engine.heatFlowRate) annotation(
    Line(points = {{-6, -40}, {-6, -10}}, color = {0, 0, 127}));
  connect(flyWheel.angularVelocity, cycleMeanFuelFlowRate_kgPerSec.angularVelocity) annotation(
    Line(points = {{80, 6}, {30, 6}, {30, 80}, {40, 80}}, color = {0, 0, 127}));
  connect(engine.fuelFlowRate, cycleMeanFuelFlowRate_kgPerSec.u) annotation(
    Line(points = {{-6, 6}, {-6, 86}, {20, 86}, {40, 86}}, color = {0, 0, 127}));
  connect(cycleMeanFuelFlowRate_kgPerSec.y, cycleMeanFuelFlowRate.u) annotation(
    Line(points = {{60, 80}, {70, 80}}, color = {0, 0, 127}));
  connect(flyWheel.angularVelocity, cycleMeanTorque.angularVelocity) annotation(
    Line(points = {{80, 6}, {30, 6}, {30, 50}, {70, 50}}, color = {0, 0, 127}));
  connect(engine.torque, cycleMeanTorque.u) annotation(
    Line(points = {{10, -6}, {20, -6}, {20, 56}, {70, 56}}, color = {0, 0, 255}));
  connect(eCU_Simple.ThrottleAngle, engine.phi_throttle) annotation(
    Line(points = {{-50, 8}, {-10, 8}, {-10, 8}, {-10, 8}}, color = {0, 0, 127}));
  connect(eCU_Simple.WasteGateOpening, engine.wastegateOpening) annotation(
    Line(points = {{-50, 6}, {-10, 6}, {-10, 6}, {-10, 6}}, color = {0, 0, 127}));
  connect(eCU_Simple.FuelMpr, engine.fuelMpr) annotation(
    Line(points = {{-50, -2}, {-20, -2}, {-20, 4}, {-10, 4}, {-10, 4}}, color = {0, 0, 127}));
  connect(engine.p_boost, eCU_Simple.p_boost) annotation(
    Line(points = {{-10, 10}, {-20, 10}, {-20, 20}, {-76, 20}, {-76, 6}, {-70, 6}}, color = {0, 0, 127}));
  connect(accelPedalOpening.y, eCU_Simple.AccelPedalOpening) annotation(
    Line(points = {{-78, 10}, {-74, 10}, {-74, 8}, {-70, 8}, {-70, 8}}, color = {0, 0, 127}));
  connect(Starter_OnewayGear.angularVelocityInput, flyWheel.angularVelocity) annotation(
    Line(points = {{32, -22}, {36, -22}, {36, 6}, {80, 6}}, color = {0, 0, 127}));
  connect(eCU_Simple.StarterVoltage, Starter_OnewayGear.voltageInput) annotation(
    Line(points = {{-50, -4}, {-20, -4}, {-20, -14}, {26, -14}, {26, -18}, {26, -18}}, color = {0, 0, 127}));
  connect(add_flangeTorque.u1, engine.torque) annotation(
    Line(points = {{50, -6}, {10, -6}, {10, -6}, {10, -6}}, color = {0, 0, 127}));
  connect(add_flangeTorque.u2, Starter_OnewayGear.torqueOutput) annotation(
    Line(points = {{50, -18}, {40, -18}, {40, -34}, {32, -34}, {32, -34}}, color = {0, 0, 127}));
  connect(add_flangeTorque.y, flyWheel.torque) annotation(
    Line(points = {{74, -12}, {76, -12}, {76, -6}, {80, -6}, {80, -6}}, color = {0, 0, 127}));
  annotation(
    experiment(StartTime = 0, StopTime = 0.3, Tolerance = 1e-6, Interval = 1e-5),
    __OpenModelica_simulationFlags( lv = "LOG_STATS", maxStepSize = "0.5e-6", outputFormat = "mat", s = "dassl", jacobian = "numerical", ls = "lis"));
end TestTCEngine_LoadMode;
