within AICE.Examples.METI_2020;

model TestThermalManagement
  extends Modelica.Icons.Example;
  replaceable package Medium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium;
  inner Modelica.Fluid.System system annotation();
  // Placement(
  //   visible = true,
  //   transformation(origin = {-210, -150}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
  // )
  // parameters
  // Specific heat of iron
  //   430 ~ 460 [J/(kg.K)]
  // Engine block mass
  //   ~= 1.0e2 [kg]
  // => Engine heat capacity
  //   ~= 5.0e4
  parameter SI.HeatCapacity C_engine = 5.0e4;
  // mean engine room temperature
  //   ~= 550 [K]
  // mean water temperature
  //   ~= 360 [K]
  // mean engine heat flow
  //   ~= 1.5e4 [W]
  // G = 1.5e4 / (550 - 360) ~= 80 [W/K]
  parameter SI.ThermalConductance G_engine = 80.0;
  // The cooling capacity is set higher to make the radiator switching easier to understand.
  // It was assumed that the ON / OFF ratio would be 50:50
  // by doubling the cooling capacity of the radiator to the heat generated by the engine.
  // mean radiation heat flow
  //   ~= 3.0e4 [W]
  // mean water temeperature
  //   ~= 360 [K]
  // air temperature
  //   ~= 300 [K]
  // G = 3.0e4 / (360 - 300) ~= 1.0e3 [W/K]
  parameter SI.ThermalConductance G_radiator = 1.0e3;
  // range of water temperature
  parameter SI.Temperature T_water_low = 358.15;
  //  80℃
  parameter SI.Temperature T_water_high = 363.15;
  //  90℃
  parameter SI.Temperature T_water_start = 293.15;
  // 20℃
  // geometries
  parameter CarUnits.Diameter d_1 = 7.0e-3;
  parameter CarUnits.Diameter d_radiatorValve = 7.0e-3;
  parameter CarUnits.Diameter d_bypassValve = 7.0e-3;
  parameter CarUnits.Diameter d_2 = 7.0e-3;
  // *** ***
  AICE.Blocks.Sources.ConstantAngle throttleOpening(k = 1.39626) annotation(
    Placement(visible = true, transformation(origin = {-130, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.SampleEngineUnitNA engine annotation(
    Placement(visible = true, transformation(origin = {-90, 110}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Mechanics.Rotational.Sources.FixedSpeed engineSpeed(w = 104.72) annotation(
    Placement(visible = true, transformation(origin = {-50, 110}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  AICE.Blocks.Thermal.HeatTransfer.Components.HeatCapacitor_AOOTII engineHeatCapacity(C = C_engine, T_start = 550.0) annotation(
    Placement(visible = true, transformation(origin = {-90, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  AICE.Blocks.Thermal.HeatTransfer.Components.ThermalConductor_AIITOO conductor(G = G_engine) annotation(
    Placement(visible = true, transformation(origin = {-90, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  AICE.Blocks.Mechanics.Rotational.Sources.FixedSpeed pumpSpeed(w = 261.799) annotation(
    Placement(visible = true, transformation(origin = {-210, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Fluid.Incompressible.Machines.TurboPump_TFAIITOO turboPump(replaceable package Medium = Medium) annotation(
    Placement(visible = true, transformation(origin = {-130, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  AICE.Blocks.Fluid.Incompressible.Vessels.ClosedVolumeWithHeat_TFAOOTII heatSink(redeclare replaceable package Medium = Medium, V = 1e-3, T_start = T_water_start, energy_heatPort = AICE.Blocks.Types.Energy.Sink) annotation(
    Placement(visible = true, transformation(origin = {-90, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Fluid.Compressible.Fittings.SimpleGenericOrifice_TFAIITOO pressureDrop1(redeclare replaceable package Medium = Medium, diameter = d_1, zeta = 4) annotation(
    Placement(visible = true, transformation(origin = {-30, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Fluid.Incompressible.Fittings.TeeJunctionVolume_TFAOOOTIII_down junction1(redeclare replaceable package Medium = Medium, V = 1.0e-4, T_start = T_water_start, energy_port_1 = AICE.Blocks.Types.Energy.Sink, energy_port_2 = AICE.Blocks.Types.Energy.Source, energy_port_3 = AICE.Blocks.Types.Energy.Source) annotation(
    Placement(visible = true, transformation(origin = {50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  // radiator pipeline
  AICE.Blocks.Fluid.Compressible.Valves.ValveCompressible_TFAIITOO radiatorValve(
    redeclare replaceable package Medium = Medium, 
    Av = AICE.Math.diameter2area(d_radiatorValve), 
    CvData = Modelica.Fluid.Types.CvTypes.Av, 
    dp_nominal = 1.0, 
    m_flow_nominal = 1.0, 
    p_nominal = system.p_start) annotation(
    Placement(visible = true, transformation(origin = {90, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Fluid.Incompressible.Vessels.ClosedVolumeWithHeat_TFAOOTII radiator(
    redeclare replaceable package Medium = Medium, 
    T_start = T_water_start, 
    V = 0.5e-3, 
    energy_heatPort = AICE.Blocks.Types.Energy.Source, 
    energy_port_a = AICE.Blocks.Types.Energy.Sink, 
    energy_port_b = AICE.Blocks.Types.Energy.Source) annotation(
    Placement(visible = true, transformation(origin = {130, -50}, extent = {{+10, -10}, {-10, 10}}, rotation = -90)));
  AICE.Blocks.Fluid.Compressible.Fittings.SimpleGenericOrifice_TFAIITOO radiatorPressureDrop(redeclare replaceable package Medium = Medium, diameter = 0.1, zeta = 4) annotation(
    Placement(visible = true, transformation(origin = {130, -110}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  // radiator thermal components
  AICE.Blocks.Thermal.HeatTransfer.Components.ThermalConductor_AIITOO radiatorConductor(G = G_radiator) annotation(
    Placement(visible = true, transformation(origin = {170, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Thermal.HeatTransfer.Sources.FixedTemperature airTemperature(T = 293.15) annotation(
    Placement(visible = true, transformation(origin = {210, -50}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  // bypass pipeline
  AICE.Blocks.Fluid.Compressible.Valves.ValveCompressible_TFAIITOO bypassValve(redeclare replaceable package Medium = Medium, Av = AICE.Math.diameter2area(d_bypassValve), CvData = Modelica.Fluid.Types.CvTypes.Av, dp_nominal = 1.0, m_flow_nominal = 1.0, p_nominal = 100000) annotation(
    Placement(visible = true, transformation(origin = {50, -50}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  AICE.Blocks.Fluid.Incompressible.Fittings.TeeJunctionVolume_TFAOOOTIII_up junction2(redeclare replaceable package Medium = Medium, V = 1.0e-4, T_start = T_water_start, energy_port_1 = AICE.Blocks.Types.Energy.Source, energy_port_2 = AICE.Blocks.Types.Energy.Sink, energy_port_3 = AICE.Blocks.Types.Energy.Sink) annotation(
    Placement(visible = true, transformation(origin = {50, -150}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Fluid.Incompressible.Sensors.TemperatureTwoPort waterTemperature(redeclare replaceable package Medium = Medium) annotation(
    Placement(visible = true, transformation(origin = {10, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Examples.METI_2020.Components.ValveController valveController(T_radiatorActivated = T_water_high, T_radiatorDeactivated = T_water_low, radiatorOpening_max = 0.97, radiatorOpening_min = 0.03) annotation(
    Placement(visible = true, transformation(origin = {50, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant fuelMpr annotation(
    Placement(visible = true, transformation(origin = {-130, 150}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
// # rotational power
// ## constants for pump
  connect(pumpSpeed.torque, turboPump.torque) annotation(
    Line(points = {{-200, -76}, {-140, -76}, {-140, -76}, {-140, -76}}, color = {0, 0, 127}));
  connect(pumpSpeed.angularVelocity, turboPump.angularVelocity) annotation(
    Line(points = {{-200, -64}, {-140, -64}, {-140, -64}, {-140, -64}}, color = {0, 0, 127}));
// ## constants for engine
  connect(throttleOpening.y, engine.phi_throttle) annotation(
    Line(points = {{-120, 110}, {-100, 110}}, color = {0, 0, 127}));
  connect(engineSpeed.angularVelocity, engine.angularVelocity) annotation(
    Line(points = {{-60, 116}, {-80, 116}}, color = {0, 0, 127}));
  connect(engineSpeed.torque, engine.torque) annotation(
    Line(points = {{-60, 104}, {-80, 104}}, color = {0, 0, 127}));
// ## engine -> heat sink
  connect(engine.temperature, engineHeatCapacity.temperature_a) annotation(
    Line(points = {{-84, 100}, {-84, 80}}, color = {0, 0, 127}));
  connect(engine.heatFlowRate, engineHeatCapacity.heatFlowRate_a) annotation(
    Line(points = {{-96, 100}, {-96, 80}}, color = {0, 0, 127}));
  connect(engineHeatCapacity.temperature_b, conductor.temperature_a) annotation(
    Line(points = {{-84, 60}, {-84, 40}}, color = {0, 0, 127}));
  connect(engineHeatCapacity.heatFlowRate_b, conductor.heatFlowRate_a) annotation(
    Line(points = {{-96, 60}, {-96, 40}}, color = {0, 0, 127}));
// ## radiator -> air
  connect(radiatorConductor.temperature_b, airTemperature.temperature) annotation(
    Line(points = {{180, -44}, {200, -44}}, color = {0, 0, 127}));
  connect(radiatorConductor.heatFlowRate_b, airTemperature.heatFlowRate) annotation(
    Line(points = {{180, -56}, {200, -56}}, color = {0, 0, 127}));
// # heat convection
  connect(conductor.heatFlowRate_b, heatSink.heatFlowRate) annotation(
    Line(points = {{-96, 20}, {-96, 20}, {-96, 0}, {-96, 0}}, color = {0, 0, 127}));
  connect(conductor.temperature_b, heatSink.temperature) annotation(
    Line(points = {{-84, 20}, {-84, 20}, {-84, 0}, {-84, 0}}, color = {0, 0, 127}));
// ## pump -> heat sink -> 1st junction
  connect(turboPump.absolutePressure_b, heatSink.absolutePressure_a) annotation(
    Line(points = {{-138, -60}, {-138, -60}, {-138, -2}, {-100, -2}, {-100, -2}}, color = {0, 0, 127}));
  connect(turboPump.temperature_b, heatSink.temperature_a) annotation(
    Line(points = {{-136, -60}, {-136, -60}, {-136, -4}, {-100, -4}, {-100, -4}}, color = {0, 0, 127}));
  connect(turboPump.specificEnthalpy_mix_b, heatSink.specificEnthalpy_outflow_a) annotation(
    Line(points = {{-134, -60}, {-134, -60}, {-134, -6}, {-100, -6}, {-100, -6}}, color = {0, 0, 127}));
  connect(turboPump.massFractions_mix_b[:], heatSink.massFractions_outflow_a[:]) annotation(
    Line(points = {{-132, -60}, {-132, -60}, {-132, -8}, {-100, -8}, {-100, -8}}, color = {0, 0, 127}, thickness = 0.5));
  connect(turboPump.massFractions_outflow_b[:], heatSink.massFractions_mix_a[:]) annotation(
    Line(points = {{-128, -60}, {-128, -60}, {-128, -12}, {-100, -12}, {-100, -12}, {-100, -12}}, color = {0, 0, 127}, thickness = 0.5));
  connect(turboPump.specificEnthalpy_outflow_b, heatSink.specificEnthalpy_mix_a) annotation(
    Line(points = {{-126, -60}, {-126, -60}, {-126, -14}, {-100, -14}, {-100, -14}}, color = {0, 0, 127}));
  connect(turboPump.massFlowRate_b, heatSink.massFlowRate_a) annotation(
    Line(points = {{-124, -60}, {-124, -60}, {-124, -16}, {-100, -16}, {-100, -16}}, color = {0, 0, 127}));
  connect(turboPump.enthalpyFlowRate_b, heatSink.enthalpyFlowRate_a) annotation(
    Line(points = {{-122, -60}, {-122, -60}, {-122, -18}, {-100, -18}, {-100, -18}}, color = {0, 0, 127}));
  connect(heatSink.absolutePressure_b, pressureDrop1.absolutePressure_a) annotation(
    Line(points = {{-80, -2}, {-40, -2}, {-40, -2}, {-40, -2}}, color = {0, 0, 127}));
  connect(heatSink.temperature_b, pressureDrop1.temperature_a) annotation(
    Line(points = {{-80, -4}, {-40, -4}, {-40, -4}, {-40, -4}}, color = {0, 0, 127}));
  connect(heatSink.specificEnthalpy_outflow_b, pressureDrop1.specificEnthalpy_mix_a) annotation(
    Line(points = {{-80, -6}, {-40, -6}, {-40, -6}, {-40, -6}}, color = {0, 0, 127}));
  connect(heatSink.massFractions_outflow_b[:], pressureDrop1.massFractions_mix_a[:]) annotation(
    Line(points = {{-80, -8}, {-40, -8}, {-40, -8}, {-40, -8}}, color = {0, 0, 127}, thickness = 0.5));
  connect(heatSink.massFractions_mix_b[:], pressureDrop1.massFractions_outflow_a[:]) annotation(
    Line(points = {{-80, -12}, {-40, -12}, {-40, -12}, {-40, -12}}, color = {0, 0, 127}, thickness = 0.5));
  connect(heatSink.specificEnthalpy_mix_b, pressureDrop1.specificEnthalpy_outflow_a) annotation(
    Line(points = {{-80, -14}, {-40, -14}, {-40, -14}, {-40, -14}}, color = {0, 0, 127}));
  connect(heatSink.massFlowRate_b, pressureDrop1.massFlowRate_a) annotation(
    Line(points = {{-80, -16}, {-38, -16}, {-38, -16}, {-40, -16}, {-40, -16}}, color = {0, 0, 127}));
  connect(heatSink.enthalpyFlowRate_b, pressureDrop1.enthalpyFlowRate_a) annotation(
    Line(points = {{-80, -18}, {-40, -18}, {-40, -18}, {-40, -18}}, color = {0, 0, 127}));
  connect(pressureDrop1.absolutePressure_b, waterTemperature.absolutePressure_a) annotation(
    Line(points = {{-20, -2}, {0, -2}, {0, -2}, {0, -2}}, color = {0, 0, 127}));
  connect(pressureDrop1.temperature_b, waterTemperature.temperature_a) annotation(
    Line(points = {{-20, -4}, {0, -4}, {0, -4}, {0, -4}}, color = {0, 0, 127}));
  connect(pressureDrop1.specificEnthalpy_mix_b, waterTemperature.specificEnthalpy_outflow_a) annotation(
    Line(points = {{-20, -6}, {0, -6}, {0, -6}, {0, -6}}, color = {0, 0, 127}));
  connect(pressureDrop1.massFractions_mix_b[:], waterTemperature.massFractions_outflow_a[:]) annotation(
    Line(points = {{-20, -8}, {0, -8}, {0, -8}, {0, -8}}, color = {0, 0, 127}, thickness = 0.5));
  connect(pressureDrop1.massFractions_outflow_b[:], waterTemperature.massFractions_mix_a[:]) annotation(
    Line(points = {{-20, -12}, {0, -12}, {0, -12}, {0, -12}}, color = {0, 0, 127}, thickness = 0.5));
  connect(pressureDrop1.specificEnthalpy_outflow_b, waterTemperature.specificEnthalpy_mix_a) annotation(
    Line(points = {{-20, -14}, {0, -14}, {0, -14}, {0, -14}}, color = {0, 0, 127}));
  connect(pressureDrop1.massFlowRate_b, waterTemperature.massFlowRate_a) annotation(
    Line(points = {{-20, -16}, {0, -16}, {0, -16}, {0, -16}}, color = {0, 0, 127}));
  connect(pressureDrop1.enthalpyFlowRate_b, waterTemperature.enthalpyFlowRate_a) annotation(
    Line(points = {{-20, -18}, {0, -18}, {0, -18}, {0, -18}}, color = {0, 0, 127}));
  connect(waterTemperature.absolutePressure_b, junction1.absolutePressure_1) annotation(
    Line(points = {{20, -2}, {40, -2}, {40, -2}, {40, -2}}, color = {0, 0, 127}));
  connect(waterTemperature.temperature_b, junction1.temperature_1) annotation(
    Line(points = {{20, -4}, {40, -4}, {40, -4}, {40, -4}}, color = {0, 0, 127}));
  connect(waterTemperature.specificEnthalpy_mix_b, junction1.specificEnthalpy_outflow_1) annotation(
    Line(points = {{20, -6}, {40, -6}, {40, -6}, {40, -6}}, color = {0, 0, 127}));
  connect(waterTemperature.massFractions_mix_b[:], junction1.massFractions_outflow_1[:]) annotation(
    Line(points = {{20, -8}, {40, -8}, {40, -8}, {40, -8}}, color = {0, 0, 127}, thickness = 0.5));
  connect(waterTemperature.massFractions_outflow_b[:], junction1.massFractions_mix_1[:]) annotation(
    Line(points = {{20, -12}, {40, -12}, {40, -12}, {40, -12}}, color = {0, 0, 127}, thickness = 0.5));
  connect(waterTemperature.specificEnthalpy_outflow_b, junction1.specificEnthalpy_mix_1) annotation(
    Line(points = {{20, -14}, {40, -14}, {40, -14}, {40, -14}}, color = {0, 0, 127}));
  connect(waterTemperature.massFlowRate_b, junction1.massFlowRate_1) annotation(
    Line(points = {{20, -16}, {40, -16}, {40, -16}, {40, -16}}, color = {0, 0, 127}));
  connect(waterTemperature.enthalpyFlowRate_b, junction1.enthalpyFlowRate_1) annotation(
    Line(points = {{20, -18}, {40, -18}, {40, -18}, {40, -18}}, color = {0, 0, 127}));
// ## control model
  connect(waterTemperature.T, valveController.T) annotation(
    Line(points = {{10, 2}, {10, 2}, {10, 30}, {40, 30}, {40, 30}}, color = {0, 0, 127}));
  connect(valveController.opening_radiator, radiatorValve.opening) annotation(
    Line(points = {{60, 36}, {90, 36}, {90, -2}, {90, -2}, {90, -2}}, color = {0, 0, 127}));
  connect(valveController.opening_bypass, bypassValve.opening) annotation(
    Line(points = {{60, 24}, {68, 24}, {68, -50}, {58, -50}, {58, -50}, {58, -50}}, color = {0, 0, 127}));
// ## radiator pipeline
  connect(junction1.absolutePressure_2, radiatorValve.absolutePressure_a) annotation(
    Line(points = {{60, -2}, {80, -2}, {80, -2}, {80, -2}}, color = {0, 0, 127}));
  connect(junction1.temperature_2, radiatorValve.temperature_a) annotation(
    Line(points = {{60, -4}, {80, -4}, {80, -4}, {80, -4}}, color = {0, 0, 127}));
  connect(junction1.specificEnthalpy_outflow_2, radiatorValve.specificEnthalpy_mix_a) annotation(
    Line(points = {{60, -6}, {80, -6}, {80, -6}, {80, -6}}, color = {0, 0, 127}));
  connect(junction1.massFractions_outflow_2[:], radiatorValve.massFractions_mix_a[:]) annotation(
    Line(points = {{60, -8}, {80, -8}, {80, -8}, {80, -8}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction1.massFractions_mix_2[:], radiatorValve.massFractions_outflow_a[:]) annotation(
    Line(points = {{60, -12}, {82, -12}, {82, -12}, {80, -12}, {80, -12}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction1.specificEnthalpy_mix_2, radiatorValve.specificEnthalpy_outflow_a) annotation(
    Line(points = {{60, -14}, {80, -14}, {80, -14}, {80, -14}}, color = {0, 0, 127}));
  connect(junction1.massFlowRate_2, radiatorValve.massFlowRate_a) annotation(
    Line(points = {{60, -16}, {80, -16}, {80, -16}, {80, -16}}, color = {0, 0, 127}));
  connect(junction1.enthalpyFlowRate_2, radiatorValve.enthalpyFlowRate_a) annotation(
    Line(points = {{60, -18}, {80, -18}, {80, -18}, {80, -18}}, color = {0, 0, 127}));
  connect(radiatorValve.absolutePressure_b, radiator.absolutePressure_b) annotation(
    Line(points = {{100, -2}, {138, -2}, {138, -40}, {138, -40}}, color = {0, 0, 127}));
  connect(radiatorValve.temperature_b, radiator.temperature_b) annotation(
    Line(points = {{100, -4}, {136, -4}, {136, -40}, {136, -40}}, color = {0, 0, 127}));
  connect(radiatorValve.specificEnthalpy_mix_b, radiator.specificEnthalpy_outflow_b) annotation(
    Line(points = {{100, -6}, {134, -6}, {134, -40}, {134, -40}}, color = {0, 0, 127}));
  connect(radiatorValve.massFractions_mix_b[:], radiator.massFractions_outflow_b[:]) annotation(
    Line(points = {{100, -8}, {132, -8}, {132, -40}, {132, -40}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiatorValve.massFractions_outflow_b[:], radiator.massFractions_mix_b[:]) annotation(
    Line(points = {{100, -12}, {128, -12}, {128, -40}, {128, -40}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiatorValve.specificEnthalpy_outflow_b, radiator.specificEnthalpy_mix_b) annotation(
    Line(points = {{100, -14}, {126, -14}, {126, -40}, {126, -40}}, color = {0, 0, 127}));
  connect(radiatorValve.massFlowRate_b, radiator.massFlowRate_b) annotation(
    Line(points = {{100, -16}, {124, -16}, {124, -40}, {124, -40}}, color = {0, 0, 127}));
  connect(radiatorValve.enthalpyFlowRate_b, radiator.enthalpyFlowRate_b) annotation(
    Line(points = {{100, -18}, {122, -18}, {122, -40}, {122, -40}}, color = {0, 0, 127}));
  connect(radiator.absolutePressure_a, radiatorPressureDrop.absolutePressure_a) annotation(
    Line(points = {{138, -60}, {138, -60}, {138, -100}, {138, -100}}, color = {0, 0, 127}));
  connect(radiator.temperature_a, radiatorPressureDrop.temperature_a) annotation(
    Line(points = {{136, -60}, {136, -60}, {136, -100}, {136, -100}}, color = {0, 0, 127}));
  connect(radiator.specificEnthalpy_outflow_a, radiatorPressureDrop.specificEnthalpy_mix_a) annotation(
    Line(points = {{134, -60}, {134, -60}, {134, -100}, {134, -100}}, color = {0, 0, 127}));
  connect(radiator.massFractions_outflow_a[:], radiatorPressureDrop.massFractions_mix_a[:]) annotation(
    Line(points = {{132, -60}, {132, -60}, {132, -100}, {132, -100}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiator.massFractions_mix_a[:], radiatorPressureDrop.massFractions_outflow_a[:]) annotation(
    Line(points = {{128, -60}, {128, -60}, {128, -100}, {128, -100}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiator.specificEnthalpy_mix_a, radiatorPressureDrop.specificEnthalpy_outflow_a) annotation(
    Line(points = {{126, -60}, {126, -60}, {126, -100}, {126, -100}}, color = {0, 0, 127}));
  connect(radiator.massFlowRate_a, radiatorPressureDrop.massFlowRate_a) annotation(
    Line(points = {{124, -60}, {124, -60}, {124, -100}, {124, -100}}, color = {0, 0, 127}));
  connect(radiator.enthalpyFlowRate_a, radiatorPressureDrop.enthalpyFlowRate_a) annotation(
    Line(points = {{122, -60}, {122, -60}, {122, -100}, {122, -100}}, color = {0, 0, 127}));
// ## bypass pipeline
  connect(junction1.absolutePressure_3, bypassValve.absolutePressure_a) annotation(
    Line(points = {{58, -20}, {58, -20}, {58, -40}, {58, -40}}, color = {0, 0, 127}));
  connect(junction1.temperature_3, bypassValve.temperature_a) annotation(
    Line(points = {{56, -20}, {56, -20}, {56, -40}, {56, -40}}, color = {0, 0, 127}));
  connect(junction1.specificEnthalpy_outflow_3, bypassValve.specificEnthalpy_mix_a) annotation(
    Line(points = {{54, -20}, {54, -20}, {54, -40}, {54, -40}}, color = {0, 0, 127}));
  connect(junction1.massFractions_outflow_3[:], bypassValve.massFractions_mix_a[:]) annotation(
    Line(points = {{52, -20}, {52, -20}, {52, -40}, {52, -40}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction1.massFractions_mix_3[:], bypassValve.massFractions_outflow_a[:]) annotation(
    Line(points = {{48, -20}, {48, -20}, {48, -40}, {48, -40}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction1.specificEnthalpy_mix_3, bypassValve.specificEnthalpy_outflow_a) annotation(
    Line(points = {{46, -20}, {46, -20}, {46, -40}, {46, -40}}, color = {0, 0, 127}));
  connect(junction1.massFlowRate_3, bypassValve.massFlowRate_a) annotation(
    Line(points = {{44, -20}, {44, -20}, {44, -40}, {44, -40}}, color = {0, 0, 127}));
  connect(junction1.enthalpyFlowRate_3, bypassValve.enthalpyFlowRate_a) annotation(
    Line(points = {{42, -20}, {42, -20}, {42, -40}, {42, -40}}, color = {0, 0, 127}));
  connect(bypassValve.absolutePressure_b, junction2.absolutePressure_3) annotation(
    Line(points = {{58, -60}, {58, -60}, {58, -140}, {58, -140}}, color = {0, 0, 127}));
  connect(bypassValve.temperature_b, junction2.temperature_3) annotation(
    Line(points = {{56, -60}, {56, -60}, {56, -140}, {56, -140}}, color = {0, 0, 127}));
  connect(bypassValve.specificEnthalpy_mix_b, junction2.specificEnthalpy_outflow_3) annotation(
    Line(points = {{54, -60}, {54, -60}, {54, -140}, {54, -140}}, color = {0, 0, 127}));
  connect(bypassValve.massFractions_mix_b[:], junction2.massFractions_outflow_3[:]) annotation(
    Line(points = {{52, -60}, {52, -60}, {52, -140}, {52, -140}}, color = {0, 0, 127}, thickness = 0.5));
  connect(bypassValve.massFractions_outflow_b[:], junction2.massFractions_mix_3[:]) annotation(
    Line(points = {{48, -60}, {48, -60}, {48, -140}, {48, -140}}, color = {0, 0, 127}, thickness = 0.5));
  connect(bypassValve.specificEnthalpy_outflow_b, junction2.specificEnthalpy_mix_3) annotation(
    Line(points = {{46, -60}, {46, -60}, {46, -140}, {46, -140}}, color = {0, 0, 127}));
  connect(bypassValve.massFlowRate_b, junction2.massFlowRate_3) annotation(
    Line(points = {{44, -60}, {44, -60}, {44, -140}, {44, -140}}, color = {0, 0, 127}));
  connect(bypassValve.enthalpyFlowRate_b, junction2.enthalpyFlowRate_3) annotation(
    Line(points = {{42, -60}, {42, -60}, {42, -140}, {42, -140}}, color = {0, 0, 127}));
// # Cooling water network
  connect(radiator.temperature, radiatorConductor.temperature_a) annotation(
    Line(points = {{140, -44}, {160, -44}, {160, -44}, {160, -44}}, color = {0, 0, 127}));
  connect(radiator.heatFlowRate, radiatorConductor.heatFlowRate_a) annotation(
    Line(points = {{140, -56}, {160, -56}, {160, -56}, {160, -56}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.enthalpyFlowRate_b, junction2.enthalpyFlowRate_2) annotation(
    Line(points = {{122, -120}, {122, -120}, {122, -158}, {60, -158}, {60, -158}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.massFlowRate_b, junction2.massFlowRate_2) annotation(
    Line(points = {{124, -120}, {124, -120}, {124, -156}, {60, -156}, {60, -156}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.specificEnthalpy_outflow_b, junction2.specificEnthalpy_mix_2) annotation(
    Line(points = {{126, -120}, {126, -120}, {126, -154}, {60, -154}, {60, -154}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.massFractions_outflow_b[:], junction2.massFractions_mix_2[:]) annotation(
    Line(points = {{128, -120}, {128, -120}, {128, -152}, {60, -152}, {60, -152}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiatorPressureDrop.massFractions_mix_b[:], junction2.massFractions_outflow_2[:]) annotation(
    Line(points = {{132, -120}, {132, -120}, {132, -148}, {60, -148}, {60, -148}}, color = {0, 0, 127}, thickness = 0.5));
  connect(radiatorPressureDrop.specificEnthalpy_mix_b, junction2.specificEnthalpy_outflow_2) annotation(
    Line(points = {{134, -120}, {134, -120}, {134, -146}, {60, -146}, {60, -146}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.temperature_b, junction2.temperature_2) annotation(
    Line(points = {{136, -120}, {136, -120}, {136, -144}, {60, -144}, {60, -144}}, color = {0, 0, 127}));
  connect(radiatorPressureDrop.absolutePressure_b, junction2.absolutePressure_2) annotation(
    Line(points = {{138, -120}, {138, -120}, {138, -142}, {60, -142}, {60, -142}}, color = {0, 0, 127}));
// ## 2nd junction -> pump
  connect(junction2.absolutePressure_1, turboPump.absolutePressure_a) annotation(
    Line(points = {{40, -142}, {-138, -142}, {-138, -80}, {-138, -80}}, color = {0, 0, 127}));
  connect(junction2.temperature_1, turboPump.temperature_a) annotation(
    Line(points = {{40, -144}, {-136, -144}, {-136, -80}, {-136, -80}}, color = {0, 0, 127}));
  connect(junction2.specificEnthalpy_outflow_1, turboPump.specificEnthalpy_mix_a) annotation(
    Line(points = {{40, -146}, {-134, -146}, {-134, -80}, {-134, -80}}, color = {0, 0, 127}));
  connect(junction2.massFractions_outflow_1[:], turboPump.massFractions_mix_a[:]) annotation(
    Line(points = {{40, -148}, {-132, -148}, {-132, -80}, {-132, -80}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction2.massFractions_mix_1[:], turboPump.massFractions_outflow_a[:]) annotation(
    Line(points = {{40, -152}, {-128, -152}, {-128, -80}, {-128, -80}}, color = {0, 0, 127}, thickness = 0.5));
  connect(junction2.specificEnthalpy_mix_1, turboPump.specificEnthalpy_outflow_a) annotation(
    Line(points = {{40, -154}, {-126, -154}, {-126, -80}, {-126, -80}}, color = {0, 0, 127}));
  connect(junction2.massFlowRate_1, turboPump.massFlowRate_a) annotation(
    Line(points = {{40, -156}, {-124, -156}, {-124, -80}, {-124, -80}}, color = {0, 0, 127}));
  connect(junction2.enthalpyFlowRate_1, turboPump.enthalpyFlowRate_a) annotation(
    Line(points = {{40, -158}, {-122, -158}, {-122, -80}, {-122, -80}}, color = {0, 0, 127}));
  connect(fuelMpr.y, engine.fuelMpr) annotation(
    Line(points = {{-118, 150}, {-108, 150}, {-108, 114}, {-100, 114}, {-100, 114}}, color = {0, 0, 127}));
  annotation(
    Icon(coordinateSystem(initialScale = 0.5)),
    Diagram(coordinateSystem(extent = {{-240, -180}, {240, 180}}, initialScale = 0.5)),
    uses(AICE(version = "0.1"), Modelica(version = "3.2.2")));
  annotation(  
    experiment(StartTime = 0, StopTime = 0.2, Tolerance = 1e-6, Interval = 4e-4),
    __OpenModelica_simulationFlags( lv = "LOG_STATS", initialStepSize = "0.5e-9", maxStepSize = "0.5e-6", outputFormat = "mat", s = "dassl", jacobian = "numerical", ls = "lis"));

end TestThermalManagement;
