within AICE.Fluid.Machines.BaseClasses;

model PartialTurboMachine "Compressor with Map - This model doesn't use the internal inertia. The flow is always positive (at this stage)"
  extends Modelica.Fluid.Interfaces.PartialTwoPort;
  extends AICE.Blocks.Mechanics.Rotational.Interfaces.Adaptors.BaseClasses.PartialAdaptor_up;
  extends BaseClasses.CommonParameters;
  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;

  parameter SI.Temperature T0_ref = 298.15 "Reference Temperature" annotation(
    Dialog(tab = "thermo", group = "reference"));
  parameter SI.Pressure P0_ref = 1.e5 "Reference Pressure" annotation(
    Dialog(tab = "thermo", group = "reference"));
  discrete SI.Temperature T_dyn;
  discrete SI.Pressure p_dyn;
  SI.Temperature T_tot_up(start=350.0);
  SI.Pressure p_up, p_tot_up;
  SI.Velocity v_in(start=1.0);
  SI.AngularVelocity w_omega "Shaft angular velocity";
  SI.MassFlowRate mFlow(start=0.1) "Compressor mass flow rate";
  SI.Torque tau_shaft "Turbine torque";
  Real pi_ratio "Pressure Ratio";
  Real T_up, d_up, H_up;
  Medium.ThermodynamicState state_up;

  Real eta_ad "Adiabatic Efficiency";
  Real w_map "Corrected or Reduced Angular Velocity";
  Real mflow_map "massflow value used for map-input";
  Real speed_map "rpm value used for map-input";
  Real gamma "Estimated heat capacity ratio";
  Real cp_up "upstream specific heat capacity";
  Real H_out "Estimated Ouput Enthalpy";
  Real delta_His;
  // delta_Hmec, phi_cor, psi_cor, ;
 

equation
// Turbine / Compressor SPECIFIC implementation/////////////////////
  if FluidMachineType == AICE.Fluid.Machines.Types.FluidMachine.Compressor then
    H_out = H_up + delta_His / max(0.1, eta_ad);
    delta_His = cp_up * (T_tot_up+273.15) * (pi_ratio ^ ((gamma - 1.0) / gamma) - 1.0); // ここと103行目とT_tot_upに273を2回足しているが，計算結果をみるとこれで正しい結果となる．（ここで273たさないとdelta Hisは負値を取ることがある）
    pi_ratio = port_b.p / p_tot_up;
  elseif FluidMachineType == AICE.Fluid.Machines.Types.FluidMachine.Turbine then
    H_out = H_up - delta_His * max(0.1, eta_ad);
    delta_His = cp_up * max(T_tot_up, 100) * (1.0 - (1/pi_ratio) ^ ((gamma - 1.0) / gamma)); // 不思議とタービン側は273足さなくともエキマニMedium温度とT_tot_upはケルビン単位で一致する
    pi_ratio = max((p_tot_up / port_b.p), 1.01);
    //pi_ratio = port_b.p / p_tot_up;
  end if;

//Get Modified variables
   speed_map = w_map * 60 / 2 / Modelica.Constants.pi;

// Corrected / Reduced SPECIFIC implementation/////////////////////
  if MapType == AICE.Fluid.Machines.Types.MapValue.Corrected then
    mflow_map = mFlow * sqrt(max(T_tot_up / T0_ref, 0.1)) * P0_ref / max(p_tot_up, 1.0);
    w_map = max(0.001, w_omega * sqrt(T0_ref / max(T_tot_up, 1.0)));
  
  elseif MapType == AICE.Fluid.Machines.Types.MapValue.Reduced then
    mflow_map = mFlow * sqrt(max(T_tot_up,10.0)) / max(p_tot_up/100000, 0.01); //Mapに合わせてpa->bar変換入り
    w_map = max(0.001, w_omega * sqrt(1.0 / max(T_tot_up, 10.0)));

  end if;

// COMMON implementation//////////////////////
// Mass balance
  port_a.m_flow = homotopy(actual=+mFlow, simplified=0.1);
  port_b.m_flow = homotopy(actual=-mFlow, simplified=-0.1);
// Species Trasnportation
  port_a.Xi_outflow = inStream(port_b.Xi_outflow);
  port_b.Xi_outflow = inStream(port_a.Xi_outflow);
  port_a.C_outflow = inStream(port_b.C_outflow);
  port_b.C_outflow = inStream(port_a.C_outflow);
// Energy balance
  H_up = port_a.h_outflow;
  port_b.h_outflow = H_out;
  port_a.h_outflow = inStream(port_a.h_outflow); //inStream(port_b.h_outflow) - delta_His / max(0.01, eta_ad);
//m flow逆流時はしない仕様だがエンタルピだけいちおう定義
//H_out = H_up + delta_His / eta_ad; moved to specific implementation
//delta_His = cp_up * T_tot_up * (pi_ratio ^ ((gamma - 1) / gamma) - 1); moved to specific implementation
//Torque balance
  flange_a.tau + tau_shaft = 0.0;
  tau_shaft = mFlow * (H_up - H_out) / max(w_omega, 1.e-1);
  w_omega = der(flange_a.phi);
//Get state variabhles
  state_up = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
//隣接するポートからPhXをもらい、その他の状態量求める
  T_up = Medium.temperature(state_up);
  p_up = Medium.pressure(state_up);
  d_up = Medium.density(state_up);
  cp_up = Medium.specificHeatCapacityCp(state_up);
  gamma = cp_up / Medium.specificHeatCapacityCv(state_up);
//pi_ratio = port_b.p / p_tot_up; moved to specific implementation
//dynamic pressure and temperature
  v_in = min(max(mFlow / d_up / (0.25 * dia_in ^ 2 * Modelica.Constants.pi), 0.1), 300);
//発散防止に流速ゼロを跨がない, mach1も超えない
  p_dyn = 0.5 * d_up * v_in ^ 2;
//p_tot_up = p_up + p_dyn;
//理論上は全圧が正しいが，発散するので静圧とす  p_tot_up = p_up + delay(p_dyn, 1E-4);
  p_tot_up = p_up;
//計算速度改善の為閉ループを切る
  T_dyn = 0.5 / cp_up * v_in ^ 2;
//T_tot_up = T_up + T_dyn;
//理論上は全温が正しいが，発散するので静温とす  T_tot_up = T_up + delay(T_dyn, 1E-4);
  T_tot_up = T_up + 273.15 ; //結果を確認した．なぜかT_upは単位Kだが代入しただけのT_tot_upはCになっている．
//計算速度改善の為閉ループを切る

  annotation(
    Icon(coordinateSystem(initialScale = 0.1)),
    experiment(StartTime = 0, StopTime = 1, Tolerance = 1e-6, Interval = 0.002),
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl"));
end PartialTurboMachine;