within AICE.Fluid.Machines;

model ElectricPump
  extends AICE.Icons.Fluid.EWaterPump;
  extends Modelica.Fluid.Interfaces.PartialTwoPort;
  extends AICE.Fluid.Machines.BaseClasses.PumpParameters;
  extends AICE.Electrical.BaseClasses.MotorParameters;
  replaceable package Medium = Modelica.Media.Water.StandardWater constrainedby Modelica.Media.Interfaces.PartialMedium;
Modelica.Electrical.Analog.Interfaces.PositivePin pin_ap
    "Positive armature pin" annotation (Placement(transformation(extent={{
            50,110},{70,90}})));
  Modelica.Electrical.Analog.Interfaces.NegativePin pin_an
    "Negative armature pin" annotation (Placement(transformation(extent={{-70,
            110},{-50,90}})));

  Modelica.Fluid.Machines.Pump pump(replaceable package Medium = Medium,
    redeclare function flowCharacteristic =
        Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.quadraticFlow (
          V_flow_nominal=V_flow_nominal, head_nominal=head_nominal),
   N_nominal = N_nominal,
    V(displayUnit="l") = 1e-8,
    energyDynamics=Modelica.Fluid.Types.Dynamics.FixedInitial,
    massDynamics=Modelica.Fluid.Types.Dynamics.FixedInitial,
    nParallel=1
    ) annotation(
    Placement(visible = true, transformation(origin = {0, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Electrical.BaseClasses.MotorStarter motor(k=k, BackVoltage_table=BackVoltage_table) annotation(
    Placement(visible = true, transformation(origin = {-54, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 1E-6, w(fixed = true, start = 100))  annotation(
    Placement(visible = true, transformation(origin = {-20, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Mechanics.Rotational.Components.OneWayClutch oneWayClutch(fn_max = 100)  annotation(
    Placement(visible = true, transformation(origin = {24, 50}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  Modelica.Mechanics.Rotational.Sources.ConstantSpeed speedMin(w_fixed = 1)  annotation(
    Placement(visible = true, transformation(origin = {68, 50}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant const(k = 1)  annotation(
    Placement(visible = true, transformation(origin = {4, 68}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(pin_an, motor.pin_an) annotation(
    Line(points = {{-60, 100}, {-60, 60}}, color = {0, 0, 255}));
  connect(pin_ap, motor.pin_ap) annotation(
    Line(points = {{60, 100}, {60, 80}, {-48, 80}, {-48, 60}}, color = {0, 0, 255}));
  connect(port_a, pump.port_a) annotation(
    Line(points = {{-100, 0}, {-10, 0}, {-10, 0}, {-10, 0}}));
  connect(port_b, pump.port_b) annotation(
    Line(points = {{100, 0}, {10, 0}, {10, 0}, {10, 0}}));
  connect(motor.flange_b, inertia.flange_a) annotation(
    Line(points = {{-44, 50}, {-30, 50}, {-30, 50}, {-30, 50}}));
  connect(inertia.flange_b, pump.shaft) annotation(
    Line(points = {{-10, 50}, {0, 50}, {0, 10}, {0, 10}}));
  connect(const.y, oneWayClutch.f_normalized) annotation(
    Line(points = {{16, 68}, {24, 68}, {24, 61}}, color = {0, 0, 127}));
  connect(speedMin.flange, oneWayClutch.flange_a) annotation(
    Line(points = {{58, 50}, {34, 50}, {34, 50}, {34, 50}}));
  connect(oneWayClutch.flange_b, inertia.flange_b) annotation(
    Line(points = {{14, 50}, {-10, 50}, {-10, 50}, {-10, 50}}));

annotation(
    experiment(StartTime = 0, StopTime = 1, Tolerance = 1e-6, Interval = 0.002));

end ElectricPump;
