within AICE.Fluid.Machines;

model TurboPump
  extends Icons.Fluid.Pump;
  extends BaseClasses.PartialPump;
  extends BaseClasses.TurboPumpParameters;

  Modelica.Fluid.Sources.MassFlowSource_T flowSource(
    redeclare replaceable package Medium = Medium,
    nPorts = 1,
    use_T_in = true,
    use_m_flow_in = true
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Fluid.Sources.FixedBoundary boundary(
    redeclare replaceable package Medium = Medium,
    nPorts = 1
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  SI.Temperature port_a_T = Medium.temperature_phX(
    port_a.p,
    actualStream(port_a.h_outflow),
    actualStream(port_a.Xi_outflow)
  );

  SI.MassFlowRate m_flow = rho * V_flow;

  SI.VolumeFlowRate V_flow(start=0);

  SI.Density rho = Modelica.Fluid.Utilities.regStep(
    m_flow,
    Medium.density_phX(
      port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow)
    ),
    Medium.density_phX(
      port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow)
    )
  );

  SI.AngularVelocity w = der(shaft.phi);

  Real gyrator;

equation

  der(gyrator) = (rho * w * r_impeller^2) - gyrator;

  der(V_flow) = (gyrator * w + port_a.p - port_b.p) / 1000.0;

  connect(port_a, boundary.ports[1]) annotation(
    Line(points = {{-100, 0}, {-20, 0}}, color = {0, 127, 255})
  );

  connect(flowSource.ports[1], port_b) annotation(
    Line(points = {{80, 0}, {100, 0}}, color = {0, 127, 255})
  );

  flowSource.m_flow_in = m_flow;
  flowSource.T_in = port_a_T;
  shaft.tau = gyrator * V_flow;

end TurboPump;
