within AICE.Fluid.Pipes.BaseClasses;

model CatalystConversionEfficiency_NoTracer "Multi-way connections of pipes with dynamic momentum balance, pressure wave and flow reversal"

  replaceable package Medium = AICE.Media.Interfaces.PartialCombustionMixtureMedium;
  extends AICE.Icons.Fluid.Catalyst;

  extends Modelica.Fluid.Interfaces.PartialTwoPort;//(replaceable package Medium = Medium);
  constant Integer nNodes = 4;
  parameter Real nParallel;
  parameter Modelica.SIunits.Length length, diameter;
  parameter Modelica.SIunits.Temperature T_start;
  parameter Modelica.SIunits.Pressure p_init;
  parameter Modelica.SIunits.MassFlowRate m_flow_start;
  parameter Real T_vs_eta_conv[:,:];
  parameter Modelica.SIunits.HeatCapacity HeatCapacity_core;
  parameter Modelica.SIunits.HeatCapacity HeatCapacity_can;
  parameter Modelica.SIunits.ThermalConductivity ThermalConductivity_core; //(unit="W/(m.K)")
  parameter Modelica.SIunits.ThermalConductance ThermalConductance_ins "Wrapping Insulator Thermal Conductance";
  parameter Modelica.SIunits.Area area_convection = diameter * nParallel ^ 0.5 * 5 * length "Can Surface Area for Convection";

  Modelica.SIunits.Temperature T_cores[nNodes];
  
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor CoreMaterial1(C = HeatCapacity_core / nNodes, T(fixed = true, start = T_start), der_T(fixed = false)) annotation(
    Placement(visible = true, transformation(origin = {-72, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor CoreMaterial2(C = HeatCapacity_core / nNodes, T(fixed = true, start = T_start), der_T(fixed = false)) annotation(
    Placement(visible = true, transformation(origin = {-18, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor CoreMaterial3(C = HeatCapacity_core / nNodes, T(fixed = true, start = T_start), der_T(fixed = false)) annotation(
    Placement(visible = true, transformation(origin = {22,20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor CoreMaterial4(C = HeatCapacity_core / nNodes, T(fixed = true, start = T_start), der_T(fixed = false)) annotation(
    Placement(visible = true, transformation(origin = {74, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor WallCondactance1(G = ThermalConductivity_core / (length / (nNodes - 1))) annotation(
    Placement(visible = true, transformation(origin = {-48, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G = ThermalConductivity_core / (length / (nNodes - 1))) annotation(
    Placement(visible = true, transformation(origin = {2, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor1(G = ThermalConductivity_core / (length / (nNodes - 1))) annotation(
    Placement(visible = true, transformation(origin = {52, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

  AICE.Fluid.Pipes.BaseClasses.DynamicReactionPipe_NoTracer FlowPath(
  redeclare package Medium=Medium,
  T_start = T_start, 
  diameter = diameter, 
  each eta_conv = {ConversionEfficiencyTable.y[1], ConversionEfficiencyTable.y[2], ConversionEfficiencyTable.y[3],ConversionEfficiencyTable.y[4]}, 
  length = length, m_flow_start = m_flow_start, 
  nNodes = nNodes, 
  nParallel = nParallel, 
  p_a_start = p_init, 
  p_b_start = p_init, 
  use_HeatTransfer = true, 
  use_T_start = true) annotation(
    Placement(visible = true, transformation(origin = {2, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor InsulatorWrap1(G = ThermalConductance_ins * nNodes) annotation(
    Placement(visible = true, transformation(origin = {-62, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor InsulatorWrap2(G = ThermalConductance_ins * nNodes) annotation(
    Placement(visible = true, transformation(origin = {-30, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor InsulatorWrap3(G = ThermalConductance_ins * nNodes) annotation(
    Placement(visible = true, transformation(origin = {32, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor InsulatorWrap4(G = ThermalConductance_ins * nNodes) annotation(
    Placement(visible = true, transformation(origin = {64, 46}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatport_can_a annotation(
    Placement(visible = true, transformation(origin = {-100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(extent = {{-110, 50}, {-90, 70}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b heatport_can_b annotation(
    Placement(visible = true, transformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(extent = {{110, 50}, {90, 70}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor Can(C = HeatCapacity_can, T(fixed = true, start = T_start), der_T(fixed = false)) annotation(
    Placement(visible = true, transformation(origin = {2, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Thermal.HeatTransfer.Components.ConvectiveResistor Convection annotation(
    Placement(visible = true, transformation(origin = {-20, 74}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatport_ambient annotation(
    Placement(visible = true, transformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Product area_x_htc annotation(
    Placement(visible = true, transformation(origin = {-50, 80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant area(k = area_convection)  annotation(
    Placement(visible = true, transformation(origin = {-82, 74}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.Blocks.Interfaces.RealInput htcInput annotation(
    Placement(visible = true, transformation(origin = {-60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90), iconTransformation(origin = {-60, 100}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
  Modelica.Blocks.Tables.CombiTable1D ConversionEfficiencyTable(columns = fill(2, nNodes), table = T_vs_eta_conv)  annotation(
    Placement(visible = true, transformation(origin = {0, -60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

 initial equation
  //FlowPath.mediums.p = fill(p_init, nNodes);
  //FlowPath.mediums.T = fill(T_start, nNodes);
  for i in 1:nNodes loop
    FlowPath.mediums[i].p = p_init;
    //FlowPath.mediums[i].T = T_start;
    FlowPath.mediums[i].Xi[:] = Medium.reference_X[:];
  end for;
  
algorithm
    T_cores[1] := CoreMaterial1.T;
    T_cores[2] := CoreMaterial2.T;
    T_cores[3] := CoreMaterial3.T;
    T_cores[4] := CoreMaterial4.T;
    ConversionEfficiencyTable.u := T_cores[:];

  
equation
  connect(CoreMaterial1.port, WallCondactance1.port_a) annotation(
    Line(points = {{-72, 10}, {-58, 10}}, color = {191, 0, 0}));
  connect(WallCondactance1.port_b, CoreMaterial2.port) annotation(
    Line(points = {{-38, 10}, {-18, 10}}, color = {191, 0, 0}));
  connect(CoreMaterial2.port, thermalConductor.port_a) annotation(
    Line(points = {{-18, 10}, {-8, 10}}, color = {191, 0, 0}));
  connect(CoreMaterial3.port, thermalConductor.port_b) annotation(
    Line(points = {{22, 10}, {12, 10}}, color = {191, 0, 0}));
  connect(CoreMaterial3.port, thermalConductor1.port_a) annotation(
    Line(points = {{22, 10}, {42, 10}}, color = {191, 0, 0}));
  connect(thermalConductor1.port_b, CoreMaterial4.port) annotation(
    Line(points = {{62, 10}, {74, 10}}, color = {191, 0, 0}));
  connect(FlowPath.heatPorts[4], CoreMaterial4.port) annotation(
    Line(points = {{2, -16}, {2, -6}, {74, -6}, {74, 10}}, color = {191, 0, 0}));
  connect(FlowPath.heatPorts[3], CoreMaterial3.port) annotation(
    Line(points = {{2, -16}, {2, -6}, {22, -6}, {22, 10}}, color = {191, 0, 0}));
  connect(FlowPath.heatPorts[1], CoreMaterial1.port) annotation(
    Line(points = {{2, -16}, {2, -6}, {-72, -6}, {-72, 10}}, color = {191, 0, 0}));
  connect(FlowPath.heatPorts[2], CoreMaterial2.port) annotation(
    Line(points = {{2, -16}, {2, -6}, {-18, -6}, {-18, 10}}, color = {191, 0, 0}));
  connect(CoreMaterial1.port, InsulatorWrap1.port_a) annotation(
    Line(points = {{-72, 10}, {-62, 10}, {-62, 36}}, color = {191, 0, 0}));
  connect(CoreMaterial2.port, InsulatorWrap2.port_a) annotation(
    Line(points = {{-18, 10}, {-30, 10}, {-30, 36}}, color = {191, 0, 0}));
  connect(CoreMaterial3.port, InsulatorWrap3.port_a) annotation(
    Line(points = {{22, 10}, {32, 10}, {32, 36}}, color = {191, 0, 0}));
  connect(CoreMaterial4.port, InsulatorWrap4.port_a) annotation(
    Line(points = {{74, 10}, {64, 10}, {64, 36}}, color = {191, 0, 0}));
  connect(Can.port, heatport_can_b) annotation(
    Line(points = {{2, 60}, {100, 60}}, color = {191, 0, 0}));
  connect(Can.port, heatport_can_a) annotation(
    Line(points = {{2, 60}, {-100, 60}}, color = {191, 0, 0}));
  connect(InsulatorWrap4.port_b, Can.port) annotation(
    Line(points = {{64, 56}, {64, 60}, {2, 60}}, color = {191, 0, 0}));
  connect(InsulatorWrap3.port_b, Can.port) annotation(
    Line(points = {{32, 56}, {31, 56}, {31, 60}, {2, 60}}, color = {191, 0, 0}));
  connect(InsulatorWrap2.port_b, Can.port) annotation(
    Line(points = {{-30, 56}, {-30, 60}, {2, 60}}, color = {191, 0, 0}));
  connect(InsulatorWrap1.port_b, Can.port) annotation(
    Line(points = {{-62, 56}, {-62, 60}, {2, 60}}, color = {191, 0, 0}));
  connect(Can.port, Convection.solid) annotation(
    Line(points = {{2, 60}, {-20, 60}, {-20, 64}}, color = {191, 0, 0}));
  connect(port_a, FlowPath.port_a) annotation(
    Line(points = {{-100, 0}, {-90, 0}, {-90, -20}, {-8, -20}}));
  connect(Convection.fluid, heatport_ambient) annotation(
    Line(points = {{-20, 84}, {-20, 92}, {0, 92}, {0, 100}}, color = {191, 0, 0}));
  connect(area_x_htc.y, Convection.Rc) annotation(
    Line(points = {{-38, 80}, {-36, 80}, {-36, 74}, {-30, 74}, {-30, 74}}, color = {0, 0, 127}));
  connect(area.y, area_x_htc.u2) annotation(
    Line(points = {{-70, 74}, {-64, 74}, {-64, 74}, {-62, 74}}, color = {0, 0, 127}));
  connect(FlowPath.port_b, port_b) annotation(
    Line(points = {{12, -20}, {86, -20}, {86, 0}, {100, 0}, {100, 0}}, color = {0, 127, 255}));
  connect(htcInput, area_x_htc.u1) annotation(
    Line(points = {{-60, 100}, {-68, 100}, {-68, 86}, {-62, 86}, {-62, 86}}, color = {0, 0, 127}));
  annotation(
    Documentation(info = "<html>
<p>
This model demonstrates the use of distributed pipe models with dynamic energy, mass and momentum balances.
At time=2s the pressure of boundary4 jumps, which causes a pressure wave and flow reversal.
</p>
<p>
Change system.momentumDynamics on the Assumptions tab of the system object from SteadyStateInitial to SteadyState,
in order to assume a steady-state momentum balance. This is the default for all models of the library.
</p>
<p>
Change the Medium from MoistAir to StandardWater, in order to investigate a medium with significantly different density.
Note the static head caused by the elevation of the pipes.
</p>
<p>
Note the appropriate use of the modelStructure of the DynamicPipe models (Advanced tab).
The default modelStructure is av_vb, i.e. volumes with a pressure state are exposed at both ports.
In many cases this gives good numerical performance, avoiding algebraic loops in connections,
e.g. if a pipe is connected to a valve or to a vessel with portsData configured.
The price to pay is a high-index DAE if two pipes are connected or if a pipe is connected to a boundary with prescribed pressure.
In such cases one might consider changing the modelStructure.
</p>
<p>
In the BranchingDynamicPipes example, {pipe1,pipe3,pipe4}.modelStructure are configured to a_v_b, while pipe2.modelStructure remains av_vb.
This avoids a high-index DAE and overdetermined initial conditions.
</p>
<img src=\"modelica://Modelica/Resources/Images/Fluid/Examples/BranchingDynamicPipes.png\" border=\"1\"
     alt=\"BranchingDynamicPipes.png\">
</html>"),
    experiment(StopTime = 3, StartTime = 0, Tolerance = 1e-6, Interval = 0.01),
    __Dymola_Commands(file(ensureSimulated = true) = "modelica://Modelica/Resources/Scripts/Dymola/Fluid/BranchingDynamicPipes/plotResults.mos" "plotResults"),
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl", maxStepSize = "1E-4", maxIntegrationOrder = "4"));
end CatalystConversionEfficiency_NoTracer;