within AICE.Fluid.Sources;

model AngleBasedInjectorAbyF

  import AICE.Math.AngleDifference4Stroke;
  import Modelica.Media.Interfaces.Choices.IndependentVariables;

  extends Icons.Fluid.Injector;
  extends Modelica.Fluid.Sources.BaseClasses.PartialFlowSource;

  AICE.Blocks.Interfaces.MassFlowRateInput intakeMassFlowRate
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 60}, extent = {{-20, -20}, {20, 20}}, rotation = 0),
      iconTransformation(origin = {-100, 60}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  Modelica.Blocks.Interfaces.RealInput phi_piston(final unit="rad")
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  parameter CarUnits.AirByFuelRatio airByFuel;
  parameter SI.Angle injectionStart "BTDC from compression";
  parameter SI.Angle injectionPeriod;

  SI.Mass intakeAirMassPerCycle(displayUnit = "g");
  SI.Mass fuelRemained(displayUnit = "g");
  SI.Mass fuelPerCycle(displayUnit = "g");
  Boolean injects;

  parameter SI.AbsolutePressure reference_p_vap = 1.e5
    "Reference pressure of vaporization";
  parameter SI.Temperature reference_T_vap = 293.15
    "Reference temperature of vaporization";
  parameter SI.SpecificEnthalpy reference_h_vap = 0.0
    "Reference specific enthalpy of vaporization";

  parameter SI.SpecificHeatCapacity Cp = 2000.0;
  parameter SI.Temperature reference_T_inj = 373.15
    "Reference temperature of Liquid Fuel in Rail";

  Medium.ThermodynamicState state_vap
    "Thermodynamics state of vaporized gas";

  SI.MassFlowRate fuelFlowRate = fuelFlowRate_internal;

protected

  SI.MassFlowRate fuelFlowRate_internal
    "Calcurated fuel flow rate";
  SI.SpecificEnthalpy h_internal
    "Calcurated Specific Enthalpy";

initial equation

  fuelPerCycle = 0.0;
  intakeAirMassPerCycle = 0.0;
  fuelRemained = 0.0;

equation

  der(fuelPerCycle) = 0;
  der(intakeAirMassPerCycle) = intakeMassFlowRate;
  fuelFlowRate_internal + der(fuelRemained) = 0;

  when ( AngleDifference4Stroke.compare(phi_piston, injectionStart) > 0 ) then
    reinit(
      intakeAirMassPerCycle,
      0.0
    );
    reinit(
      fuelPerCycle,
      pre(intakeAirMassPerCycle) / airByFuel
    );
    reinit(
      fuelRemained,
      pre(fuelRemained) + pre(intakeAirMassPerCycle) / airByFuel
    );
  end when;

  injects = (0 <= fuelRemained);
  if injects then
    fuelFlowRate_internal = fuelPerCycle / injectionPeriod * der(phi_piston);
  else
    fuelFlowRate_internal = 0;
  end if;

  state_vap = Medium.setState_pTX(
    reference_p_vap,
    reference_T_vap,
    Medium.reference_X_fuel
  );

  h_internal = (
    Medium.specificEnthalpy(state_vap)
    - reference_h_vap
    + Cp * (reference_T_inj - reference_T_vap)
  );

  Modelica.Fluid.Utilities.checkBoundary(Medium.mediumName, Medium.substanceNames,
    Medium.singleState, true, Medium.reference_X_fuel, "AngleBasedInjector");

  if Medium.ThermoStates == IndependentVariables.ph or
     Medium.ThermoStates == IndependentVariables.phX then
     medium.h = h_internal;
  else
     medium.T = Medium.temperature(Medium.setState_phX(medium.p, h_internal, Medium.reference_X_fuel));
  end if;
  sum(ports.m_flow) = -fuelFlowRate_internal;
  medium.Xi = Medium.reference_X_fuel[1:Medium.nXi];
  ports.C_outflow = fill(0.0, nPorts, Medium.nC);

end AngleBasedInjectorAbyF;
