within AICE.Fluid.Vessels.BaseClasses;

partial model ClosedVolumeParameters_hot
  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;
  outer Modelica.Fluid.System system;
  parameter SI.Volume V_hot "Volume_hot";
  // Assumptions
  parameter Modelica.Fluid.Types.Dynamics energyDynamics_hot = system.energyDynamics "Formulation of energy balance" annotation(
    Evaluate = true,
    Dialog(tab = "Assumptions_hot", group = "Dynamics"));
  parameter Modelica.Fluid.Types.Dynamics massDynamics_hot = system.massDynamics "Formulation of mass balance" annotation(
    Evaluate = true,
    Dialog(tab = "Assumptions_hot", group = "Dynamics"));
  // Initialization
  parameter Medium.AbsolutePressure p_hot_start = system.p_start "Start value of pressure" annotation(
    Dialog(tab = "Initialization_hot"));
  parameter Boolean use_T_hot_start = true "= true, use T_start, otherwise h_start" annotation(
    Dialog(tab = "Initialization_hot"),
    Evaluate = true);
  parameter Medium.Temperature T_hot_start = if use_T_hot_start then system.T_start else Medium.temperature_phX(p_hot_start, h_hot_start, X_hot_start) "Start value of temperature" annotation(
    Dialog(tab = "Initialization_hot", enable = use_T_start));
  parameter Medium.SpecificEnthalpy h_hot_start = if use_T_hot_start then Medium.specificEnthalpy_pTX(p_hot_start, T_hot_start, X_hot_start) else Medium.h_default "Start value of specific enthalpy" annotation(
    Dialog(tab = "Initialization_hot", enable = not use_T_hot_start));
  parameter Medium.MassFraction X_hot_start[Medium.nX] = Medium.X_default "Start value of mass fractions m_i/m" annotation(
    Dialog(tab = "Initialization_hot"));
  // ,
  // enable=Medium.nXi > 0
  // Port properties
  parameter Medium.MassFlowRate m_hot_flow_nominal = if system.use_eps_Re then system.m_flow_nominal else 1e2 * system.m_flow_small "Nominal value for mass flow rates in ports" annotation(
    Dialog(tab = "Advanced_hot", group = "Port properties"));
  parameter SI.MassFlowRate m_hot_flow_small(min = 0) = if system.use_eps_Re then system.eps_m_flow * m_hot_flow_nominal else system.m_flow_small "Regularization range at zero mass flow rate" annotation(
    Dialog(tab = "Advanced_hot", group = "Port properties"));
  parameter Boolean use_Re_hot = system.use_eps_Re "= true, if turbulent region is defined by Re, otherwise by m_flow_small" annotation(
    Dialog(tab = "Advanced_hot", group = "Port properties"),
    Evaluate = true);
end ClosedVolumeParameters_hot;