within AICE.Fluid.Vessels.BaseClasses;

model DualFlowHeatExchange
  extends Interfaces.DualFlowInterface;
  replaceable package Medium = Modelica.Media.Interfaces.PartialMedium;
  parameter String HxEfficiencyMap;
  Modelica.Media.Interfaces.PartialMixtureMedium.ThermodynamicState state_hot_in,  state_cool_in;
  Modelica.SIunits.MassFlowRate mflow_hot_1, mflow_hot_2, mflow_cool_1;
  Modelica.SIunits.HeatFlowRate q_hx;
  Modelica.SIunits.Temperature T_hot_1(min=200, max=400), T_hot_2_temp(min=200, max=400), T_hot_2(min=200, max=400),T_cool_1(min=200, max=400);
  //Modelica.SIunits.Temperature T_hot_1, T_hot_2_temp, T_hot_2,T_cool_1;
  Modelica.SIunits.SpecificEnthalpy h_hot_1, h_hot_2_temp;
  Modelica.SIunits.VolumeFlowRate vflow_hot, vflow_cool;
  Real eta_hx;
  Modelica.Blocks.Tables.CombiTable2D efficiencyTable(fileName = HxEfficiencyMap, tableName = "table1", tableOnFile = true) annotation(
    Placement(visible = true, transformation(origin = {-48, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

algorithm
  efficiencyTable.u1 := max(0, vflow_hot);
  efficiencyTable.u2 := max(0, vflow_cool);
  eta_hx := min(1.0, max(0.0, efficiencyTable.y));
  
equation
//（1）入り口状態取得
  state_hot_in = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
  state_cool_in = Medium.setState_phX(port_1.p, inStream(port_1.h_outflow), inStream(port_1.Xi_outflow));
  T_hot_1 = state_hot_in.T;
  T_cool_1 = state_cool_in.T;
  mflow_hot_1 = max(0, port_a.m_flow);
  mflow_hot_2 = min(-0.00000001, port_b.m_flow);
  mflow_cool_1 = max(0, port_1.m_flow);
  h_hot_1 = inStream(port_a.h_outflow);
//（2）入り口体積流量　被冷媒
  vflow_hot = mflow_hot_1 / Medium.density(state_hot_in);
//（3）入り口体積流量　冷媒
  vflow_cool = mflow_cool_1 / Medium.density(state_cool_in);
//(4)効率からHot側出口温度定常想定値T_hot_2_temp 決定
  eta_hx = (T_hot_1 - T_hot_2_temp) / max(T_hot_1 - T_cool_1, 0.1);
//(5) Hot側出口温度想定からエンタルピ想定に変換
  h_hot_2_temp = Medium.specificEnthalpy_pTX(port_b.p, T_hot_2_temp, port_b.Xi_outflow);
//(6)エンタルピデルタからq_hxを計算　最終的に使われる熱伝達量はq_hx
  q_hx = mflow_hot_1 * (h_hot_1 - h_hot_2_temp);
  T_hot_2 = Medium.temperature_phX(port_b.p,port_b.h_outflow,port_b.Xi_outflow);

  annotation(
    experiment(StartTime = 0, StopTime = 1, Tolerance = 1e-6, Interval = 0.002),
    __OpenModelica_commandLineOptions = "--matchingAlgorithm=PFPlusExt --indexReductionMethod=dynamicStateSelection -d=initialization,NLSanalyticJacobian,newInst",
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl"));
end DualFlowHeatExchange;