within AICE.Math.AngleDifference4Stroke;

operator record Difference

  import Modelica.Constants.pi;

public

  SI.Angle internalValue(min=-2*pi, max=+2*pi);
  // This is an internal value that can only be used to calculate the angle difference.
  // So, protected may be more correct.

public

  encapsulated type UnitKind = enumeration(rad, deg);

  encapsulated function fromAngle
  
    import Modelica;
    import Modelica.Constants.pi;
    import AICE.Math.AngleDifference4Stroke.Difference;
    import AICE.CarUnits.Conversions;
  
    extends Modelica.Icons.Function;
  
    input Real phi;
    input Difference.UnitKind unit = Difference.UnitKind.rad;
    output Difference result;
  
  protected
  
    Modelica.SIunits.Angle phi_rad;
  
  algorithm
  
    if unit == Difference.UnitKind.rad then
      phi_rad := phi;
    elseif unit == Difference.UnitKind.deg then
      phi_rad := Conversions.from_deg(phi);
    else // default
      phi_rad := phi;
    end if;
    result := Difference( mod( phi_rad + 2*pi, 4*pi ) - 2*pi );
    
    annotation(Inline = true);
  end fromAngle;

  // numeric operators
  encapsulated operator '+'
    function add
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Difference result;
    algorithm
      result := Difference.fromAngle(
        d1.internalValue + d2.internalValue
      );
      annotation(Inline=true);
    end add;
  end '+';

  encapsulated operator '-'
    function negate
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1;
      output Difference result;
    algorithm
      result := Difference.fromAngle(-d1.internalValue);
      annotation(Inline=true);
    end negate;

    function subtract
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Difference result;
    algorithm
      result := d1 + (-d2);
      annotation(Inline=true);
    end subtract;
  end '-';

  // relational operators
  encapsulated operator '=='
    function eq
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := tmp.internalValue == 0;
      annotation(Inline=true);
    end eq;
  end '==';

  encapsulated operator '<>'
    function ne
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := tmp.internalValue <> 0;
      annotation(Inline=true);
    end ne;
  end '<>';

  encapsulated operator '<'
    function lt
      import Modelica.Constants.pi;
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := (-2*pi < tmp.internalValue and tmp.internalValue < 0);
      annotation(Inline=true);
    end lt;
  end '<';

  encapsulated operator '<='
    function le
      import Modelica.Constants.pi;
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := (-2*pi < tmp.internalValue and tmp.internalValue <= 0);
      annotation(Inline=true);
    end le;
  end '<=';

  encapsulated operator '>'
    function gt
      import Modelica.Constants.pi;
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := (0 < tmp.internalValue and tmp.internalValue < 2*pi);
      annotation(Inline=true);
    end gt;
  end '>';

  encapsulated operator '>='
    function ge
      import Modelica.Constants.pi;
      import AICE.Math.AngleDifference4Stroke.Difference;
      input Difference d1, d2;
      output Boolean result;
    protected
      Difference tmp;
    algorithm
      tmp := d1 - d2;
      result := (0 <= tmp.internalValue and tmp.internalValue < 2*pi);
      annotation(Inline=true);
    end ge;
  end '>=';

end Difference;
