within AICE.Mechanics.Rotational.Components.BaseClasses;

model FrictionWithExteriorMAP "Friction output ω[rad/s], input τ[Nm]"
  extends Icons.Friction;
  extends PartialFriction;
  
  import AICE.CarUnits;
  import Modelica.SIunits;
  
  SIunits.Angle phi "Angle between shaft flanges (flange_a, flange_b) and support";
  SIunits.Torque tau "Friction torque";
  SIunits.AngularVelocity w "Absolute angular velocity of flange_a and flange_b";
  
  SIunits.AngularAcceleration a "Absolute angular acceleration of flange_a and flange_b";

  Modelica.Blocks.Interfaces.RealOutput abs_w annotation(
    Placement(visible = true, transformation(origin = {-70, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90), iconTransformation(origin = {-70, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
  Modelica.Blocks.Interfaces.RealInput abs_tau annotation(
    Placement(visible = true, transformation(origin = {70, 100}, extent = {{10, 10}, {-10, -10}}, rotation = 90), iconTransformation(origin = {70, 100}, extent = {{10, 10}, {-10, -10}}, rotation = 90)));
equation
  
  // NO static friction
  tau0 = 0;
  tau0_max = tau0;
  
  free = false;
  flange_a.phi = flange_b.phi;  
  phi = flange_a.phi - phi_support;
  w = der(phi);
  a = der(w);
  
  w_relfric = w;
  a_relfric = a;
  
  flange_a.tau + flange_b.tau - tau = 0;

  abs_w = abs(w);

  tau = if locked then sa*unitTorque else (
        if startForward then
          + abs_tau
        else if startBackward then
          - abs_tau
        else if pre(mode) == Forward then
          + abs_tau
        else
          - abs_tau
  );

  lossPower = w_relfric * tau;
end FrictionWithExteriorMAP;
