within AICE.Mechanics.Rotational.Components;

model CrankAndConnRod "Crankshaft and connecting rod displacement"

  import Modelica.Constants.pi;

  extends AICE.Icons.PowerUnits.PistonCrank;

  extends Modelica.Mechanics.Rotational.Interfaces.PartialElementaryRotationalToTranslational;

  parameter SI.Radius r_crank(displayUnit = "mm") = 0.046 "Crank radius";
  parameter SI.Position offset_crank(displayUnit = "mm") = 0.0 "Crank offset position";
  parameter SI.Length l_connRod(displayUnit = "mm") = 0.138 "Connecting rod length";

  parameter SI.Angle phi_crank_TDC = asin( offset_crank / (l_connRod + r_crank) );
  parameter SI.Angle phi_crank_BDC = asin( offset_crank / (l_connRod - r_crank) ) + pi;

  parameter SI.Length pistonStroke(displayUnit = "mm") = (
    (l_connRod + r_crank) * cos(phi_crank_TDC) + (l_connRod - r_crank) * cos(phi_crank_BDC)
  );

  SI.Angle phi_crank "Crank angle from negative x-axis direction (ccw)";
  SI.Angle phi_connRod "Connectiong rod angle from positive x-axis direction (ccw)";

equation

  phi_crank = flangeR.phi;

  // x axis condition
  // // The below equation is a modified version of the following equation
  // // to avoid 2nπ ambiguity in phi_connRod.
  // offset_crank = l_connRod * sin(phi_connRod) + r_crank * sin(phi_crank);
  phi_connRod = asin( ( offset_crank - r_crank * sin(phi_crank) ) / l_connRod );

  // y axis condition
  //   In `Modelica.Fluid.Machines.SweptVolume`, the volume increases as `flange.s` increases.
  //   `flangeT.s` of this model shall satisfy the following requirements.
  //     | flangeT.s == 0 (at TDC)
  //     | flangeT.s >  0 (at BDC)
  flangeT.s = -(
    (l_connRod * cos(phi_connRod) + r_crank * cos(phi_crank))  // current piston height
    - sqrt((l_connRod + r_crank)^2 - offset_crank^2)  // maximum piston height
  );

  // Energy conservation
  0 = der(flangeR.phi) * flangeR.tau + der(flangeT.s) * flangeT.f;

end CrankAndConnRod;
