within AICE.Mechanics.Rotational.Components;

model FrictionByNeMap

  import Modelica.SIunits;

  extends BaseClasses.PartialFriction;
  extends Icons.Friction;
  
  parameter String filename "friction table path Ne[rpm] v.s. torque[N.m]";
  
  AICE.Mechanics.Rotational.Components.BaseClasses.FrictionWithExteriorMAP friction(
    free = free,
    tau0 = tau0,
    tau0_max = tau0_max,
    sa = sa,
    w_relfric = w_relfric,
    lossPower = lossPower
  ) annotation(
    Placement(visible = true, transformation(origin = {0, 0}, extent = {{-40, -40}, {40, 40}}, rotation = 0)));
  
  Modelica.Blocks.Tables.CombiTable1Ds combiTable1Ds(
    fileName = filename,
    smoothness = Modelica.Blocks.Types.Smoothness.ContinuousDerivative,
    tableName = "Ne_torque",
    tableOnFile = true
  ) annotation(
    Placement(visible = true, transformation(origin = {0, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    
  SIunits.Torque tau;

equation
  connect(flange_a, friction.flange_a) annotation(
    Line(points = {{-100, 0}, {-40, 0}}));
  connect(friction.flange_b, flange_b) annotation(
    Line(points = {{40, 0}, {100, 0}}));
  connect(friction.support, support) annotation(
    Line(points = {{0, -40}, {0, -40}, {0, -100}}));
  connect(friction.heatPort, heatPort) annotation(
    Line(points = {{-40, -40}, {-100, -40}, {-100, -100}}, color = {191, 0, 0}));
  combiTable1Ds.u = CarUnits.Conversions.to_rpm(friction.abs_w);
  connect(combiTable1Ds.y[1], friction.abs_tau) annotation(
    Line(points = {{12, 70}, {28, 70}, {28, 40}}, color = {0, 0, 127}));
  tau = friction.tau;
end FrictionByNeMap;
