within AICE.Mechanics.Rotational.Components;

model PhaseDelays

  parameter Integer nFlanges_b "Number Of flanges_b";
  parameter SI.Angle delays[nFlanges_b] = fill(0, nFlanges_b)
    "Delay of each flanges_b angle relative to flange_a";

  Interfaces.Flange_a flange_a
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Interfaces.Flanges_b flanges_b[nFlanges_b] annotation(
    Placement(
      visible = true,
      transformation(origin = {100, 0}, extent = {{-10, -40}, {10, 40}}, rotation = 0),
      iconTransformation(origin = {100, 0}, extent = {{-10, -40}, {10, 40}}, rotation = 0)
    )
  );

equation

  // torque balance
  0 = flange_a.tau + sum(flanges_b.tau);

  // phase delay
  for iFlange in 1:nFlanges_b loop
    flange_a.phi - delays[iFlange] = flanges_b[iFlange].phi;
  end for;

  annotation(
    Icon(
      graphics={
        Line(
          points={{-100,0},{100,  0}}
        ),
        Line(
          points={{-100,0},{100,+26}}
        ),
        Line(
          points={{-100,0},{100,-26}}
        ),
        Text(
          extent={{-150,100},{150,60}},
          lineColor={0,0,255},
          textString="%name"
        )
      }
    )
  );

end PhaseDelays;
