within AICE.Mechanics.Rotational.Components.Tests.Gears;

model TestMultipleIdealGear
  extends Modelica.Icons.Example;

  // Sources
  Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, -10}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  AICE.Mechanics.Rotational.Components.Tests.Gears.TorqueCurve torqueCurve(
    peakTorque = 200, w_peak = 523.599
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-80, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 90)
    )
  );

  Modelica.Mechanics.Rotational.Sources.Torque torque
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  // Test target
  AICE.Mechanics.Rotational.Components.MultipleIdealGear multipleIdealGear(
    nPosition = 3,
    ratios = {1.2, 1.0, 0.8}
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  // Gear position
  parameter SI.Time span = 1;

  Integer gearPosition = integer(
    mod(time/span, multipleIdealGear.nPosition) + 1
  );

  // Loads
  Modelica.Mechanics.Rotational.Components.Inertia inertia(
    J = 0.05,
    w(displayUnit="rpm", fixed = true, start = 10.472)
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {20, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Components.Damper damper(
    d(displayUnit = "N.m/rpm") = 0.381972
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {50, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Components.Fixed fixed
  annotation(
    Placement(
      visible = true,
      transformation(origin = {80, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );


  // Monitor
  CarUnits.AngularVelocity w_monitor = inertia.w;

equation

  multipleIdealGear.position = gearPosition;

  // Sources
  connect(speedSensor.flange, multipleIdealGear.flange_a)
  annotation(Line(points = {{-50, 0}, {-50, 10}, {-20, 10}}));

  connect(speedSensor.w, torqueCurve.u)
  annotation(Line(points = {{-50, -20}, {-50, -30}, {-80, -30}, {-80, 0}}, color = {0, 0, 127}));

  connect(torqueCurve.y, torque.tau)
  annotation(Line(points = {{-80, 20}, {-80, 50}, {-50, 50}, {-50, 40}}, color = {0, 0, 127}));

  connect(torque.flange, multipleIdealGear.flange_a)
  annotation(Line(points = {{-50, 20}, {-50, 10}, {-20, 10}}));

  // Loads
  connect(multipleIdealGear.flange_b, inertia.flange_a)
  annotation(Line(points = {{0, 10}, {10, 10}}));

  connect(inertia.flange_b, damper.flange_a)
  annotation(Line(points = {{30, 10}, {40, 10}}));

  connect(damper.flange_b, fixed.flange)
  annotation(Line(points = {{60, 10}, {80, 10}, {80, -30}}));

annotation(
  experiment(
    StartTime =  0.0,
    StopTime  = 12.0
  )
);

end TestMultipleIdealGear;