within AICE.PowerUnits.Combustions.Assemblies;

partial model PartialMultiCylinderEngine
  extends Interfaces.PartialFluidEngine;

  parameter Integer nCylinders = 1;

  replaceable Interfaces.PartialFluidEngine[nCylinders] singleCylinderEngines(
    redeclare each replaceable package Medium = Medium
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  replaceable Interfaces.PartialManifold intakeManifold(
    redeclare replaceable package Medium = Medium,
    final nCylinders = nCylinders
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-60, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  replaceable Interfaces.PartialManifold exhaustManifold(
    redeclare replaceable package Medium = Medium,
     final nCylinders = nCylinders
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {60, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  replaceable Interfaces.PartialCrankShaft crankShaft(
    final nCylinders = nCylinders
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {70, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

equation

  connect(port_intake, intakeManifold.port_a) annotation(
    Line(points = {{-60, 100}, {-60, 80}}, color = {0, 127, 255})
  );
  connect(port_exhaust, exhaustManifold.port_a) annotation(
    Line(points = {{+60, 100}, {+60, 80}}, color = {0, 127, 255})
  );
  connect(intakeManifold.ports_b[:], singleCylinderEngines[:].port_intake) annotation(
    Line(
      points = {{-60, 60}, {-60, 40}, {-12, 40}, {-12, 20}},
      color = {0, 127, 255}, thickness = 0.5
    )
  );
  connect(exhaustManifold.ports_b[:], singleCylinderEngines[:].port_exhaust) annotation(
    Line(
      points = {{+60, 60}, {+60, 40}, {+12, 40}, {+12, 20}},
      color = {0, 127, 255}, thickness = 0.5
    )
  );

  connect(singleCylinderEngines[:].flange, crankShaft.flanges_b[:]) annotation(
    Line(points = {{20, 0}, {60, 0}}, thickness = 0.5)
  );
  connect(flange, crankShaft.flange_a) annotation(
    Line(points = {{100, 0}, {80, 0}})
  );

end PartialMultiCylinderEngine;
