within AICE.PowerUnits.Combustions.Cams;

model CamProfileFromFile
  import Modelica.Constants.pi;

  extends Icons.PowerUnits.Cam;
  extends BaseClasses.PartialCam;

  type TableRange = enumeration(
    To720        "[   0属, +720属)",
    PlusMinus360 "[-360属, +360属)"
  );

  parameter TableRange tableRange = TableRange.PlusMinus360;

  parameter SI.Angle phi_centor = 0 "ATDC from exhaust";

  parameter String fileName_lift
    "Table file name of the relationship" +
    " between cam angle and lift"
  annotation (
    Dialog(
      group="Table data definition",
      loadSelector(
        filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
        caption="Open file in which lift table is present"
      )
    )
  );

  parameter String fileName_CD
    "Table file name of the relationship" +
    " between lift and CD (discharge coefficient)"
  annotation (
    Dialog(
      group="Table data definition",
      loadSelector(
        filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
        caption="Open file in which CD table is present"
      )
    )
  );

  Modelica.Blocks.Interfaces.BooleanOutput opens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Math.UnitConversions.To_deg to_deg
  annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Tables.CombiTable1D angle2lift(
    fileName = fileName_lift, tableName = "lift", tableOnFile = true
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {20, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Tables.CombiTable1D lift2cd(
    fileName = fileName_CD, tableName = "cd", tableOnFile = true
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Logical.Greater greater
  annotation(
    Placement(
      visible = true,
      transformation(origin = {30, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Sources.Constant opening_min(
    k = 1e-6
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  SI.Angle phi_fromCentor = (
    angleSensor.phi - (2*pi + phi_centor)
  );

equation

  connect(flange, angleSensor.flange) annotation(
    Line(points = {{-100, 0}, {-80, 0}})
  );

  if tableRange == TableRange.To720 then
    to_deg.u = mod(phi_fromCentor       , 4*pi);
  elseif tableRange == TableRange.PlusMinus360 then
    to_deg.u = mod(phi_fromCentor + 2*pi, 4*pi) - 2*pi;
  end if;

  connect(to_deg.y, angle2lift.u[1]) annotation(
    Line(points = {{0, 0}, {10, 0}}, color = {0, 0, 127})
  );
  connect(angle2lift.y[1], lift2cd.u[1]) annotation(
    Line(points = {{30, 0}, {40, 0}}, color = {0, 0, 127})
  );
  connect(lift2cd.y[1], opening) annotation(
    Line(points = {{60, 0}, {100, 0}}, color = {0, 0, 127})
  );

  connect(lift2cd.y[1], greater.u1) annotation(
    Line(points = {{60, 0}, {80, 0}, {80, 80}, {10, 80}, {10, 60}, {20, 60}}, color = {0, 0, 127})
  );
  connect(opening_min.y, greater.u2) annotation(
    Line(points = {{0, 52}, {20, 52}}, color = {0, 0, 127})
  );

  connect(greater.y, opens) annotation(
    Line(points = {{40, 60}, {60, 60}}, color = {255, 0, 255})
  );

end CamProfileFromFile;
