within AICE.PowerUnits.Combustions.Cams;

model SimpleGenericCam
  extends Icons.PowerUnits.Cam;
  extends BaseClasses.PartialCam;

  type FourStrokeTiming = SI.Angle(
    min = 0,
    max = 4*Modelica.Constants.pi
  );

  parameter FourStrokeTiming phi_open "ATDC from compression";
  parameter FourStrokeTiming phi_lobe = (phi_open + phi_close) / 2 "ATDC from compression";
  parameter FourStrokeTiming phi_close "ATDC from compression";

  Modelica.Blocks.Interfaces.BooleanOutput opens
  annotation(
    Placement(
      visible = true,
      transformation(origin = {60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0),
      iconTransformation(origin = {60, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  parameter SI.Angle openingPeriod = AICE.Math.AngleDifference4Stroke.compare(
    phi_lobe, phi_open
  );

  parameter SI.Angle closingPeriod = AICE.Math.AngleDifference4Stroke.compare(
    phi_close, phi_lobe
  );

  SI.Angle phi_from_open = AICE.Math.AngleDifference4Stroke.compare(
    flange.phi, phi_open
  );

  SI.Angle phi_from_lobe = AICE.Math.AngleDifference4Stroke.compare(
    flange.phi, phi_lobe
  );

  SI.Angle phi_from_close = AICE.Math.AngleDifference4Stroke.compare(
    flange.phi, phi_close
  );

  function cubicOpening
    extends Modelica.Icons.Function;
    input Real u;
    output Real y;
  algorithm
    y := 3*u^2 - 2*u^3;
  end cubicOpening;

  constant Real opening_min = 1e-6;

equation

  assert(
    Modelica.Constants.eps < openingPeriod,
    "phi_open < phi_lobe"
  );

  assert(
    Modelica.Constants.eps < closingPeriod,
    "phi_lobe < phi_close"
  );

  if (0 < phi_from_open and phi_from_lobe <= 0) then
    opening =     cubicOpening( phi_from_open / openingPeriod );
  elseif (0 < phi_from_lobe and phi_from_close <= 0) then
    opening = 1 - cubicOpening( phi_from_lobe / closingPeriod );
  else
    opening = 0.0;
  end if;

  opens = opening > opening_min;

  flange.tau = 0.0;

end SimpleGenericCam;
