within AICE.PowerUnits.Combustions.Chambers.Examples;

model TestWiebeWoschniChamber2zones
  extends Modelica.Icons.Example;

  inner Modelica.Fluid.System system(
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {-70, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  parameter CarUnits.AirByFuelRatio airByFuel = 15.0;

  final parameter SI.MassFraction[Medium.nX] X_mixture = (
    airByFuel * Medium.reference_X     [:] .+
    1         * Medium.reference_X_fuel[:]
  ) / (airByFuel+1);

  replaceable package Medium = AICE.Media.IdealGases.Examples.Sample5MixtureGasStoichComb;
  //replaceable package Medium = AICE.Media.IdealGases.Examples.Sample5MixtureGas;

  // Fluid components
  Modelica.Fluid.Sources.FixedBoundary mixtureBoundary(
    redeclare replaceable package Medium = Medium,
    nPorts = 1,
    X = X_mixture
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-70, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Fluid.Sources.FixedBoundary exhaustBoundary(
    redeclare replaceable package Medium = Medium,
    nPorts = 1
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {70, 30}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  AICE.PowerUnits.Combustions.Cams.SimpleGenericCam intakeCam(
    phi_open  =  6.45772,
    phi_close = 10.9956
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  AICE.PowerUnits.Combustions.Cams.SimpleGenericCam exhaustCam(
    phi_open  =  1.5708,
    phi_close =  6.28319
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {30, 60}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  Modelica.Fluid.Valves.ValveLinear intakeValve(
    redeclare replaceable package Medium = Medium,
    dp_nominal = 100000,
    m_flow_nominal = 0.15
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Fluid.Valves.ValveLinear exhaustValve(
    redeclare replaceable package Medium = Medium,
    dp_nominal = 100000,
    m_flow_nominal = 0.1
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {30, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );


  AICE.PowerUnits.Combustions.Chambers.WiebeWoschniChamber2zones chamber(
    redeclare replaceable package Medium = Medium, 
    T_start = 1269,
    V_displacement = 500e-6,
    X_start = X_mixture,
    bore = 0.086,
    c1_woschni_compressionPower = 2.28,
    c1_woschni_intakeExhaust    = 6.18,
    c2_woschni = 0.00324,
    compressionRatio = 12,
    m_wiebe = 1,
    nPorts = 2,
    p_start = 32.06e+5,
    phi_combustion_begin = 0.0698132,
    phi_combustion_period = 0.523599,
    use_HeatTransfer = true,
    use_exhaustValveOpens = true,
    use_intakeValveOpens = true,
    use_phi_piston = true,
    use_portsData = false
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {0, -10}, extent = {{-10, 10}, {10, -10}}, rotation = 0)
    )
  );

  // Rotational components
  AICE.Mechanics.Rotational.Components.CrankAndConnRod crankAndConnRod(
    l_connRod = 138e-3,
    offset_crank = 0,
    r_crank = chamber.pistonStroke / 2
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {0, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 90)
    )
  );

  Modelica.Mechanics.Rotational.Sources.ConstantSpeed constantSpeed(
    w_fixed(displayUnit = "rev/min") = 314.159
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {-30, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor
  annotation(
    Placement(
      visible = true,
      transformation(origin = {40, -18}, extent = {{5, -5}, {-5, 5}}, rotation = 0)
    )
  );

  // Heat transfer components
  Modelica.Thermal.HeatTransfer.Components.HeatCapacitor heatCapacitor(
    C = 100,
    T(displayUnit = "K", fixed = true, start = 550)
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {-50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(
    G = 15
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {-70, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 90)
    )
  );

  Modelica.Thermal.HeatTransfer.Sources.FixedTemperature fixedTemperature(
    T = 293.15
  ) annotation (
    Placement(
      visible = true,
      transformation(origin = {-70, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 90)
    )
  );

equation

  // Fluid connections
  connect(mixtureBoundary.ports[1], intakeValve.port_a) annotation(
    Line(points = {{-60, 30}, {-40, 30}}, color = {0, 127, 255})
  );

  connect(intakeValve.port_b, chamber.ports[1]) annotation(
    Line(points = {{-20, 30}, {-2, 30}, {-2, 0}}, color = {0, 127, 255})
  );

  connect(exhaustBoundary.ports[1], exhaustValve.port_b) annotation(
    Line(points = {{60, 30}, {40, 30}}, color = {0, 127, 255})
  );

  connect(exhaustValve.port_a, chamber.ports[2]) annotation(
    Line(points = {{ 20, 30}, { 2, 30}, { 2, 0}}, color = {0, 127, 255})
  );


  // Rotational connections
  connect(constantSpeed.flange, crankAndConnRod.flangeR) annotation(
    Line(points = {{-20, -70}, {0, -70}, {0, -60}})
  );

  connect(crankAndConnRod.flangeT, chamber.flange) annotation(
    Line(points = {{0, -40}, {0, -20}}, color = {0, 127, 0})
  );


  // Heat transfer connections
  connect(thermalConductor.port_a, fixedTemperature.port) annotation(
    Line(points = {{-70, -40}, {-70, -60}}, color = {191, 0, 0})
  );
  connect(heatCapacitor.port, thermalConductor.port_b) annotation(
    Line(points = {{-50, -10}, {-70, -10}, {-70, -20}}, color = {191, 0, 0})
  );
  connect(chamber.heatPort, heatCapacitor.port) annotation(
    Line(points = {{-10, -10}, {-50, -10}}, color = {191, 0, 0})
  );


  // Cam and timing connections
  connect(constantSpeed.flange, angleSensor.flange)
  annotation(
    Line(points = {{-20, -70}, {50, -70}, {50, -18}, {45, -18}})
  );

  connect(angleSensor.phi, chamber.phi_piston)
  annotation(
    Line(
      points = {{35, -18}, {10, -18}},
      color = {0, 0, 127}
    )
  );

  connect(constantSpeed.flange, intakeCam.flange) annotation(
    Line(points = {{-20, -70}, {50, -70}, {50, 80}, {-30, 80}, {-30, 70}})
  );

  connect(constantSpeed.flange, exhaustCam.flange) annotation(
    Line(points = {{-20, -70}, {50, -70}, {50, 80}, { 30, 80}, { 30, 70}})
  );

  connect(intakeCam.opening, intakeValve.opening) annotation(
    Line(points = {{-30, 50}, {-30, 40}}, color = {0, 0, 127})
  );

  connect(exhaustCam.opening, exhaustValve.opening) annotation(
    Line(points = {{ 30, 50}, { 30, 40}}, color = {0, 0, 127})
  );

  connect(intakeCam.opens, chamber.intakeValveOpens)
  annotation(
    Line(
      points = {{-24, 54}, {-24, 45}, {15, 45}, {15, -16}, {10, -16}},
      color = {255, 0, 255}
    )
  );

  connect(exhaustCam.opens, chamber.exhaustValveOpens)
  annotation(
    Line(
      points = {{ 36, 54}, { 36, 45}, {15, 45}, {15, -14}, {10, -14}},
      color = {255, 0, 255}
    )
  );

annotation(
    experiment(StartTime = 0, StopTime = 0.09, Tolerance = 1e-6, Interval = 0.000180723),
    __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl", initialStepSize = "1E-6", maxStepSize = "1E-6"));
end TestWiebeWoschniChamber2zones;