within AICE.PowerUnits.Combustions.Examples.BaseClasses;

partial model SimpleEngineUnitParameters

  constant Integer nCylinders = 4;



  // # boundary condition parameters

  // ## supercharged boundary
  parameter SI.AbsolutePressure
    p_supercharged "Average of supercharged air pressure"
    annotation(
      Dialog(tab="Boundaries", group="Pressures")
    );

  parameter SI.Temperature
    T_supercharged "Average of supercharged air temperature"
    annotation(
      Dialog(tab="Boundaries", group="Temperatures")
    );

  // ## exhaust boundary
  parameter SI.AbsolutePressure
    p_exhaust "Average of exhaust gas pressure"
    annotation(
      Dialog(tab="Boundaries", group="Pressures")
  );

  parameter SI.Temperature
    T_exhaust "Average of exhaust gas temperature"
    annotation(
      Dialog(tab="Boundaries", group="Temperatures")
    );

  // ## engine room boundary
  parameter SI.Temperature
    T_engineRoom "Average of engine room temperature"
    annotation(
      Dialog(tab="Boundaries", group="Temperatures")
    );



  // # geometry parameters

  // ## total displacement volume
  parameter CarUnits.Volume
    V_displacement "Total displacement volume"
    annotation(
      Dialog(group="Geometries")
    );

  // ## Compression ratio (CR)
  parameter CarUnits.CompressionRatio
    compressionRatio "Compression ratio V_BDC/V_TDC"
    annotation(
      Dialog(group="Geometries")
    );

  // ## intake pipe (pre-throttle)
  parameter CarUnits.Volume
    V_intakePipe "Volume of intake pipe"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Diameter
    d_intakePipe "Diameter of intake pipe"
    annotation(
      Dialog(group="Geometries")
    );

  // ## throttle
  parameter CarUnits.Diameter
    d_throttle "Diameter of throttle"
    annotation(
      Dialog(group="Geometries")
    );

  // ## intake pipe (post-throttle)
  parameter CarUnits.Volume
    V_intakeManifold "Volume of intake manifold"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Diameter
    d_valve_intake "Diameter of intake valve"
    annotation(
      Dialog(group="Geometries")
    );

  // ## exhaust pipe
  parameter CarUnits.Diameter
    d_valve_exhaust "Diameter of exhaust valve"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Volume
    V_exhaustManifold "Volume of exhaust manifold"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Diameter
    d_exhaustPipe "Diameter of exhaust pipe"
    annotation(
      Dialog(group="Geometries")
    );

  // ## crank & chamber
  parameter CarUnits.Length
    l_connRod "Length of connecting rod"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Position
    offset_crank "Crank offset from piston axis"
    annotation(
      Dialog(group="Geometries")
    );

  parameter CarUnits.Diameter
    bore "Diameter of cylinder"
    annotation(
      Dialog(group="Geometries")
    );



  // # start value parameters

  // ## intake pipe (pre-throttle)
  parameter SI.AbsolutePressure
    p_start_intakePipe "Start value of pressure at intake pipe"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_intakePipe "Start value of temperature at intake pipe"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  // ## intake pipe (post-throttle)
  parameter SI.AbsolutePressure
    p_start_intakeManifold "Start value of pressure at intake manifold"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_intakeManifold "Start value of temperature at intake manifold"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  // ## chambers
  parameter SI.AbsolutePressure
    p_start_chamber1 "Start value of pressure at 1st chamber"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_chamber1 "Start value of temperature at 1st chamber"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  parameter SI.AbsolutePressure
    p_start_chamber2 "Start value of pressure at 2nd chamber"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_chamber2 "Start value of temperature at 2nd chamber"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  parameter SI.AbsolutePressure
    p_start_chamber3 "Start value of pressure at 3rd chamber"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_chamber3 "Start value of temperature at 3rd chamber"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  parameter SI.AbsolutePressure
    p_start_chamber4 "Start value of pressure at 4th chamber"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_chamber4 "Start value of temperature at 4th chamber"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );

  // ## exhaust pipe
  parameter SI.AbsolutePressure
    p_start_exhaustManifold "Start value of pressure at exhaust manifold"
    annotation(
      Dialog(tab="Initialization", group="Pressures")
    );

  parameter SI.Temperature
    T_start_exhaustManifold "Start value of temperature at exhaust manifold"
    annotation(
      Dialog(tab="Initialization", group="Temperatures")
    );



  // # table data path patameters
  parameter String
    fileName_lift_intake "File where intake valve lift table is stored"
    annotation (
      Dialog(
        group="Table data definition",
        loadSelector(
          filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
          caption="Open file in which table is present"
        )
      )
    );

  parameter String
    fileName_cd_intake "File where intake valve cd table is stored"
    annotation (
      Dialog(
        group="Table data definition",
        loadSelector(
          filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
          caption="Open file in which table is present"
        )
      )
    );

  parameter String
    fileName_lift_exhaust "File where exhaust valve lift table is stored"
    annotation (
      Dialog(
        group="Table data definition",
        loadSelector(
          filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
          caption="Open file in which table is present"
        )
      )
    );

  parameter String
    fileName_cd_exhaust "File where exhaust valve cd table is stored"
    annotation (
      Dialog(
        group="Table data definition",
        loadSelector(
          filter="Text files (*.txt);;MATLAB MAT-files (*.mat)",
          caption="Open file in which table is present"
        )
      )
    );



  // # other parameters
  parameter CarUnits.Mass
    fuelPerCycle "Fuel consumption during one revolution of the crankshaft";

  parameter SI.Angle
    injectionStart "BTDC from compression";

  parameter SI.Angle
    injectionPeriod "Angle between start and stop of injection";

  parameter SI.Temperature
    reference_T_vap "Reference temperature of fuel vapotization";

  parameter Real m_wiebe;

  parameter SI.Angle
    phi_combustion_begin
    "BTDC from piston compression TDC";

  parameter SI.Angle
    phi_combustion_period
    "Anglar period until 1-exp(-a_wiebe) of fuel burns";

  parameter Real c1_woschni_intakeExhaust;
  parameter Real c1_woschni_compressionPower;
  parameter Real c2_woschni;

end SimpleEngineUnitParameters;
