within AICE.PowerUnits.Combustions.Examples;

model SimpleEngineUnit
  extends PowerUnits.Interfaces.PartialPowerUnit;
  extends BaseClasses.SimpleEngineUnitDefaultValues1;

  extends Icons.PowerUnits.EngineUnit;

protected

  parameter SI.AbsolutePressure[nCylinders]
    p_start_chambers = {
      p_start_chamber1,
      p_start_chamber2,
      p_start_chamber3,
      p_start_chamber4
    };

  parameter SI.Temperature[nCylinders]
    T_start_chambers = {
      T_start_chamber1,
      T_start_chamber2,
      T_start_chamber3,
      T_start_chamber4
    };

  final parameter CarUnits.Volume
    V_displacement_cylinder = V_displacement / nCylinders;

  final parameter CarUnits.Mass
    fuelPerCycle_cylinder = fuelPerCycle * 2 / nCylinders;

public

  Modelica.Blocks.Interfaces.RealInput phi_throttleOpening(
    unit="rad", displayUnit="deg"
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0),
      iconTransformation(origin = {-100, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  replaceable package Medium = AICE.Media.IdealGases.Examples.Sample5MixtureGas(
    final combustionLHV = {5020343.454} // [J/mol]
  );

  inner Modelica.Fluid.System system annotation(
    Placement(
      visible = true,
      transformation(origin = {70, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Fluid.Sources.FixedBoundary superchargedBoundary(
    redeclare replaceable package Medium = Medium,
    nPorts = 1,
    use_p = true, p = p_supercharged,
    use_T = true, T = T_supercharged
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-70, 30}, extent = {{-5, -5}, {5, 5}}, rotation = 0)
    )
  );

  Modelica.Fluid.Sources.FixedBoundary exhaustBoundary(
    redeclare replaceable package Medium = Medium,
    nPorts = 1,
    use_p, p = p_exhaust,
    use_T, T = T_exhaust
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {+70, 30}, extent = {{5, -5}, {-5, 5}}, rotation = 0)
    )
  );

  AICE.Fluid.Fittings.Orifices.FixedOrifice intakePressureDrop(
    redeclare replaceable package Medium = Medium,
    diameter = d_intakePipe
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, 30}, extent = {{-5, -5}, {5, 5}}, rotation = 0)
    )
  );

  Modelica.Fluid.Vessels.ClosedVolume intakeVolume(
    redeclare replaceable package Medium = Medium,
    nPorts = 2,
    use_portsData = false,
    V = V_intakePipe,
    p_start = p_start_intakePipe,
    use_T_start = true, T_start = T_start_intakePipe
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-40, 40}, extent = {{-5, -5}, {5, 5}}, rotation = 0)
    )
  );

  Fluid.Valves.ButterflyValve throttle(
    redeclare replaceable package Medium = Medium,
    diameter = d_throttle
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 30}, extent = {{-5, 5}, {5, -5}}, rotation = 0)
    )
  );

  AICE.PowerUnits.Combustions.Examples.SimpleInline4CylinderSIEngine multiCylinderEngine(
    redeclare replaceable package Medium = Medium,
    intakeManifold(
      use_T_start = true,
      p_start = p_start_intakeManifold,
      T_start = T_start_intakeManifold,
      V = V_intakeManifold
    ),
    exhaustManifold(
      use_T_start = true,
      p_start = p_start_exhaustManifold,
      T_start = T_start_exhaustManifold,
      V = V_exhaustManifold
    ),
    singleCylinderEngines(
      each valve_intake(
        diameter = d_valve_intake,
        fileName_lift = fileName_lift_intake,
        fileName_cd = fileName_cd_intake
      ),
      each valve_exhaust(
        diameter = d_valve_exhaust,
        fileName_lift = fileName_lift_exhaust,
        fileName_cd = fileName_cd_exhaust
      ),
      each injector(
        fuelPerCycle = fuelPerCycle_cylinder,
        injectionStart = injectionStart,
        injectionPeriod = injectionPeriod,
        reference_T_vap = reference_T_vap
      ),
      each crankAndConnRod(
        l_connRod = l_connRod,
        offset_crank = offset_crank
      ),
      chamber(
        each V_displacement = V_displacement_cylinder,
        each compressionRatio = compressionRatio,
        each bore = bore,
        each phi_combustion_begin = phi_combustion_begin,
        each phi_combustion_period = phi_combustion_period,
        each m_wiebe = m_wiebe,
        each c1_woschni_intakeExhaust = c1_woschni_intakeExhaust,
        each c1_woschni_compressionPower = c1_woschni_compressionPower,
        each c2_woschni = c2_woschni,
        p_start = p_start_chambers,
        T_start = T_start_chambers
      )
    )
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  AICE.Fluid.Fittings.Orifices.FixedOrifice exhaustPressureDrop(
    redeclare replaceable package Medium = Medium,
    diameter = d_exhaustPipe
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {40, 30}, extent = {{-5, -5}, {5, 5}}, rotation = 0)
    )
  );

  Modelica.Thermal.HeatTransfer.Sources.FixedTemperature fixedEngineRoomTemperature(
    T = T_engineRoom
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, -50}, extent = {{-5, -5}, {5, 5}}, rotation = 0)
    )
  );

equation

  // interface connections
  connect(phi_throttleOpening, throttle.phi_opening) annotation(
    Line(points = {{-100, 0}, {-30, 0}, {-30, 26}}, color = {0, 0, 127})
  );
  connect(multiCylinderEngine.flange, flange) annotation(
    Line(points = {{20, 0}, {100, 0}, {100, 0}, {100, 0}})
  );

  // fluid connections
  connect(superchargedBoundary.ports[1], intakePressureDrop.port_a) annotation(
    Line(points = {{-65, 30}, {-55, 30}}, color = {0, 127, 255})
  );

  connect(intakeVolume.ports[1], intakePressureDrop.port_b) annotation(
    Line(points = {{-40, 35}, {-40, 30}, {-45, 30}}, color = {0, 127, 255})
  );
  connect(intakeVolume.ports[2], throttle.port_a) annotation(
    Line(points = {{-40, 35}, {-40, 30}, {-35, 30}}, color = {0, 127, 255})
  );

  connect(multiCylinderEngine.port_intake, throttle.port_b) annotation(
    Line(points = {{-12, 20}, {-12, 30}, {-25, 30}}, color = {0, 127, 255})
  );
  connect(multiCylinderEngine.port_exhaust, exhaustPressureDrop.port_a) annotation(
    Line(points = {{+12, 20}, {+12, 30}, {+35, 30}}, color = {0, 127, 255})
  );

  connect(exhaustPressureDrop.port_b, exhaustBoundary.ports[1]) annotation(
    Line(points = {{45, 30}, {65, 30}}, color = {0, 127, 255})
  );

  // thermal connection
  connect(fixedEngineRoomTemperature.port, multiCylinderEngine.heatPort) annotation(
    Line(points = {{-45, -50}, {-30, -50}, {-30, -20}, {-20, -20}}, color = {191, 0, 0}));

end SimpleEngineUnit;
