within AICE.PowerUnits.Combustions.Examples;

model SimpleInline4CylinderSIEngine

  extends Interfaces.PartialFluidEngine(
    useHeatPort = true
  );

  extends Icons.PowerUnits.MultiCylinder;

  final parameter Integer nCylinders = 4;

  AICE.PowerUnits.Combustions.Examples.SimpleSingleCylinderSIEngine[nCylinders] singleCylinderEngines(
    redeclare each replaceable package Medium = Medium,
    each useHeatPort = true
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)
    )
  );

  AICE.Fluid.Fittings.LumpedManifold intakeManifold(
    redeclare replaceable package Medium = Medium,
    final nCylinders = nCylinders
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-60, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  AICE.Fluid.Fittings.LumpedManifold exhaustManifold(
    redeclare replaceable package Medium = Medium,
     final nCylinders = nCylinders
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {60, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  AICE.Mechanics.Rotational.Components.Ideal4CylinderCrankShaft crankShaft(
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {70, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

equation

  connect(port_intake, intakeManifold.port_a) annotation(
    Line(points = {{-60, 100}, {-60, 80}}, color = {0, 127, 255})
  );
  connect(port_exhaust, exhaustManifold.port_a) annotation(
    Line(points = {{+60, 100}, {+60, 80}}, color = {0, 127, 255})
  );
  connect(intakeManifold.ports_b[:], singleCylinderEngines[:].port_intake) annotation(
    Line(
      points = {{-60, 60}, {-60, 40}, {-12, 40}, {-12, 20}},
      color = {0, 127, 255}, thickness = 0.5
    )
  );
  connect(exhaustManifold.ports_b[:], singleCylinderEngines[:].port_exhaust) annotation(
    Line(
      points = {{+60, 60}, {+60, 40}, {+12, 40}, {+12, 20}},
      color = {0, 127, 255}, thickness = 0.5
    )
  );

  connect(singleCylinderEngines[:].flange, crankShaft.flanges_b[:]) annotation(
    Line(points = {{20, 0}, {60, 0}}, thickness = 0.5)
  );
  connect(flange, crankShaft.flange_a) annotation(
    Line(points = {{100, 0}, {80, 0}})
  );

  for i in 1:nCylinders loop
    connect(singleCylinderEngines[i].heatPort, heatPort) annotation(
      Line(
        points = {{-20, -20}, {-60, -20}, {-60, -100}, {-100, -100}},
        color = {191, 0, 0}
      )
    );
  end for;

end SimpleInline4CylinderSIEngine;
