within AICE.PowerUnits.Combustions.Examples;

model SimpleSingleCylinderSIEngine
  extends Interfaces.PartialFluidEngine;

  extends Icons.PowerUnits.SingleCylinder;

  import Modelica.Constants.pi;

  final parameter CarUnits.Length l_connRod =
    crankAndConnRod.l_connRod;

  final parameter CarUnits.Position offset_crank =
    crankAndConnRod.offset_crank;

  final parameter CarUnits.Length pistonStroke =
    chamber.pistonStroke;

  final parameter CarUnits.Radius r_crank = (
    pistonStroke / 2
    * sqrt(
      1 - 4*offset_crank^2 / (4*l_connRod^2 - pistonStroke^2)
    )
  );

public

  AICE.Fluid.Valves.CamBasedValve valve_intake(
    redeclare replaceable package Medium = Medium,
    camTableStartAngle = -360,
    phi_deadZone = 0.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 40}, extent = {{10, 10}, {-10, -10}}, rotation = 180)
    )
  );
  AICE.Fluid.Valves.CamBasedValve valve_exhaust(
    redeclare replaceable package Medium = Medium,
    camTableStartAngle = -360,
    phi_deadZone = 0.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {30, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );
  AICE.Fluid.Sources.AngleBasedInjector injector(
    redeclare replaceable package Medium = Medium,
    nPorts = 1
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 70}, extent = {{-10, -10}, {10, 10}}, rotation = -90)
    )
  );

  AICE.PowerUnits.Combustions.Chambers.WiebeWoschniChamber chamber(
    redeclare replaceable package Medium = Medium,
    use_HeatTransfer = true,
    use_portsData = false,
    nPorts = 3,
    use_phi_piston = true,
    use_intakeValveOpens = true,
    use_exhaustValveOpens = true
  )
  annotation(
    Placement(
      visible = true,
      transformation(origin = {0, 0}, extent = {{20, -20}, {-20, 20}}, rotation = 180)
    )
  );

  AICE.Mechanics.Rotational.Components.CrankAndConnRod crankAndConnRod(
    r_crank = r_crank,
    l_connRod = l_connRod,
    offset_crank = offset_crank
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {0, -50}, extent = {{10, 10}, {-10, -10}}, rotation = -90)
    )
  );

  Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(
    Placement(
      visible = true,
      transformation(origin = {70, 0}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Sources.RealExpression zero(
    y = 0.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-80, 54}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

equation

  connect(angleSensor.flange, flange) annotation(
    Line(points = {{80, 0}, {100, 0}}));

  connect(angleSensor.phi, valve_intake.phi_crank) annotation(
    Line(points = {{60, 0}, {55, 0}, {55, 85}, {-30, 85}, {-30, 50}}, color = {0, 0, 127}));
  connect(angleSensor.phi, valve_exhaust.phi_crank) annotation(
    Line(points = {{60, 0}, {55, 0}, {55, 85}, {+30, 85}, {+30, 50}}, color = {0, 0, 127}));
  connect(angleSensor.phi, injector.phi_piston) annotation(
    Line(points = {{60, 0}, {55, 0}, {55, 85}, {  0, 85}, {  0, 80}}, color = {0, 0, 127}));

  connect(angleSensor.phi, chamber.phi_piston)
  annotation(
    Line(
      points = {{60, 0}, {55, 0}, {55, -16}, {20, -16}},
      color = {0, 0, 127}
    )
  );

  // fluid connections
  connect(port_intake, valve_intake.port_a) annotation(
    Line(points = {{-60, 100}, {-60, 100}, {-60, 40}, {-40, 40}, {-40, 40}}, color = {0, 127, 255}));

  connect(valve_intake.port_b, chamber.ports[1]) annotation(
    Line(points = {{-20, 40}, {-6, 40}, {-6, 20}, {0, 20}}, color = {0, 127, 255}));
  connect(injector.ports[1], chamber.ports[2]) annotation(
    Line(points = {{  0, 60}, { 0, 60}, { 0, 20}, {0, 20}}, color = {0, 127, 255}, thickness = 0.5));
  connect(valve_exhaust.port_a, chamber.ports[3]) annotation(
    Line(points = {{+20, 40}, {+6, 40}, {+6, 20}, {0, 20}}, color = {0, 127, 255}));

  connect(valve_exhaust.port_b, port_exhaust) annotation(
    Line(points = {{40, 40}, {60, 40}, {60, 102}, {60, 102}, {60, 100}}, color = {0, 127, 255}));

  connect(valve_intake.opens, chamber.intakeValveOpens)
  annotation(
    Line(
      points = {{-20, 46}, {-15, 46}, {-15, 35}, {50, 35}, {50, -12}, {20, -12}},
      color = {255, 0, 255}
    )
  );

  connect(valve_exhaust.opens, chamber.exhaustValveOpens)
  annotation(
    Line(
      points = {{ 40, 46}, { 50, 46}, { 50, 35}, {50, 35}, {50,  -8}, {20,  -8}},
      color = {255, 0, 255}
    )
  );

  // other connections
  connect(chamber.heatPort, heatPort) annotation(
    Line(points = {{-20, 0}, {-60, 0}, {-60, -100}, {-100, -100}, {-100, -100}}, color = {191, 0, 0}));
  connect(crankAndConnRod.flangeR, flange) annotation(
    Line(points = {{0, -60}, {0, -60}, {0, -80}, {80, -80}, {80, 0}, {100, 0}, {100, 0}}));
  connect(crankAndConnRod.flangeT, chamber.flange) annotation(
    Line(points = {{0, -40}, {0, -40}, {0, -20}, {0, -20}}, color = {0, 127, 0}));

  connect(zero.y, valve_intake.phi_offset) annotation(
    Line(points = {{-68, 54}, {-44, 54}, {-44, 46}, {-40, 46}, {-40, 46}, {-38, 46}}, color = {0, 0, 127}));
  connect(zero.y, valve_exhaust.phi_offset) annotation(
    Line(points = {{-68, 54}, {16, 54}, {16, 46}, {20, 46}, {20, 46}, {22, 46}}, color = {0, 0, 127}));

end SimpleSingleCylinderSIEngine;
