within AICE.Research.AlgebraicLoops;

model TestDrive
  extends Modelica.Icons.Example;

  Modelica.Mechanics.Rotational.Components.Inertia inertia1(
    J = 0.1,
    phi(start = 0),
    w(fixed = true, start = Modelica.SIunits.Conversions.from_rpm(1500)),  // initial engine speed
    a(start = 0)
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-10, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Components.Clutch clutch1(
    fn_max = 100
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {20, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  AICE.VEH.TRA.TRC0A trc0a1(
    Cmapfile = Files.loadResource("modelica://AICE/Resources/TableData/Drivetrain/GenDefTorConC00.dat"),
    TRmapfile = Files.loadResource("modelica://AICE/Resources/TableData/Drivetrain/GenDefTorConTR00.dat")
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {20, -70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Mechanics.Rotational.Components.Inertia inertia2(
    J = 0.001
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {50, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  AICE.VEH.TRA.CVT0B cvt0b1(DTAU1 = 0.03, TAU0 = 0.01)  annotation(
    Placement(visible = true, transformation(origin = {92, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  AICE.VEH.CHA.CHA0A cha0a1(MASS = 1000)  annotation(
    Placement(visible = true, transformation(origin = {184, 74}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Mechanics.Rotational.Components.IdealGear idealGear1(ratio = 6)  annotation(
    Placement(visible = true, transformation(origin = {166, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Step step5(height = 0, offset = 0, startTime = 0) annotation(
    Placement(visible = true, transformation(origin = {144, 84}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.Constant const2(k = 0) annotation(
    Placement(visible = true, transformation(origin = {222, 74}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));

  AICE.VEH.DRV.DRV0A drv0a1(
    BRAKE_GAIN = 3, PID_GAIN = 9, PID_Ti = 90
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-156, 44}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Sources.RealExpression realExpression1(y = cha0a1.VEL_X)  annotation(
    Placement(visible = true, transformation(origin = {-208, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

  Modelica.Mechanics.Rotational.Components.Brake brake1(
    fn_max = 1000
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {120, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  AICE.VEH.CYC.CYC0A cyc0a1(CYCTYP = AICE.VEH.CYC.CYC0A.cycle_type.uddscol)  annotation(
    Placement(
      visible = true,
      transformation(origin = {-218, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Math.Gain gain1(k = 1 / 3.6)  annotation(
    Placement(visible = true, transformation(origin = {-184, 70}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

  AICE.VEH.ECU.ECVTA ecvta1(
    TR = 55,
    mapfilename = Files.loadResource("modelica://AICE/Resources/TableData/Drivetrain/GenDefCVTECU00.dat")
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-30, 66}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  Modelica.Blocks.Math.Gain gain2(k = 1 / 100) annotation(
    Placement(visible = true, transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Continuous.FirstOrder firstOrder1(T = 0.4)  annotation(
    Placement(visible = true, transformation(origin = {50, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain gain3(k =  3.6) annotation(
    Placement(visible = true, transformation(origin = {-70, 72}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Nonlinear.DeadZone deadZone1(deadZoneAtInit = true, uMax = 15 / 3.6, uMin = -1.e4)  annotation(
    Placement(visible = true, transformation(origin = {-70, -2}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Math.Gain gain4(k = 10)  annotation(
    Placement(visible = true, transformation(origin = {-16, -2}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));

  AICE.Research.AlgebraicLoops.Components.Accel2Throttle accel2Throttle(
    throttle_min = 0.0698132,
    throttle_max = 1.5708
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-120, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)
    )
  );

  AICE.Examples.METI_2019.Components.SampleEngineUnit engine(
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-90, -50}, extent = {{-10, -10}, {+10, +10}}, rotation = 0)
    )
  );

  AICE.Blocks.Thermal.HeatTransfer.Sources.FixedTemperature engineWallTemperature(
    T = 550.0
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-90, -80}, extent = {{-10, 10}, {10, -10}}, rotation = 90)
    )
  );

  AICE.Blocks.Mechanics.Rotational.Interfaces.Adaptors.Adaptor_AOTI adaptor_engineRotational(
    energy = AICE.Blocks.Types.Energy.Sink
  ) annotation(
    Placement(
      visible = true,
      transformation(origin = {-50, -50}, extent = {{10, -10}, {-10, 10}}, rotation = 0)
    )
  );

equation

  // engine connections
  connect(drv0a1.THR, accel2Throttle.accel) annotation(
    Line(points = {{-146, 40}, {-144, 40}, {-144, -50}, {-130, -50}}, color = {0, 0, 127})
  );
  connect(accel2Throttle.throttle, engine.phi_throttle) annotation(
    Line(points = {{-110, -50}, {-100, -50}}, color = {0, 0, 127})
  );

  connect(engineWallTemperature.temperature, engine.temperature) annotation(
    Line(points = {{-84, -70}, {-84, -60}}, color = {0, 0, 127})
  );
  connect(engineWallTemperature.heatFlowRate, engine.heatFlowRate) annotation(
    Line(points = {{-96, -70}, {-96, -60}}, color = {0, 0, 127})
  );

  connect(engine.angularVelocity, adaptor_engineRotational.angularVelocity_b) annotation(
    Line(points = {{-80, -44}, {-60, -44}}, color = {0, 0, 127})
  );
  connect(engine.torque, adaptor_engineRotational.torque_b) annotation(
    Line(points = {{-80, -56}, {-60, -56}}, color = {0, 0, 127})
  );

  // inertia1 connections
  connect(adaptor_engineRotational.flange_a, inertia1.flange_a) annotation(
    Line(points = {{-40, -50}, {-20, -50}}));
  connect(inertia1.flange_b, clutch1.flange_a) annotation(
    Line(points = {{0, -50}, {0, -30}, {10, -30}}));
  connect(inertia1.flange_b, trc0a1.flange_a) annotation(
    Line(points = {{0, -50}, {0, -70}, {10, -70}}));

  // inertia2 connections
  connect(clutch1.flange_b, inertia2.flange_a) annotation(
    Line(points = {{30, -30}, {40, -30}, {40, -50}, {52, -54}}));
  connect(trc0a1.flange_b, inertia2.flange_a) annotation(
    Line(points = {{30, -70}, {40, -70}, {40, -50}, {52, -54}}));
  connect(inertia2.flange_b, cvt0b1.flange_a) annotation(
    Line(points = {{60, -50}, {70, -50}, {70, 10}, {82, 10}}));

  connect(cvt0b1.flange_b, idealGear1.flange_a) annotation(
    Line(points = {{102, 10}, {156, 10}}));
  connect(idealGear1.flange_b, cha0a1.FRW) annotation(
    Line(points = {{176, 10}, {190, 10}, {190, 65}}));
  connect(gain4.y, clutch1.f_normalized) annotation(
    Line(points = {{-5, -2}, {20, -2}, {20, -20}}, color = {0, 0, 127}));
  connect(gain3.y, ecvta1.VehV) annotation(
    Line(points = {{-59, 72}, {-38, 72}}, color = {0, 0, 127}));
  connect(gain3.u, realExpression1.y) annotation(
    Line(points = {{-82, 72}, {-128, 72}, {-128, 24}, {-164, 24}, {-164, 10}, {-196, 10}}, color = {0, 0, 127}));
  connect(gain2.y, ecvta1.throttle) annotation(
    Line(points = {{-59, 40}, {-51, 40}, {-51, 60}, {-38, 60}}, color = {0, 0, 127}));
  connect(gain2.u, drv0a1.THR) annotation(
    Line(points = {{-82, 40}, {-146, 40}}, color = {0, 0, 127}));
  connect(ecvta1.RATIORef, firstOrder1.u) annotation(
    Line(points = {{-22, 60}, {38, 60}}, color = {0, 0, 127}));
  connect(firstOrder1.y, cvt0b1.CVTRatio) annotation(
    Line(points = {{61, 60}, {92, 60}, {92, 20}}, color = {0, 0, 127}));
  connect(deadZone1.y, gain4.u) annotation(
    Line(points = {{-59, -2}, {-28, -2}}, color = {0, 0, 127}));
  connect(deadZone1.u, realExpression1.y) annotation(
    Line(points = {{-82, -2}, {-90, -2}, {-90, 24}, {-164, 24}, {-164, 10}, {-196, 10}}, color = {0, 0, 127}));
  connect(brake1.flange_b, cha0a1.REW) annotation(
    Line(points = {{130, 52}, {178, 52}, {178, 65}}));
  connect(step5.y, cha0a1.ALPHA) annotation(
    Line(points = {{155, 84}, {150.5, 84}, {150.5, 80}, {176, 80}}, color = {0, 0, 127}));
  connect(cha0a1.WIND, const2.y) annotation(
    Line(points = {{192, 71}, {211, 71}, {211, 74}}, color = {0, 0, 127}));
  connect(cyc0a1.y, gain1.u) annotation(
    Line(points = {{-209, 70}, {-196, 70}}, color = {0, 0, 127}));
  connect(gain1.y, drv0a1.VREF) annotation(
    Line(points = {{-172, 70}, {-166, 70}, {-166, 48}, {-164, 48}}, color = {0, 0, 127}));
  connect(brake1.f_normalized, drv0a1.BRK) annotation(
    Line(points = {{120, 64}, {120, 64}, {120, 94}, {-134, 94}, {-134, 52}, {-146, 52}, {-146, 50}}, color = {0, 0, 127}));
  connect(realExpression1.y, drv0a1.VMES) annotation(
    Line(points = {{-196, 10}, {-164, 10}, {-164, 38}, {-164, 38}}, color = {0, 0, 127}));

  annotation(
    Icon(coordinateSystem(initialScale = 0.5)),
    Diagram(coordinateSystem(extent = {{-200, -100}, {200, 100}}, initialScale = 0.5))
  );

end TestDrive;
