within AICE.Research;

model VolumeReinit
  import Modelica.Constants.pi;

  // Mass and energy balance, ports
  extends Modelica.Fluid.Vessels.BaseClasses.PartialLumpedVessel(
    final fluidVolume = V,
    vesselArea = 1, heatTransfer(surfaceAreas={4*pi*(3/4*V/pi)^(2/3)}));
  //pi*(3/4*V)^(2/3),
  SI.Volume V "Volume";
  SI.Volume V0(start = 1.0) "Volume";
  Boolean positive;
  
equation

  Wb_flow = 0;
  for i in 1:nPorts loop
    vessel_ps_static[i] = medium.p;
  end for;
  

  //V = 1 + 0.2 * sin(time*10);
  der(V0) = 0.0;
  V = V0 + time*0.1;
  
  
  positive = Modelica.Math.sin(time*10-1.57) > 0;
  when time == 0.1 then
  //when positive and not pre(positive) then
  //when edge(positive) then
    reinit(medium.p, pre(medium.p)*1.5);
    reinit(medium.T, 323.15);
    reinit(medium.X[1], 1.0);
    reinit(medium.X[2], 0.0);
    //reinit(mXi[1] , 1.58495);
    //reinit(mXi[2] , 0.0);

    //reinit(m, 1.0);
    //reinit(m, pre(m)*1.5);
    //reinit(mXi, pre(mXi)/0.8);
    //reinit(U, pre(U)*1.5);
    //reinit(V0, pre(V0)*0.8);
  end when;
  
  annotation (defaultComponentName="volume",
    Icon(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{
          100,100}}), graphics={Ellipse(
        extent={{-100,100},{100,-100}},
        lineColor={0,0,0},
        fillPattern=FillPattern.Sphere,
        fillColor={170,213,255}), Text(
        extent={{-150,12},{150,-18}},
        lineColor={0,0,0},
        textString="V=%V")}),
  Documentation(info="<html>
<p>
Ideally mixed volume of constant size with two fluid ports and one medium model.
The flow properties are computed from the upstream quantities, pressures are equal in both nodes and the medium model if <code>use_portsData=false</code>.
Heat transfer through a thermal port is possible, it equals zero if the port remains unconnected.
A spherical shape is assumed for the heat transfer area, with V=4/3*pi*r^3, A=4*pi*r^2.
Ideal heat transfer is assumed per default; the thermal port temperature is equal to the medium temperature.
</p>
<p>
If <code>use_portsData=true</code>, the port pressures represent the pressures just after the outlet (or just before the inlet) in the attached pipe.
The hydraulic resistances <code>portsData.zeta_in</code> and <code>portsData.zeta_out</code> determine the dissipative pressure drop between volume and port depending on
the direction of mass flow. See <a href=\"modelica://Modelica.Fluid.Vessels.BaseClasses.VesselPortsData\">VesselPortsData</a> and <i>[Idelchik, Handbook of Hydraulic Resistance, 2004]</i>.
</p>
</html>"),
  experiment(StartTime = 0, StopTime = 1, Tolerance = 1e-6, Interval = 0.002),
  __OpenModelica_simulationFlags(lv = "LOG_STATS", outputFormat = "mat", s = "dassl"));

end VolumeReinit;