#ifndef __EXT_C_MAP1D1
#define __EXT_C_MAP1D1
/*
 MAP1D1.C
 Frederic Brix
 version 001
 20170217

 The target is to test the file opening and the instantiation of the table.
  gcc -x c -c -o MAP1D1.o MAP1D1.c
  gcc -shared -o MAP1D1.dll MAP1D1.o
 to produce a DLL
 */

#include "MAP1D1.h"

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>


/* INITIALISATION */
Linear1D_UTI* initMAP1D1(char* fileName, int cyclic) {
  Linear1D_UTI* curve = malloc(sizeof(Linear1D_UTI));
  FILE *f;
  int n,num,index,i,j,SIZE; // Counter for the table size ,
  // and some counters and reference when we change size
  char buff[100];
  double X,Y,tmp;
  int s;

  double beta,alpha, gamma;
  double bet2,fact;
  double resolution = 1.e-12;
  double *a,*b,*c,*r,*x,*gam2,*u,*z;

  num=0; /* Initialization of the counter of elements*/
  errno = 0;
  f=fopen(fileName,"r");
  index=0;
  n=2;
  while(fgets( buff, sizeof buff, f ) != NULL && index<500 && (n=sscanf( buff, "%lf%lf", &X, &Y ))==2){
    (*curve).X[index]=X;
    (*curve).Y[index]=Y;
    index=index+1;
  }
  (*curve).size=index;
  s = fclose(f);
  if (s == EOF) perror("File closing problem \n");

  // Reordering
  num=index;
  for (i=0; i<(num-1); i++) {
    for (j=0; j<(num-1-i); j++) {
      if ((*curve).X[j+1] < (*curve).X[j]) {
        tmp = (*curve).X[j];
        (*curve).X[j] = (*curve).X[j+1];
        (*curve).X[j+1] = tmp;
        tmp = (*curve).Y[j];
        (*curve).Y[j] = (*curve).Y[j+1];
        (*curve).Y[j+1] = tmp;
      }
    }
  }

  // CLEANING
  (*curve).cyclic = (cyclic==1)?1:0;
  //Extremities compatibility in the case of cyclic splines
  if (cyclic==1) {
    tmp=((*curve).Y[0]+(*curve).Y[num-1])/2;
    (*curve).Y[0]=(*curve).Y[num-1]=tmp;
  }
  // Remove redundancies
  SIZE = num;
  for (i=0; i<(SIZE-1); i++) {
    if (((*curve).X[i+1] - (*curve).X[i]) < resolution ) {
      for (j=i; j<(SIZE-1);j++) {
        (*curve).X[j]=(*curve).X[j+1];
        (*curve).Y[j]=(*curve).Y[j+1];
      }
      SIZE=SIZE-1;
    }
  }

  (*curve).size=SIZE;

  return curve;
}


void closeMAP1D1(Linear1D_UTI* curve) {
  if (curve == NULL) return;
  free(curve);
}

double linintMAP1D1(Linear1D_UTI* curve, double u) {
  double ur,BASE;
  int ju,jm,jl;
  double A2,B2;
  double DX;
  if (curve == NULL) return 0;
  if ((*curve).cyclic==1) {
    BASE = (*curve).X[(*curve).size-1]-(*curve).X[0];
    ur=fmod(u-(*curve).X[0],BASE);
    if (ur<0) {ur=ur+BASE;} // To make it a cyclic function (hope the solver can stand this).
    ur=ur+(*curve).X[0];
    // LOCATE
    jl=0;
    ju=(*curve).size-1;
    while (ju-jl>1) {
      jm=(ju+jl)>>1;
      if (ur>=(*curve).X[jm]) {
        jl=jm;
      } else {
        ju=jm;
      }
    } // end of LOCATE
    if ((*curve).X[jl]==(*curve).X[jl+1]) {
      return (*curve).Y[jl];
    } else {
      DX=((*curve).X[jl+1]-(*curve).X[jl]);
      A2=((*curve).X[jl+1]-ur)/DX;
      B2=1.0-A2;
      return A2*(*curve).Y[jl]+B2*(*curve).Y[jl+1];
    }
  } // end of CYCLIC
  else {
    if (u<(*curve).X[0]) {
      DX=((*curve).X[1]-(*curve).X[0]);
      A2=((*curve).X[1]-u)/DX;
      B2=1.0-A2;
      return A2*(*curve).Y[0]+B2*(*curve).Y[1];
    } else if (u<(*curve).X[(*curve).size-1]) {
      // LOCATE
      jl=0;
      ju=(*curve).size-1;
      while (ju-jl>1) {
        jm=(ju+jl)>>1;
        if(u>=(*curve).X[jm]) {
          jl=jm;
        } else {
          ju=jm;
        }
      } // end of LOCATE
      if ((*curve).X[jl]==(*curve).X[jl+1]) {
        return (*curve).Y[jl];
      } else {
        DX=((*curve).X[jl+1]-(*curve).X[jl]);
        A2=((*curve).X[jl+1]-u)/DX;
        B2=1.0-A2;
        return A2*(*curve).Y[jl]+B2*(*curve).Y[jl+1];
      }
    } else {
      ju=(*curve).size-1;
      jl=(*curve).size-2;
      DX=((*curve).X[ju]-(*curve).X[jl]);
      A2=((*curve).X[ju]-u)/DX;
      B2=1.0-A2;
      return A2*(*curve).Y[jl]+B2*(*curve).Y[ju];
    }
  } // end of NON CYCLIC
}

int SizeLinear1D_UTI(void* object){
  Linear1D_UTI* curve = (Linear1D_UTI*) object;
  return (*curve).size;
}

int IsCyclicLinear1D_UTI(void* object){
  Linear1D_UTI* curve = (Linear1D_UTI*) object;
  return (*curve).cyclic;
}
#endif
