#ifndef __EXT_C_MAP2D1
#define __EXT_C_MAP2D1
/*
  MAP2D1.cp
  Version 2.0 20161102
  Frederic Brix

 This file deals with a 2D map declared by a succession of parametric curves.
 n is the number of curves for a parameter X[i] for i=0:(n-1)
 Each curve has m_i elements , not necessarily identical all driven by (Y[j],Z[j]).
 The target is to test the file opening and the instantiation of the table.
  gcc -x c -c -o MAP2D1.o MAP2D1.cp
  gcc -shared -o MAP2D1.dll MAP2D1.o to produce a DLL

 */

#include "MAP2D1.h"

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>


/*
  filename is the file name of the data file X1[i] Y[i]
  extrap is the indication about extrapolation extrap=1 or extreme values extrap=0
*/

MAP2D1_UTI* initMAP2D1(char* fileName) {
  MAP2D1_UTI* map2d1 = malloc(sizeof(MAP2D1_UTI));
  FILE *f;           /* f is the data file and debug is the debug message file */
  int num,stllong;   /* num is the number of X1 values and stllong is the length of the debug file name */
  char stlFile[200]; /* name of the debug file */
  char buff[100];    /* buffer to read the data file */
  int index,nX,n,nYZ;  /* index to read the data file */
  double X,Y,Z;        /* intermediate values of X and Y while reading */
  int s;             /* s is an error flag for the opening and closing of files */
  int i,j,k;           /* i and j are indexes used during the reordering of the data */
  int ord[100];      /* used to track the reordering of Xs */
  int tmpi;           /* for the reordering of Xs */
  double tmp;        /* tmp is also used during the reordering of the data */
  double resolution = 1.e-11; /* resolution is controversial as it modifies the input data
                                 it is used to differential and separate identical values */
  int SIZE;          /* running SIZE of the loaded data after cleaning and reordering */
  double xm,xM,x0,ym,y0,yM; /* used to calculate the derivative of the table dY/dx */
  /*initialization of ord the initial position of Xs that will follow the permutations of this bubble algo*/
  for (i=0; i<99;i++){
    ord[i]=i;
  }
  num=0; /* Initialization of the counter of elements*/
  // Opening of files f to input the data and debug to send messages
  errno = 0; /* defined in errno.h */

  // This block extends the name of the input file with a debug string to generate a debug file.
  stlFile[0]='\0';
  perror(fileName); //CHECK
  strcat(stlFile,fileName);
  stllong = strlen(stlFile);
  stlFile[stllong-4]='\0';
  strcat(stlFile,".debug");
  // f is the handler of the file
  errno = 0;
  f=fopen(fileName,"r");

  // Data reading and treatment
  // The file format is such that first we enter nX, the number of Xs
  // then come the nX values of Xs (without any character of lines mentioning that we have reached the end of the list.
  // The Xs line are also completed with a second element nYZ, the number of YZ element for the curve-map
  // Then all the YZ elements are listed in line Y Z without any separators ...
  // this DATA file is not a human format file , sorry
  //  fprintf(debug,"Starting reading \n");
  n=1;
  fgets( buff, sizeof buff, f); /*Reads the first line in order to get the number of X elements */
  n=sscanf( buff,"%d",&nX); /* n tracks the number of characters that are read in the line  */
  (*map2d1).nX=nX;

  n=2; /* the next phase is the input of Xs (double) and nYZs (integer) element */
  index=0; /* index is the reading progression index for Xs input ... there are nX lines to read*/
  for (index=0;index<((*map2d1).nX);index++){
    fgets( buff, sizeof buff, f );
    n=sscanf( buff, "%lf%i", &X, &nYZ );

    (*map2d1).X[index]=X;
    (*map2d1).nYZ[index]=nYZ;
  }


  // Reading of each Y[i][j] and Z[i][j]
  // index is used again but here the reading progression is renewed for each Xs and limited by [0-(nYZ[i]-1)]
  // i is the progression index for Xs ... there are nX values between 0 to nX-1
  for (i=0;i<((*map2d1).nX);i++){
    n=2; // two elements are expected per line. If not I simply here issue a warning or do nothing.
    // The format of the file should be handled and secured by the generation phase .. not the reading phase.
    for (index=0;index<((*map2d1).nYZ[i]);index++){
      fgets( buff, sizeof buff, f );
      n=sscanf( buff, "%lf%lf", &Y, &Z );
      (*map2d1).Y[i][index]=Y;
      (*map2d1).Z[i][index]=Z;
    } // end of YZ
  } // end of for for X

  // We can close the data file
  s = fclose(f);
  if (s == EOF) perror("File closing problem (1) \n");

 // Reordering the Xs
  // This bubble algorithm is efficient but slow expecially with the arrival of Y and Z values that we have to carry
  // This algorithm will be modified if required .. it is used only once in the simulation !
  // The first part will track the value that is not in the right place and invert Xs and then nYZ
  // and then the time consuming reordering of Y and Z (that is to say nYZ values every time
  // and then finally ord the initial ordering value ... (REMARQUE : This ord is intended for another algorithm
  // that does the ordering of YZ later but at memory cost not time.
  num = (*map2d1).nX;
  for (i=0; i<(num-1); i++) {
    for (j=0; j<(num-1-i); j++) {
      if ((*map2d1).X[j+1] < (*map2d1).X[j]) {
        tmp = (*map2d1).X[j];
        (*map2d1).X[j] = (*map2d1).X[j+1];
        (*map2d1).X[j+1] = tmp;
        tmpi = (*map2d1).nYZ[j];
        (*map2d1).nYZ[j] = (*map2d1).nYZ[j+1];
        (*map2d1).nYZ[j+1] = tmpi;
        for (k=0;(k<(tmpi)-1)||(k<(*map2d1).nYZ[j]-1);k++){
          tmp = (*map2d1).Y[j][k];
          (*map2d1).Y[j][k]=(*map2d1).Y[j+1][k];
          (*map2d1).Y[j+1][k]=tmp;
          tmp = (*map2d1).Z[j][k];
          (*map2d1).Z[j][k]=(*map2d1).Z[j+1][k];
          (*map2d1).Z[j+1][k]=tmp;
        }
        n = ord[j];
        ord[j] = ord[j+1];
        ord[j+1]=n;
      }
    }
  }

  // REORDERING OF Ys
  num = (*map2d1).nX;
  for (i=0; i<(num); i++) {
    n=(*map2d1).nYZ[i];
    for (j=0; j<(n-1); j++) {
      for (k=0; k<(n-1-j); k++) {
        if ((*map2d1).Y[i][k+1] < (*map2d1).Y[i][k]) {
          tmp = (*map2d1).Y[i][k];
          (*map2d1).Y[i][k] = (*map2d1).Y[i][k+1];
          (*map2d1).Y[i][k+1] = tmp;
          tmp = (*map2d1).Z[i][k];
          (*map2d1).Z[i][k] = (*map2d1).Z[i][k+1];
          (*map2d1).Z[i][k+1] = tmp;
        }
      }
    }
  }

  return map2d1;
}

void closeMAP2D1(MAP2D1_UTI* map2d1) {
  if (NULL == map2d1) return;
  //  perror("Closing of the object\n");
  //  free(came -> nYZ);
  //  free(came -> X);
  //  free(came -> Y);
  //  free(came -> Z);
  free(map2d1);
}

double linintMAP2D1(MAP2D1_UTI* map2d1, double u, double v) {
  double z1m,z2m,z11,z12,z22,z21;
  double y11,y12,y22,y21;
  double x1,x2;
  int ju,jm,jl;
  int ku,km,kl;
  // LOCATE
  if (u<(*map2d1).X[0]){   // ZONE {1},{2} et {3}
    jl=-1;
    ju=0;
    // Find algorithm to find extrapolation method taking into account other curves
    // Here we localize first if we are below the curve 0
    if (v<(*map2d1).Y[ju][0]){  // ZONE {1}
      return (*map2d1).Z[ju][0];
    } // END OF ZONE {1}
    else if (v>(*map2d1).X[(*map2d1).nYZ[ju]-1]){ // ou au dessus // ZONE {2}
      return (*map2d1).Z[ju][(*map2d1).nYZ[ju]-1];
    }/*END OF ZONE {2}*/
    else {  // sinon ou exactement a l interieur ceci avec ku,kl,km  // ZONE {3} between {1} and {2}
      kl=0;
      ku=(*map2d1).nYZ[ju]-1;
      while (ku-kl>1){
        km=(ku+kl)>>1;
        if(v>=(*map2d1).Y[ju][km]) {
          kl=km;
        } else {
          ku=km;
        }
      } // End of localization in ZONE {3}
      // here we have ju=0 and jl=-1 because X<min(Xs)=(*map2d1).X[0]
      // We are extrapolating the map by taking the value at the border of the X and Y domain
      // still doing linear interpolation
      y11=(*map2d1).Y[ju][kl];
      y12=(*map2d1).Y[ju][ku];
      z11=(*map2d1).Z[ju][kl];
      z12=(*map2d1).Z[ju][ku];
      if (y11==y12){
        return (z11+z12)/2.0;
      } else {
        return z11+(z12-z11)/(y12-y11)*(v-y11);
      }
    } // END of ZONE {3}
  } // END OF ZONE {1}+{2}+{3}
  else if (u>(*map2d1).X[(*map2d1).nX-1]) { // ZONE {4},{5} and {6}  for which X> max(Xs)=().X[().nX-1]
    jl=(*map2d1).nX-1;
    ju=jl+1;
    // TROUVER UNE EXTRAPOLATION QUI TIENNE COMPTE DES AUTRES COURBES SI POSSIBLE
    // Ici on localise d abord si on est en dessous de la courbe 0
    if (v<(*map2d1).Y[jl][0]) {  // ZONE {4}
      return (*map2d1).Z[jl][0];
    } // END OF ZONE {4}
    else if (v>(*map2d1).Y[jl][(*map2d1).nYZ[jl]-1]) { // ou au dessus // ZONE {5}
      return (*map2d1).Z[jl][(*map2d1).nYZ[jl]-1];
    } // END OF ZONE {5}
    else {  // sinon ou exactement a l interieur ceci avec ku,kl,km // ZONE{6}
      kl=0;
      ku=(*map2d1).nYZ[jl]-1;
      while (ku-kl>1) {
        km=(ku+kl)>>1;
        if (v>=(*map2d1).Y[jl][km]) {
          kl=km;
        } else {
          ku=km;
        }
      } // End of localization in ZONS {6}
      // here we have jl=(*map2d1).nX-1 and ju=jl+1 because X>max(Xs)=(*map2d1).X[(*map2d1).nX-1]
      // We are extrapolating the map by taking the value at the border of the X and Y domain
      // still doing linear interpolation
      y11=(*map2d1).Y[jl][kl];
      y12=(*map2d1).Y[jl][ku];
      z11=(*map2d1).Z[jl][kl];
      z12=(*map2d1).Z[jl][ku];
      if (y11==y12){
        return (z11+z12)/2.0;
      } else {
        return z11+(z12-z11)/(y12-y11)*(v-y11);
      }
    } // END OF ZONE {6}
  } // END OF ZONE {4}+{5}+{6}
  else { // BEGINING OF ZONE {7}+{8}+{9} with {9} the most used probably as it is entirely within the data set
    jl=0;
    ju=(*map2d1).nX-1;
    while (ju-jl>1){
      jm=(ju+jl)>>1;
      if (u>=(*map2d1).X[jm]) {
        jl=jm;
      } else {
        ju=jm;
      }
    } // End of localization of X in the data set
    x1=(*map2d1).X[jl];
    x2=(*map2d1).X[ju];
    if (v <(*map2d1).Y[jl][0] ) { // We want to indentify the location of v in the data set , on the jl line
      // this will give us the values of y11 y12 z11 and z12
      y11 = (*map2d1).Y[jl][0];
      z11 = (*map2d1).Z[jl][0];
      y12 = (*map2d1).Y[jl][0];
      z12 = (*map2d1).Z[jl][0];
    } else if (v>(*map2d1).Y[jl][(*map2d1).nYZ[jl]-1]) {
      y11 = (*map2d1).Y[jl][(*map2d1).nYZ[jl]-1];
      z11 = (*map2d1).Z[jl][(*map2d1).nYZ[jl]-1];
      y12 = (*map2d1).Y[jl][(*map2d1).nYZ[jl]-1];
      z12 = (*map2d1).Z[jl][(*map2d1).nYZ[jl]-1];
    } else /*1*/ {
      // We must localize y11 into the map2d1 set line
      kl=0;
      ku=(*map2d1).nYZ[jl]-1;
      while (ku-kl>1){
        km=(ku+kl)>>1;
        if(v>=(*map2d1).Y[jl][km]) {
          kl=km;
        } else {
          ku=km;
        }
      }
      y11=(*map2d1).Y[jl][kl];
      y12=(*map2d1).Y[jl][ku];
      z11=(*map2d1).Z[jl][kl];
      z12=(*map2d1).Z[jl][ku];
    } // END OF Identification of  y11 y12 z11 and z12
    // Now we want  y21 y22 z21 and z22 situated in the jl+1 = ju line of Xs , whcih is x2
    if (v<(*map2d1).Y[ju][0]){ // We want to indentify the location of v in the data set , on the jl line
      // this will give us the values of y21 y22 z21 and z22
      y21 = (*map2d1).Y[ju][0];
      z21 = (*map2d1).Z[ju][0];
      y22 = (*map2d1).Y[ju][0];
      z22 = (*map2d1).Z[ju][0];
    } else if (v>(*map2d1).Y[ju][(*map2d1).nYZ[ju]-1]) {
      y21 = (*map2d1).Y[ju][(*map2d1).nYZ[ju]-1];
      z21 = (*map2d1).Z[ju][(*map2d1).nYZ[ju]-1];
      y22 = (*map2d1).Y[ju][(*map2d1).nYZ[ju]-1];
      z22 = (*map2d1).Z[ju][(*map2d1).nYZ[ju]-1];
    } else {
      // We must localize y21 into the map2d1 set line
      kl=0;
      ku=(*map2d1).nYZ[ju]-1;
      while (ku-kl>1){
        km=(ku+kl)>>1;
        if(v>=(*map2d1).Y[ju][km]) {
          kl=km;
        } else {
          ku=km;
        }
      }
      y21=(*map2d1).Y[ju][kl];
      y22=(*map2d1).Y[ju][ku];
      z21=(*map2d1).Z[ju][kl];
      z22=(*map2d1).Z[ju][ku];
    } // END OF Identification of  y21 y22 z21 and z22 for the Xs=x2
    // We can calculated the intermediate z1m and z2m
    if (y12==y11){
      z1m=(z12+z11)/2.0;
    } else {
      z1m=(z12-z11)/(y12-y11)*(v-y11)+z11;
    }
    if (y22==y21){
      z2m=(z22+z21)/2.0;
    } else {
      z2m=(z22-z21)/(y22-y21)*(v-y21)+z21;
    }
    if (x1==x2){
      return (z1m+z2m)/2.0;
    } else {
      return (u-x1)*(z2m-z1m)/(x2-x1)+z1m;
    }
  }// END OF ZONE {7}+{8}+{9}
}
#endif
