
# previous reaction vessel design

```plantuml

namespace Modelica.Fluid {

  namespace Modelica.Fluid.Interfaces {
    class PartialHeatTransfer <<partial>> {
      parameter SI.CoefficientOfHeatTransfer k = 0
      parameter SI.Temperature T_ambient = system.T_ambient
    }

    class PartialLumpedVolume <<partial>> {
    }
  }

  namespace Modelica.Fluid.Vessels.BaseClasses {

    class PartialLumpedVessel <<partial>>

    namespace Modelica.Fluid.Vessels.BaseClasses.HeatTransfer {
      class PartialVesselHeatTransfer <<partial>>
      class IdealHeatTransfer
      class ConstantHeatTransfer

      Modelica.Fluid.Interfaces.PartialHeatTransfer <|-- PartialVesselHeatTransfer
      PartialVesselHeatTransfer <|-- IdealHeatTransfer
      PartialVesselHeatTransfer <|-- ConstantHeatTransfer
    }

    Modelica.Fluid.Interfaces.PartialLumpedVolume <|-- PartialLumpedVessel
    Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer \
      *.. PartialLumpedVessel
  }
}

namespace AICE.Fluid {

  namespace AICE.Fluid.ReactionVessels.BaseClasses {


    class PartialLumpedReactionVessel <<partial>>

    namespace AICE.Fluid.ReactionVessels.BaseClasses.HeatTransfers {
      class PartialVesselHeatTransfer <<partial>> {
        input SI.Mass[n, Medium.nXi] mXis
      }
      class IdealHeatTransfer
      class ConstantHeatTransfer

      Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer \
        <|-- PartialVesselHeatTransfer

      PartialVesselHeatTransfer <|-- IdealHeatTransfer

      Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.IdealHeatTransfer \
        <|-- IdealHeatTransfer

      PartialVesselHeatTransfer <|-- ConstantHeatTransfer

      Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.ConstantHeatTransfer \
        <|-- ConstantHeatTransfer
    }

    namespace AICE.Fluid.ReactionVessels.BaseClasses.ReactionModels {
      class PartialReactionModel <<partial>>
      class NoReactionModel

      PartialReactionModel <|-- NoReactionModel
    }

    AICE.Fluid.ReactionVessels.BaseClasses.ReactionModels.PartialReactionModel \
      *.... PartialLumpedReactionVessel

    AICE.Fluid.ReactionVessels.BaseClasses.HeatTransfers.PartialVesselHeatTransfer \
      *.... PartialLumpedReactionVessel

    Modelica.Fluid.Interfaces.PartialLumpedVolume <|-- PartialLumpedReactionVessel
  }

  namespace AICE.Fluid.Interfaces {
    class PartialExpandableThermodynamics <<partial>>
  }

  AICE.Fluid.Interfaces.PartialExpandableThermodynamics \
    <|-- AICE.Fluid.ReactionVessels.BaseClasses.HeatTransfers.PartialVesselHeatTransfer
  AICE.Fluid.Interfaces.PartialExpandableThermodynamics \
    <|-- AICE.Fluid.ReactionVessels.BaseClasses.ReactionModels.PartialReactionModel
  AICE.Fluid.Interfaces.PartialExpandableThermodynamics \
    <|-- AICE.Fluid.ReactionVessels.BaseClasses.PartialLumpedReactionVessel
}
```

# new chamber design

```plantuml
@startuml
namespace Modelica.Fluid {

  namespace Modelica.Fluid.Interfaces {
    class PartialHeatTransfer <<partial>> {
      parameter SI.CoefficientOfHeatTransfer k = 0
      parameter SI.Temperature T_ambient = system.T_ambient
    }

    class PartialLumpedVolume <<partial>> {
    }
  }

  namespace Modelica.Fluid.Vessels.BaseClasses {

    class PartialLumpedVessel <<partial>>

    namespace Modelica.Fluid.Vessels.BaseClasses.HeatTransfer {
      class PartialVesselHeatTransfer <<partial>>
      class IdealHeatTransfer
      class ConstantHeatTransfer

      Modelica.Fluid.Interfaces.PartialHeatTransfer <|-- PartialVesselHeatTransfer
      PartialVesselHeatTransfer <|-- IdealHeatTransfer
      PartialVesselHeatTransfer <|-- ConstantHeatTransfer
    }

    Modelica.Fluid.Interfaces.PartialLumpedVolume <|-- PartialLumpedVessel
    Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer \
      *.. PartialLumpedVessel
  }
}

namespace AICE.PowerUnits.Combustions {

  namespace AICE.PowerUnits.Combustions.Interfaces {
    class PartialCylinderComponent <<partial>>
  }

  namespace AICE.PowerUnits.Combustions.Chambers {

    namespace AICE.PowerUnits.Combustions.Chambers.BaseClasses {

      class PartialLumpedChamber <<partial>>

      AICE.PowerUnits.Combustions.Interfaces.PartialCylinderComponent \
        <|-- PartialLumpedChamber

      namespace AICE.PowerUnits.Combustions.Chambers.BaseClasses.ReactionModels {
        class PartialChamberReactionModel <<partial>>

        AICE.PowerUnits.Combustions.Interfaces.PartialCylinderComponent \
          <|-- PartialChamberReactionModel
      }

      namespace AICE.PowerUnits.Combustions.Chambers.BaseClasses.HeatTransfers {
        class PartialChamberHeatTransfer <<partial>>
        class IdealHeatTransfer
        class ConstantHeatTransfer

        Modelica.Fluid.Vessels.BaseClasses.HeatTransfer.PartialVesselHeatTransfer \
          <|-- PartialChamberHeatTransfer
        AICE.PowerUnits.Combustions.Interfaces.PartialCylinderComponent \
          <|-- PartialChamberHeatTransfer

        PartialChamberHeatTransfer <|-- IdealHeatTransfer
        PartialChamberHeatTransfer <|-- ConstantHeatTransfer
      }

      Modelica.Fluid.Interfaces.PartialLumpedVolume <|----- PartialLumpedChamber

      AICE.PowerUnits.Combustions.Chambers.BaseClasses.ReactionModels.PartialChamberReactionModel \
        *.. PartialLumpedChamber

      AICE.PowerUnits.Combustions.Chambers.BaseClasses.HeatTransfers.PartialChamberHeatTransfer \
        *.. PartialLumpedChamber

    }

  }

}
@enduml
```
