# `MultipleDiscreteGear`

複数のギア比を、整数のギア位置の入力に応じて選択し、入力側と出力側の回転数をギア比に応じたものとする、回転運動のモデル。

ニュートラルギアを実装し、動力が伝わらない状態を表現する。

ニュートラルギア以外のギア位置では、与えられたエネルギー効率に基づいて入力トルクと出力トルクの関係を与える。

# 仕様

## クラス名

`AICE.Mechanics.Rotational.Components.MultipleDiscreteGear`

- `Multiple`は、複数のギア位置から選択できることを意味する
- `Discrete`は、ギア位置が整数で与えられることから、ギア比(回転数、トルク、...etc)が不連続な変化となりうることを意味する。

## パッケージ構成

| パッケージ名 | コメント |
| ----------- | -------- |
| `AICE.Mechanics.Rotational.Components` | `MultipleDiscreteGear` を追加するパッケージ |
| `AICE.Mechanics.Rotational.Components.Tests` | 単体テストを設置するパッケージ |
| `AICE.Mechanics.Rotational.Components.BaseClasses` | (基底クラスを追加する必要性のある場合) |
| `AICE.Mechanics.Rotational.Interfaces` | (`Modelica.Rotational.Interfaces` 相当の抽象度の基底クラスを追加する必要性のある場合) |

## 基底クラス

`Modelica.Mechanics.Rotational.Components.LossyGear` の継承関係と同じとする。

![class diagram from "MultipleDiscreteGear.plantuml"
](http://www.plantuml.com/plantuml/svg/bP51ImD138Nlyolc3tGEdakXh5LxM2Ywu3dCfkv09XaoMKHq_xkMYb1OwJQNN9BtlSTp31bhWOY-bHufHGZTW9g1mXE2TGfSZwXXpuPwX8ZLVph_Cv6WLdy0jGIq8yp81lhH2FV9aZ3GwVyWQg_91Xcjy-jrEPzlDhyYavlSd8KMum2SOWtFOd2MMsoXUvS70dx3kkN-PIn5rEwkmxVxPZSFRYGNOMIx1BSZMIg4zwb6HSD71FsJFgt5USBI35-hrIpsadQbm71RQVyo9_SD)

## パラメータ および 追加のコネクタ

| 宣言 | コメント | デフォルト値 |
| ---- | ------- | ----------- |
| `parameter Integer nGearPosition(min=1) annotation(Evaluate=true)` | ギア位置の最大数 例えば6なら、N,1,2,3,4,5,6  | 与えない |
| `parameter Modelica.SIunits.DimensionlessRatio[nGearPosition] ratios_gear` | ギア比の配列 | 与えない |
| `parameter Modelica.SIunits.Efficiency[nGearPosition] etas_gear` | エネルギー効率の配列 | `ones(nGearPosition)` -> ニュートラル以外では損失が生じない |

パラメータ名は、これを変更してもよい。  
その他、必要に応じて宣言してもよい。

### コネクタ
| 宣言 | 因果(causality) | コメント |
| ---- | -------------- | -------- |
| `Modelica.Blocks.Interfaces.IntegerInput gearPosition(min=0, max=nGearPosition)` | input | ギア位置 0はNeutral 1..nGearPosition は上記パラメータに従う。 |
| 継承したコネクタ | acausal |  |

コネクタ名は、これを変更してもよい。  
コネクタはこれですべてとする。

# テスト要件

## エネルギー保存則

`useHeatPort = true`でモデルを実行し、`heatPort`に熱が反映されるようにする。

- `flange_a` へと __入力__ されるエネルギー `power_input`
- `flange_b` から __出力__ されるエネルギー `power_output`
- `heatPort` から __出力__ されるエネルギー `power_loss`

エネルギー保存則は以下の式で表される。

```
power_input = power_output + power_loss
```

この関係式が成り立つことを確認する。

## ギア位置を変更しないテスト

`flange_a` に回転数を可変としたソースを、 `flange_b` に適切な負荷モデル(`Inertia`, `SpringDamper`などを組み合わせる)を接続する。

各テストではギア位置を固定し、変更しない。

### ギア位置 N==0 について

負荷モデルの回転数が、`flange_a` 側のソースの回転数の影響を受けていないことを確認する。

### ギア位置 1..6 について

回転数が`ratios_gear[gearPosition]`に応じたものになっていることを確認する。
エネルギー効率 `- (der(flange_b.phi) * flange_b.tau) / (der(flange_a.phi) * flange_a.tau)` を計算し、`etas_gear[gearPosition]`に応じたものになっていることを確認する。

## ギア位置を変更するテスト

`flange_a`に回転数を固定したソースを、`flange_b`に適切な負荷モデルを接続する。

## N以外からのシフトアップ

`1 -> 2`, `3 -> 4` のように、ギア位置を一つだけ進めたとき、

- ギア位置変更をする前
- ギア位置変更をしたしばらく後

について、ギア位置を変更しないテストのときと同じふるまい(回転数、効率)をすることを確認する。
また、`flange_a.phi`, `flange_b.phi`がC0級連続である(途切れない)ことを確認する。

## N以外へのシフトダウン

`5 -> 4`, `3 -> 2` のように、ギア位置を一つだけ戻したとき、

- ギア位置変更をする前
- ギア位置変更をしたしばらく後

について、ギア位置を変更しないテストのときと同じふるまい(回転数、効率)をすることを確認する。
また、`flange_a.phi`, `flange_b.phi`がC0級連続である(途切れない)ことを確認する。

## Nからのシフトアップ

`N -> 1` とニュートラルからギアを進めたとき

- ギア位置変更をする前
- ギア位置変更をしたしばらく後

について、ギア位置を変更しないテストのときと同じふるまい(回転数、効率)をすることを確認する。
また、`flange_a.phi`, `flange_b.phi`がC0級連続である(途切れない)ことを確認する。

## Nへのシフトダウン

`1 -> N` とニュートラルへギアを戻したとき

- ギア位置変更をする前
- ギア位置変更をしたしばらく後

について、ギア位置を変更しないテストのときと同じふるまい(回転数、効率)をすることを確認する。
また、`flange_a.phi`, `flange_b.phi`がC0級連続である(途切れない)ことを確認する。

[実装について検討すべき事項 実装に不明点などあればリンク先を参照せよ](./MultipleDiscreteGear-Consideration.md)
